; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; $Id: poly.pro,v 1.1 1993/04/02 19:43:31 idl Exp $

FUNCTION POLY,X,C
;+
; NAME:
;	POLY
;
; PURPOSE:
;	Evaluate a polynomial function of a variable.
;
; CATEGORY:
;	C1 - Operations on polynomials.
;
; CALLING SEQUENCE:
;	Result = POLY(X,C)
;
; INPUTS:
;	X:	The variable.  This value can be a scalar, vector or array.
;
;	C:	The vector of polynomial coefficients.  The degree of 
;		of the polynomial is N_ELEMENTS(C) - 1.
;
; OUTPUTS:
;	POLY returns a result equal to:
;		 C(0) + c(1) * X + c(2)*x^2 + ...
;
; COMMON BLOCKS:
;	None.
;
; SIDE EFFECTS:
;	None.
;
; RESTRICTIONS:
;	None.
;
; PROCEDURE:
;	Straightforward.
;
; MODIFICATION HISTORY:
;	DMS, Written, January, 1983.
;-

on_error,2		;Return to caller if an error occurs
N = N_ELEMENTS(C)-1	;Find degree of polynomial
Y = c(n)
for i=n-1,0,-1 do y = y * x + c(i)
return,y
end
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; $Id: xregistered.pro,v 1.2 1993/11/17 19:05:32 ali Exp $

; Copyright (c) 1992-1993, Research Systems, Inc.  All rights reserved.
;	Unauthorized reproduction prohibited.


function XRegistered, NAME, NOSHOW = NOSHOW
;+
; NAME: 
;	XREGISTERED
;
; PURPOSE:
;	This function returns true if the widget named as its argument 
;	is currently registered with the XMANAGER as an exclusive widget, 
;	otherwise this routine returns false.
;
; CATEGORY:
;	Widgets.
;
; CALLING SEQUENCE:
;	Result = XREGISTERED(Name)
;
; INPUTS:
;	Name:	A string containing the name of the widget in question.
;
; KEYWORD PARAMETERS:
;	NOSHOW:	If the widget in question is registered, it is brought
;		to the front of all the other windows by default.  Set this
;		keyword to keep the widget from being brought to the front.
;
; OUTPUTS:
;	If the named widget is registered, XREGISTERED returns the number
;	of instances of that name in the list maintained by XMANAGER.  
;	Otherwise, XREGISTERED returns 0.
;
; COMMON BLOCKS:
;	MANAGED
;
; SIDE EFFECTS:
;	Brings the widget to the front of the desktop if it finds one.
;
; RESTRICTIONS:
;	None.
;
; PROCEDURE:
;	Searches the list of exclusive widget names and if a match is found
;	with the one in question, the return value is modified.
;
; MODIFICATION HISTORY:
;	Written by Steve Richards, November, 1990
;	Jan, 92 - SMR	Fixed a bug where an invalid widget
;			was being referenced with 
;			WIDGET_CONTROL and the /SHOW keyword.
;	17 November 1993 - AB and SMR. Added ID validity checking to
;			fix a bug where already dead widgets were being
;			accessed.
;-

  COMMON MANAGED, ids, names, $
		nummanaged, inuseflag, $
		backroutines, backids, backnumber, nbacks, validbacks, $
		blocksize, $
		cleanups, $
		outermodal

  answer = 0					; Nothing matched yet

  IF(KEYWORD_SET(nummanaged)) THEN begin
    validids = WHERE(ids ne 0, answer)
    if (answer eq 0) then goto, done		; No valid ids
    registered = WHERE(names(validids) EQ NAME, answer)
    if (answer eq 0) then goto, done		; Name not registered
    registered = validids(registered)
  endif else goto, done				; Nothing being managed.

  ; Assertion: answer cannot be 0 at this point
  validids = WIDGET_INFO(ids(registered), /VALID)
  IF((validids(0) EQ 1) AND $
     (NOT(KEYWORD_SET(NOSHOW)))) THEN $
    WIDGET_CONTROL, ids(registered(0)), /SHOW

done:
  RETURN, answer

END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; $Id: cw_bgroup.pro,v 1.4 1993/10/07 22:23:31 doug Exp $

; Copyright (c) 1992-1993, Research Systems, Inc.  All rights reserved.
;	Unauthorized reproduction prohibited.
;+
; NAME:
;	CW_BGROUP
;
; PURPOSE:
;	CW_BGROUP is a compound widget that simplifies creating
;	a base of buttons. It handles the details of creating the
;	proper base (standard, exclusive, or non-exclusive) and filling
;	in the desired buttons. Events for the individual buttons are
;	handled transparently, and a CW_BGROUP event returned. This
;	event can return any one of the following:
;		- The Index of the button within the base.
;		- The widget ID of the button.
;		- The name of the button.
;		- An arbitrary value taken from an array of User values.
;
; CATEGORY:
;	Compound widgets.
;
; CALLING SEQUENCE:
;		Widget = CW_BGROUP(Parent, Names)
;
;	To get or set the value of a CW_BGROUP, use the GET_VALUE and
;	SET_VALUE keywords to WIDGET_CONTROL. The value of a CW_BGROUP
;	is:
;
;		-----------------------------------------------
;		Type		Value
;		-----------------------------------------------
;		normal		None
;		exclusive   	Index of currently set button
;		non-exclusive	Vector indicating the position
;				of each button (1-set, 0-unset)
;		-----------------------------------------------
;
;
; INPUTS:
;       Parent:		The ID of the parent widget.
;	Names:		A string array, containing one string per button,
;			giving the name of each button.
;
; KEYWORD PARAMETERS:
;
;	BUTTON_UVALUE:	An array of user values to be associated with
;			each button and returned in the event structure.
;	COLUMN:		Buttons will be arranged in the number of columns
;			specified by this keyword.
;	EVENT_FUNCT:	The name of an optional user-supplied event function 
;			for buttons. This function is called with the return
;			value structure whenever a button is pressed, and 
;			follows the conventions for user-written event
;			functions.
;	EXCLUSIVE:	Buttons will be placed in an exclusive base, with
;			only one button allowed to be selected at a time.
;	FONT:		The name of the font to be used for the button
;			titles. If this keyword is not specified, the default
;			font is used.
;	FRAME:		Specifies the width of the frame to be drawn around
;			the base.
;	IDS:		A named variable into which the button IDs will be
;			stored, as a longword vector.
;	LABEL_LEFT:	Creates a text label to the left of the buttons.
;	LABEL_TOP:	Creates a text label above the buttons.
;	MAP:		If set, the base will be mapped when the widget
;			is realized (the default).
;	NONEXCLUSIVE:	Buttons will be placed in an non-exclusive base.
;			The buttons will be independent.
;	NO_RELEASE:	If set, button release events will not be returned.
;	RETURN_ID:	If set, the VALUE field of returned events will be
;			the widget ID of the button.
;	RETURN_INDEX:	If set, the VALUE field of returned events will be
;			the zero-based index of the button within the base.
;			THIS IS THE DEFAULT.
;	RETURN_NAME:	If set, the VALUE field of returned events will be
;			the name of the button within the base.
;	ROW:		Buttons will be arranged in the number of rows
;			specified by this keyword.
;	SCROLL:		If set, the base will include scroll bars to allow
;			viewing a large base through a smaller viewport.
;	SET_VALUE:	The initial value of the buttons. This is equivalent
;			to the later statement:
;
;			WIDGET_CONTROL, widget, set_value=value
;
;	SPACE:		The space, in pixels, to be left around the edges
;			of a row or column major base. This keyword is
;			ignored if EXCLUSIVE or NONEXCLUSIVE are specified.
;	UVALUE:		The user value to be associated with the widget.
;	XOFFSET:	The X offset of the widget relative to its parent.
;	XPAD:		The horizontal space, in pixels, between children
;			of a row or column major base. Ignored if EXCLUSIVE
;			or NONEXCLUSIVE are specified.
;	XSIZE:		The width of the base. 
;	X_SCROLL_SIZE:	The width of the viewport if SCROLL is specified.
;	YOFFSET:	The Y offset of the widget relative to its parent.
;	YPAD:		The vertical space, in pixels, between children of
;			a row or column major base. Ignored if EXCLUSIVE
;			or NONEXCLUSIVE are specified.
;	YSIZE:		The height of the base. 
;	Y_SCROLL_SIZE:	The height of the viewport if SCROLL is specified.
;
; OUTPUTS:
;       The ID of the created widget is returned.
;
; SIDE EFFECTS:
;	This widget generates event structures with the following definition:
;
;		event = { ID:0L, TOP:0L, HANDLER:0L, SELECT:0, VALUE:0 }
;
;	The SELECT field is passed through from the button event. VALUE is
;	either the INDEX, ID, NAME, or BUTTON_UVALUE of the button,
;	depending on how the widget was created.
;
; RESTRICTIONS:
;	Only buttons with textual names are handled by this widget.
;	Bitmaps are not understood.
;
; MODIFICATION HISTORY:
;	15 June 1992, AB
;	7 April 1993, AB, Removed state caching.
;-


pro CW_BGROUP_SETV, id, value

  ON_ERROR, 2						;return to caller

  stash = WIDGET_INFO(id, /CHILD)
  WIDGET_CONTROL, stash, GET_UVALUE=state, /NO_COPY

  case state.type of
    0: message,'unable to set plain button group value'
    1: begin
	  WIDGET_CONTROL, SET_BUTTON=0, state.ids(state.excl_pos)
	  state.excl_pos = value
	  WIDGET_CONTROL, /SET_BUTTON, state.ids(value)
	end
    2: begin
	  n = n_elements(value)-1
	  for i = 0, n do begin
	    state.nonexcl_curpos(i) = value(i)
	    WIDGET_CONTROL, state.ids(i), SET_BUTTON=value(i)
	  endfor
	end
  endcase

  WIDGET_CONTROL, stash, SET_UVALUE=state, /NO_COPY
end



function CW_BGROUP_GETV, id, value

  ON_ERROR, 2						;return to caller

  stash = WIDGET_INFO(id, /CHILD)
  WIDGET_CONTROL, stash, GET_UVALUE=state, /NO_COPY

  case state.type of
    0: message,'unable to get plain button group value'
    1: ret = state.excl_pos
    2: ret = state.nonexcl_curpos
  endcase

  WIDGET_CONTROL, stash, SET_UVALUE=state, /NO_COPY

  return, ret

end



function CW_BGROUP_EVENT, ev
  base = ev.handler
  stash = WIDGET_INFO(base, /CHILD)
  WIDGET_CONTROL, stash, GET_UVALUE=state, /NO_COPY
  WIDGET_CONTROL, ev.id, get_uvalue=uvalue

  ret = 1			;Assume we return a struct
  case state.type of
    0:
    1: if (ev.select eq 1) then begin
	 if (state.is_motif and (uvalue ne state.excl_pos)) then $
	    WIDGET_CONTROL, SET_BUTTON=0, state.ids(state.excl_pos)
	  state.excl_pos = uvalue
	ENDIF else begin
	  if (state.no_release ne 0) then ret = 0
	  if (uvalue eq state.excl_pos) then begin
	    ; Don't allow them to unset the current button
	    ret = 0
            if (state.is_motif) then WIDGET_CONTROL, /SET_BUTTON, ev.id
	  endif
	ENDELSE
    2: begin
	  ; Keep track of the current state
	  state.nonexcl_curpos(uvalue) = ev.select
          if (state.no_release ne 0) then ret = 0
	end
  endcase

  if ret then begin		;Return a struct?
      ret = { ID:base, TOP:ev.top, HANDLER:0L, SELECT:ev.select, $
	       VALUE:state.ret_arr(uvalue) }
      efun = state.efun
      WIDGET_CONTROL, stash, SET_UVALUE=state, /NO_COPY
      if efun ne '' then return, CALL_FUNCTION(efun, ret) $
      else return, ret
  endif else begin		;Trash the event
      WIDGET_CONTROL, stash, SET_UVALUE=state, /NO_COPY
      return, 0
  endelse
end


function CW_BGROUP, parent, names, $
	BUTTON_UVALUE = button_uvalue, COLUMN=column, EVENT_FUNCT = efun, $
	EXCLUSIVE=excl, FONT=font, FRAME=frame, IDS=ids, LABEL_TOP=label_top, $
	LABEL_LEFT=label_left, MAP=map, $
	NONEXCLUSIVE=nonexcl, NO_RELEASE=no_release, RETURN_ID=return_id, $
	RETURN_INDEX=return_index, RETURN_NAME=return_name, $
	ROW=row, SCROLL=scroll, SET_VALUE=sval, SPACE=space, UVALUE=uvalue, $
	XOFFSET=xoffset, XPAD=xpad, XSIZE=xsize, X_SCROLL_SIZE=x_scroll_size,$
	YOFFSET=yoffset, YPAD=ypad, YSIZE=ysize, Y_SCROLL_SIZE=y_scroll_size


  IF (N_PARAMS() ne 2) THEN MESSAGE, 'Incorrect number of arguments'

  ON_ERROR, 2						;return to caller

  ; Set default values for the keywords
  version = WIDGET_INFO(/version)
  if (version.toolkit eq 'OLIT') then def_space_pad = 4 else def_space_pad = 3
  IF (N_ELEMENTS(column) eq 0) 		then column = 0
  IF (N_ELEMENTS(excl) eq 0) 		then excl = 0
  IF (N_ELEMENTS(frame) eq 0)		then frame = 0
  IF (N_ELEMENTS(map) eq 0)		then map=1
  IF (N_ELEMENTS(nonexcl) eq 0)		then nonexcl = 0
  IF (N_ELEMENTS(no_release) eq 0)	then no_release = 0
  IF (N_ELEMENTS(row) eq 0)		then row = 0
  IF (N_ELEMENTS(scroll) eq 0)		then scroll = 0
  IF (N_ELEMENTS(space) eq 0)		then space = def_space_pad
  IF (N_ELEMENTS(uvalue) eq 0)		then uvalue = 0
  IF (N_ELEMENTS(xoffset) eq 0)		then xoffset=0
  IF (N_ELEMENTS(xpad) eq 0)		then xpad = def_space_pad
  IF (N_ELEMENTS(xsize) eq 0)		then xsize = 0
  IF (N_ELEMENTS(x_scroll_size) eq 0)	then x_scroll_size = 0
  IF (N_ELEMENTS(yoffset) eq 0)		then yoffset=0
  IF (N_ELEMENTS(ypad) eq 0)		then ypad = def_space_pad
  IF (N_ELEMENTS(ysize) eq 0)		then ysize = 0
  IF (N_ELEMENTS(y_scroll_size) eq 0)	then y_scroll_size = 0
  



  top_base = 0L
  if (n_elements(label_top) ne 0) then begin
    next_base = WIDGET_BASE(parent, XOFFSET=xoffset, YOFFSET=yoffset, /COLUMN)
    junk = WIDGET_LABEL(next_base, value=label_top)
    top_base = next_base
  endif else next_base = parent
  if (n_elements(label_left) ne 0) then begin
    next_base = WIDGET_BASE(next_base, XOFFSET=xoffset, YOFFSET=yoffset, /ROW)
    junk = WIDGET_LABEL(next_base, value=label_left)
    if (top_base eq 0L) then top_base = next_base
  endif
  ; We need some kind of outer base to hold the users UVALUE
  if (top_base eq 0L) then begin
    top_base = WIDGET_BASE(parent, XOFFSET=xoffset, YOFFSET=yoffset)
    next_base = top_base
  endif

  ; Set top level base attributes
  WIDGET_CONTROL, top_base, MAP=map, EVENT_FUNC='CW_BGROUP_EVENT', $
	FUNC_GET_VALUE='CW_BGROUP_GETV', PRO_SET_VALUE='CW_BGROUP_SETV', $
	SET_UVALUE=uvalue

  ; The actual button holding base
  base = WIDGET_BASE(next_base, COLUMN=column, EXCLUSIVE=excl, FRAME=frame, $
	NONEXCLUSIVE=nonexcl, ROW=row, SCROLL=scroll, SPACE=space, $
	XPAD=xpad, XSIZE=xsize, X_SCROLL_SIZE=x_scroll_size, $
	YPAD=ypad, YSIZE=ysize, Y_SCROLL_SIZE=y_scroll_size)

  n = n_elements(names)
  ids = lonarr(n)
  for i = 0, n-1 do begin
    if (n_elements(font) eq 0) then begin
      ids(i) = WIDGET_BUTTON(base, value=names(i), UVALUE=i)
    endif else begin
      ids(i) = WIDGET_BUTTON(base, value=names(i), FONT=font, UVALUE=i)
    endelse
  endfor

  ; Keep the state info in the real (inner) base UVALUE.
  ; Pick an event value type:
  ;	0 - Return ID
  ;	1 - Return INDEX
  ;	2 - Return NAME
  ret_type = 1
  if KEYWORD_SET(RETURN_ID) then ret_type = 0
  if KEYWORD_SET(RETURN_NAME) then ret_type = 2
  if KEYWORD_SET(BUTTON_UVALUE) then ret_type = 3
    case ret_type of
      0: ret_arr = ids
      1: ret_arr = indgen(n)
      2: ret_arr = names
      3: ret_arr = button_uvalue
    endcase
  type = 0
  if (excl ne 0) then type = 1
  if (nonexcl ne 0) then type = 2
  is_motif = WIDGET_INFO(/VERSION)
  is_motif = is_motif.style eq 'Motif'
  if n_elements(efun) le 0 then efun = ''
  state = { type:type, $	; 0-Standard, 1-Exclusive, 2-Non-exclusive
	    ret_arr:ret_arr, $	; Vector of event values
	    efun : efun, $	; Name of event fcn
	    nonexcl_curpos:intarr(n), $	; If non-exclus, tracks state
	    excl_pos:0, $			; If exclusive, current button
	    ids:ids, $			; Ids of buttons
	    no_release:no_release, $	; Non-0 if no rel. events
 	    is_motif:is_motif }		; TRUE if Motif widgets
  WIDGET_CONTROL, WIDGET_INFO(top_base, /CHILD), SET_UVALUE=state, /NO_COPY

  if (n_elements(sval) ne 0) then CW_BGROUP_SETV, top_base, sval

  return, top_base
END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; $Id: filepath.pro,v 1.6 1993/07/22 17:35:36 Moglie Exp $

FUNCTION FILEPATH, FILENAME, ROOT_DIR=root_dir, SUBDIRECTORY=subdir, $
	TERMINAL = TERMINAL, TMP = TMP
;+
; NAME:
;	FILEPATH
;
; PURPOSE:
;	Given the name of a file in the IDL distribution,
;	FILEPATH returns the fully-qualified path to use in
;	opening the file. Operating system dependencies
;	are taken into consideration. This routine is used by RSI to
;	make the User Library portable.
;
; CATEGORY:
;	File Management.
;
; CALLING SEQUENCE:
;	Result = FILEPATH('filename' [, SUBDIRECTORY = subdir]) 
;
; INPUTS:
;    filename:	The lowercase name of the file to be opened. No device
;		or directory information should be included.
;
; KEYWORDS:
;    ROOT_DIR: The name of the directory from which the resulting path
;	should be based. If not present, the value of !DIR is used.
;	This keyword is ignored if TERMINAL or TMP are specified.
;
;    SUBDIRECTORY:	The name of the subdirectory in which the file
;		should be found. If this keyword is omitted, the main 
;		directory is used.  This variable can be either a scalar 
;		string or a string array with the name of each level of 
;		subdirectory depth represented as an element of the array.
;
;		For example, to get a path to the file DETERM in the "userlib"
;		subdirectory to the IDL "lib" subdirectory, enter:
;
;		path = FILEPATH("determ", SUBDIRECTORY = ["lib", "userlib"])
;
;    TERMINAL:	Return the filename of the user's terminal.
;
;	  TMP:	The file is a scratch file.  Return a path to the
;		proper place for temporary files under the current operating
;		system.
;
; OUTPUTS:
;	The fully-qualified file path is returned.  If one of the subdirectory
;	keywords is not specified, the file is assumed to exist in the
;	main distribution directory.
;
; COMMON BLOCKS:
;	None.
;
; RESTRICTIONS:
;	Don't specify more than one of the keywords in a single call.
;
;
; EXAMPLE:
;	To get a path to the file DETERM in the "userlib" subdirectory to the 
;	IDL "lib" subdirectory, enter:
;
;		path = FILEPATH("determ", SUBDIRECTORY = ["lib", "userlib"])
;
;	The variable "path" contains a string that is the fully-qualified file
;	path for the file DETERM.
;
; MODIFICATION HISTORY:
;	December, 1989, AB, RSI (Formalized from original by DMS)
;	October, 1990, SG, RSI (added support for MSDOS)
;	February, 1991, SMR, RSI (added string array support for multi-level
;				  directories)
;	21 April 1993, AB, Added ROOT_DIR keyword.
;-


ON_ERROR,2		                        ;Return to caller if an error 
						;occurs

do_tmp = KEYWORD_SET(TMP)			;get temporary path if existing
path = ''

IF (KEYWORD_SET(TERMINAL)) THEN BEGIN
  CASE STRUPCASE(!VERSION.OS) OF
    'VMS': RETURN,'SYS$OUTPUT:'
    'WINDOWS': RETURN,'con'
    'WIN32'  :  RETURN,'con'
    'MACOS'  : BEGIN
	       PRINT, "No terminal device for the Mac"
               RETURN, ''
	       END
    ELSE: RETURN,'/dev/tty'
  ENDCASE
ENDIF

IF (do_tmp) THEN BEGIN
  CASE STRUPCASE(!VERSION.OS) OF
    'VMS': root_dir = 'SYS$LOGIN:'
    'WINDOWS': root_dir = '\tmp'
    'WIN32': root_dir = '\tmp'
    'MACOS': begin
	root_dir = !DIR
	if (n_params() EQ 0) then filename = "IDL Temp File"
      end
    ELSE: root_dir = '/tmp'
  ENDCASE
ENDIF ELSE BEGIN
  IF (not KEYWORD_SET(ROOT_DIR)) THEN root_dir = !DIR
  IF (KEYWORD_SET(SUBDIR)) THEN BEGIN		;if the SUBDIR keyword is set
    SUBDIR = STRLOWCASE(SUBDIR)			;then include each level in the
    CASE STRUPCASE(!VERSION.OS) OF		;path separated by the correct
      'VMS': divider = "."			;directory level delimiter for
      'WINDOWS': divider = "\"			;the current O.S.
      'WIN32': divider = "\"			;the current O.S.
      'MACOS' : divider = ":"
      ELSE: divider = "/"
    ENDCASE
    FOR i = 0, N_ELEMENTS(SUBDIR) - 1 DO BEGIN
	path = path + SUBDIR(i)
	IF(i NE N_ELEMENTS(SUBDIR) - 1) THEN $
	  path = path + divider
    ENDFOR
    if STRUPCASE(!VERSION.OS) EQ 'MACOS' THEN path = path + divider
  ENDIF
ENDELSE


CASE STRUPCASE(!VERSION.OS) OF
  'VMS': BEGIN
      IF (NOT do_tmp) THEN BEGIN
        IF (path EQ '') THEN path = '000000'
        path = '[' + path + ']'
        IF (root_dir ne !dir) THEN path = ''
      ENDIF
    END
  'WINDOWS': BEGIN
      path = '\' + path
      IF (path NE '\') THEN path = path + '\'
    END
  'WIN32': BEGIN
      path = '\' + path
      IF (path NE '\') THEN path = path + '\'
    END
  'MACOS':
  ELSE: BEGIN
      path = '/' + path
      IF (path NE '/') THEN path = path + '/'
    END
ENDCASE
RETURN, root_dir + path + filename

END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; $Id: xmanager.pro,v 1.6 1993/11/17 19:04:27 ali Exp $

; Copyright (c) 1992-1993, Research Systems, Inc.  All rights reserved.
;	Unauthorized reproduction prohibited.

PRO XUnregister, corpse

	;This procedure removes a dead widget from the Xmanagers common
	;block.  The space taken by the dead widget in the id and name
	;lists is just zeroed.

COMMON MANAGED, ids, names, $
		nummanaged, inuseflag, $
		backroutines, backids, backnumber, nbacks, validbacks, $
		blocksize, $
		cleanups, $
		outermodal

janitor = 0						;check to see if the
victim = WHERE(ids EQ corpse, janitor)			;dying widget has a
IF(janitor NE 0) THEN $					;cleanup routine and
  IF(STRLEN(cleanups(victim(0))) NE 0) THEN $		;call it if so.
    CALL_PROCEDURE, cleanups(victim(0)), corpse

deadindex = WHERE(ids EQ corpse, found)			;find dead index
IF(found NE 0) THEN BEGIN
  IF(KEYWORD_SET(backroutines)) THEN BEGIN		;find living background
    backindex = WHERE(backids EQ corpse, backfound)
    IF(backfound NE 0) THEN BEGIN			;if background tasks
      backroutines(backindex) = ''			;are registered for
      backids(backindex) = 0L				;the corpse, remmove
      backnumber = 0					;them
      nbacks = nbacks - backfound
    ENDIF
    IF(nbacks NE 0) THEN $				;keep a list of which
      validbacks = WHERE(backids NE 0) $		;background slots are
    ELSE validbacks = -1				;valid in the id list
  ENDIF
  ids(deadindex) = 0L					;zero out the dead
  names(deadindex) = ''					;widgets list entries
  cleanups(deadindex) = ''
  nummanaged = nummanaged - found
ENDIF
END		;end of XUnregister procedure

;-----------------------------------------------------------------------------

PRO XManager, NAME, ID, $
		BACKGROUND = BACKGROUND, $
		CLEANUP = CLEANUP, $
		EVENT_HANDLER = EVENT_HANDLER, $
		GROUP_LEADER = GROUP_LEADER, $
		JUST_REG = JUST_REG, $
		MODAL = MODAL
;+
; NAME:
;	XMANAGER
;
; PURPOSE:
;	Provide main event loop and management for widgets created using IDL.
;
; CATEGORY:
;	Widgets.
;
; CALLING SEQUENCE:
;	XMANAGER [, Name, ID]
;
; OPTIONAL INPUTS:
;	NAME:	A string that contains the name of the routine that creates 
;		the widget.
;  
;	ID:	The widget ID of the base of the new widget.
;
; KEYWORD PARAMETERS:
;   BACKGROUND:
;      -------------------------------------------------------------------
;      | PLEASE NOTE: This keyword is OBSOLETE. It's functionality is    |
;      | provided by the TIMER keyword to the WIDGET_CONTROL procedure.  |
;      -------------------------------------------------------------------
;		A string that contains the name of a background task procedure
;		to be called when the event loop is idle.  The background task
;		procedure has only one parameter.  The parameter is the widget
;		ID of the top-level base widget associated with the background
;		task.  The background task should be very short and quick in 
;		execution.  Long, complicated routines will hinder the 
;		smoothness of event processing.  See the "Simple Widget
;		Examples" (available from the XDEMO examples menu or by
;		entering WEXMASTER at the IDL prompt) for an example of a 
;		background task widget.
;
;      CLEANUP:	This keyword contains a string that is the name of the
;		routine called when the widget dies.  If not specified,
;		no routine is called.  The cleanup routine must accept one 
;		parameter which is the widget id of the dying widget.
;
;EVENT_HANDLER:	The name of the routine that is to be called when a
;		widget event occurs in the widget being registered.  If this 
;		keyword is not supplied, the Xmanager will try to process the
;		event by calling the routine name with "_event" appended.  See
;		below for a more detailed explanation.
;
; GROUP_LEADER:	The widget id of the group leader for the
;		widget being processed.  When the leader dies either by the 
;		users actions or some other routine, all widgets that have that
;		leader will also die.
;
;		For example, a widget that views a help file for a demo 
;		widget would have that demo widget as it's leader.  When
;		the help widget is registered, it sets the keyword 
;		GROUP_LEADER to the widget id of the demo widget. If 
;		the demo widget is destroyed, the help widget led by 
;		the it would be killed by the XMANAGER.
;
;     JUST_REG:	This keyword tells the manager to just register the widget
;		but not to start doing the event processing.  This is useful
;		when you want to register a group of related top level widgets
;		before beginning processing.
;
;	 MODAL:	When this keyword is set, the widget that is being registered
;		traps all events and desensitizes all the other widgets.  It
;		is useful when input from the user is necessary to continue.
;		Once the modal widget dies, the others are resensitized and the
;		normal event processing is restored.
;
; OUTPUTS:
;	No explicit outputs.
;
; COMMON BLOCKS:
;	MANAGED:  Common block used for widget maintenance.  This common block
;		is internal and should not be referenced outside the IDL
;		supplied routines.  It is subject to change without notice.
;	XMAN_BCK_OBSOLETE: Used to keep track of BACKGROUND keyword message.
;	
;
; SIDE EFFECTS:
;	XMANAGER takes control of event processing and locks out the IDL 
;	prompt until all widgets have been destroyed.
;
; RESTRICTIONS:
;	Widgets being registered with the Xmanager must provide at least two 
;	routines.  The first routine creates the widget and registers it with
;	the manager and the second routine processes the events that occur 
;	within that widget.  Optionally, a third routine can is used for the
;	processing of background tasks while awaiting input.  An example 
;	widget is suplied below that uses only two routines.  For more 
;	information on using the background tasks, see the XEXAMPLE routine
;	that is found in this same library.  It demonstrates background 
;	routines.  Also, the "Simple Widget Examples", that can be viewed by
;	entering WEXMASTER at the IDL prompt, demonstrate many aspects of
;	widget programming. 
;
;   !!!!! WARNING !!!!!!!!! WARNING !!!!!!!!! WARNING !!!!!!!!! WARNING !!!!!
;
;	Although this is a library routine, it may change in the future in
;	its internal implementation.  For future upgradability, it is best
;	not to modify or even worry about what this routine does internally.
;
; EXAMPLE USE:
;	To create a widget named Example that is just a base widget with a done
;	button using the XMANAGER you would do the following:
;
;
;	;------ first - the event handler routine ------;
;
;     PRO example_event, ev			;this is the routine that 
;						;deals with the events in the 
;						;example widget.
;	
;	WIDGET_CONTROL, ev.id, GET_UVALUE = uv	;the uservalue is retrieved 
;						;from the widget where the 
;						;event occured
;
;	if(uv eq 'DONE') then $			;if the event occured in the
;	  WIDGET_CONTROL, ev.top, /DESTROY	;done button then kill the 
;     END					;widget example
;
;
;	;------ second - the main routine ------;
;
;     PRO example				;this is the main routine
;						;that builds the widget and
;						;registers it with the Xmanager
;	
;	base = WIDGET_BASE(TITLE = 'Example')	;first the base is created
;	
;	done = WIDGET_BUTTON(base, $		;next the done button is 
;			     TITLE = 'DONE', $	;created and it's user value
;			     UVALUE = 'DONE')	;set to "DONE"
;
;	WIDGET_CONTROL, base, /REALIZE		;the widget is realized
;
;	XManager, 'example', base		;finally the example widget
;						;is registered with the 
;						;Xmanager
;     END
;
;	notes:	First the event handler routine is listed.  The handler
;		routine has the same name as the main routine with the 
;		characters "_event" added.  If you would like to use another
;		event handler name, you would need to pass it's name in as
;		a string to the EVENT_HANDLER keyword.  Also notice that the
;		event routine is listed before the main routine.  This is 
;		because the compiler will not compile the event routine if
;		it was below the main routine.  This is only needed if both
;		routines reside in the same file and the file name is the same
;		as the main routine name with the ".pro" extension added.
;
;
; PROCEDURE:
;	When the first widget is registered, initialize the lists and then 
;	start processing events.  Continue registering widgets and dispatching
;	events until all the widgets have been destroyed.  When a widget is 
;	killed, destroy all widgets that list the destroyed widget as their 
;	leader, if any.
;
; RELATED FUNCTIONS AND PROCEDURES:
;	XREGISTERED, XMANAGERTOOL, XBACKREGISTER
;
; MODIFICATION HISTORY: Written by Steve Richards, November, 1990
;	SMR, Mar,  1991	Added a cleanup routine keyword to allow dying
;			widgets to clean themselves up when dying.
;	SMR, May,  1991 Fixed a bug found by Diane Parchomchuk where 
;			an error occured when registering a widget 
;			right after destroying another.
;	SMR & ACY, July, 1991
;			Fixed a bug found by Debra Wolkovitch where
;			lone widgets being destroyed and new ones 
;			created caused problems.
;	SMR, Sept, 1991	Changed cleanup to use the new WIDGET_INFO
;			routine.
;	SMR & ACY, Oct,  1991
;			Fixed a bug where a background event that 
;			unregistered itself after a time would result
;			in an XMANAGER error.
; 	SMR, Mar.  1992	Changed XMANAGER to use enhanced widget functions for
;			event processing.
;	SMR, Nov.  1992 Changed modal widget handling allowing nesting of
;			modal widgets.  The first modal desensitizes all 
;			current widgets and subsequent modals only desensitize
;			the modal that called them.
;	JIY, Apr.  1993 Changed modal widget handling process to not run
;			the event loop for nested modal widgets.
;			Allowed for multiple modal widgets.
;	AB & SMR, 17 November 1993
;			Added ID validity checking to desensitizing of modal
;			widgets to fix a bug where already dead widgets
;			were being accessed.
;-

COMMON MANAGED, ids, names, $
		nummanaged, inuseflag, $
		backroutines, backids, backnumber, nbacks, validbacks, $
		blocksize, $
		cleanups, $
		outermodal

COMMON XMAN_BCK_OBSOLETE, obsolete

IF(NOT(KEYWORD_SET(outermodal))) THEN outermodal = 0	;initialization

doeventloop = 0						;flag set if the main
							;event loop to be used

;------------- New Widget Registration Section ---------

IF(N_PARAMS() EQ 0) THEN BEGIN				;no parameters so just
  IF (NOT(KEYWORD_SET(nummanaged))) THEN RETURN		;start up the xmanager
  validids = WIDGET_INFO(ids(0:nummanaged-1), /VALID)	;loop if widgets 
  badids = WHERE((validids EQ 0), deaduns)		;present
  IF(deaduns NE 0) THEN $
    FOR i = 0, deaduns-1 do $
      XUnRegister, ids(badids(i))			;event loop again
  IF(KEYWORD_SET(ids)) THEN doeventloop = 1
  IF(NOT(KEYWORD_SET(nummanaged))) THEN $
    MESSAGE, 'No Widgets Are Currently being managed'
ENDIF ELSE IF(N_PARAMS() NE 2) THEN BEGIN
  MESSAGE, 'Wrong Number of Arguments, ' + $
	   'Usage = Xmanager, [name, id]'

ENDIF ELSE BEGIN					;register a new widget

  IF (KEYWORD_SET(IDS)) THEN BEGIN			;make sure that the new
    position = WHERE(ID EQ IDS, found)			;widget ID is not
    IF (found NE 0) THEN $				;currently in the IDS
      XUnregister, ID					;common block.
  ENDIF							;This is only
							;true when a widget is
							;killed and another is
							;created before
							;events are processed

  IF(NOT KEYWORD_SET(EVENT_HANDLER)) THEN $		;if no event handler 
		EVENT_HANDLER = NAME + '_event'		;was specified, assume
							;that the widget name +
							;"_event" exists

  IF(KEYWORD_SET(BACKGROUND)) THEN BEGIN
    ; Make sure the window manager will not kill a widget that has a
    ; background in progress
    WIDGET_CONTROL, id, /DELAY_DESTROY
    if not KEYWORD_SET(obsolete) then begin
      obsolete = 1
      message,/INFO,"The BACKGROUND keyword to theXMANAGER procedure is obsolete. It is superceeded by the TIMER keyword to the WIDGET_CONTROL procedure."
    endif
  endif

  WIDGET_CONTROL, id, /MANAGED				; Mark it.

  IF(NOT KEYWORD_SET(CLEANUP)) THEN $			;specify a null cleanup
		CLEANUP = ''				;if keyword not set

  IF(KEYWORD_SET(GROUP_LEADER)) THEN $			;register the group
    WIDGET_CONTROL, ID, GROUP_LEADER = GROUP_LEADER	;leader

  IF(NOT(KEYWORD_SET(nummanaged))) THEN $		;keep track of the
    nummanaged = 1 $					;number of managed
  ELSE $						;widgets
    nummanaged = nummanaged + 1

  IF(NOT(KEYWORD_SET(inuseflag)) AND $			;if just_register is
     NOT(KEYWORD_SET(JUST_REG))) THEN $			;set, don't process
	doeventloop = 1					;events

  IF(N_ELEMENTS(ids) EQ 0) THEN BEGIN			;create the first block
    blocksize = 64					;and set the size of
    ids = LONARR(blocksize)				;the blocks to be used
    names = STRARR(blocksize)				;in the future
    cleanups = STRARR(blocksize)
    freeindex = 0
  ENDIF ELSE BEGIN
    freeindex = WHERE(ids EQ 0, found)			;otherwise find holes
    IF(found EQ 0) THEN BEGIN				;in the current blocks
      ids = [ids, LONARR(blocksize)]			;or allocate a new one
      names = [names, STRARR(blocksize)]		;if all are full
      cleanups = [cleanups, STRARR(blocksize)]
      freeindex = nummanaged - 1
    ENDIF
  ENDELSE

  names(freeindex(0)) = NAME				;put the information
  ids(freeindex(0)) = ID				;into the blocks
  cleanups(freeindex(0)) = CLEANUP

  IF(NOT(KEYWORD_SET(nbacks))) THEN $
    nbacks = 0

  IF(KEYWORD_SET(BACKGROUND)) THEN BEGIN

    nbacks = nbacks + 1

    IF(N_ELEMENTS(backids) EQ 0) THEN BEGIN
      backids = LONARR(blocksize)			;build the first block
      backroutines = STRARR(blocksize)			;for background tasks
      freebacks = 0
      backnumber = 0
    ENDIF ELSE BEGIN					;otherwise, just
      freebacks = WHERE(backids EQ 0, found)		;find free spots or
      IF(found EQ 0) THEN BEGIN				;make new ones if full
        backids = [backids, LONARR(blocksize)]
        backroutines = [backroutines, $
			STRARR(blocksize)]
        freebacks = nbacks - 1
      ENDIF
    ENDELSE

    backids(freebacks(0)) = ID				;add the background
    backroutines(freebacks(0)) = STRUPCASE(BACKGROUND)	;in the first free spot

    validbacks = WHERE(backids NE 0)

    WIDGET_CONTROL, /EVENT_BREAK			;since an xmanager
							;loop could have called
							;this routine that is
							;not processing bcks, 
							;break out so this one
  ENDIF ;background set					;gets processed.

  WIDGET_CONTROL, ID, KILL_NOTIFY = 'XUnregister', $	;set the new widget's
		      EVENT_PRO = EVENT_HANDLER		;callbacks

  ; new non-modal widget created from a modal widget
  ; should be modal as well!
  if (keyword_set(outermodal) and (not keyword_set(modal))) then add_modal = 1;

;----------------- Modal Section -----------------------

  IF (KEYWORD_SET(MODAL) or keyword_set(add_modal))THEN BEGIN
							;when a modal widget
							;is called, all widgets
    if not keyword_set (add_modal) then begin
      IF (KEYWORD_SET(outermodal)) THEN BEGIN		;currently sensitive
        IF ((where(outermodal eq ID))(0) eq -1) THEN BEGIN
							;need to be desenitized
          senslist = outermodal 			;making sure that a
          numsens  = n_elements(senslist)		;widget ID is not being
        ENDIF ELSE numsens = 0				;reused.
        doeventloop = 0
      ENDIF ELSE BEGIN					;nested calls to modal
        desens = where(ids NE ID, found)		;widgets are handled by
        IF (found NE 0) THEN BEGIN			;the stack so each 
	  senslist = ids(desens)			;new modal only
	  numsens = N_ELEMENTS(senslist)		;desensitizes the modal
        ENDIF ELSE numsens = 0				;below it.
      ENDELSE
    end else begin
      numsens     = 0
      doeventloop = 0
    endelse

    oldouter = outermodal				;for more than one
    if (keyword_set(add_modal)) then begin		;modal widget, add it
      leng = n_elements (outermodal)			;to modal widget list.
      temp = lonarr(leng+1)
      temp(0) = outermodal
      temp(leng) = id
      outermodal = temp
    end else $
    outermodal = ID

    IF (numsens NE 0) THEN $	
      FOR i = 0, numsens - 1 DO $			;desensitize all
        IF(WIDGET_INFO(senslist(i),/VALID) EQ 1) THEN $ ;widgets active when
	  WIDGET_CONTROL, senslist(i), SENSITIVE = 0	;this modal is called.
							;either all or the 
							;modal that called me.

    bad = 0
    while (not bad) do begin				;loop as long as 
      junk = widget_event (outermodal, bad_id = bad)	;modal list is alive.
      if ((where(outermodal eq bad))(0) eq -1) then bad = 0 else begin
        good = where(outermodal ne bad)
        if (good(0) eq -1) then bad = 1 else begin
          outermodal = outermodal (good)
          bad = 0;
        endelse
      endelse
    endwhile

    IF (numsens NE 0) THEN $
      FOR i = 0, numsens - 1 DO $
        IF(WIDGET_INFO(senslist(i),/VALID) EQ 1) THEN $	;now resensitize only
	  WIDGET_CONTROL, senslist(i), /SENSITIVE	;the widgets made 
      							;insensitive by this
							;modal request

    outermodal = oldouter				;restore my caller
							;as the outer modal
  ENDIF ;modal
ENDELSE ;xmanager called with 2 params

;----------------- Main Event Loop ---------------------

IF(doeventloop NE 0) THEN BEGIN				;if not modal and the
  WHILE(KEYWORD_SET(nummanaged)) DO BEGIN		;event loop should be
    inuseflag = 1					;executed, do it!
    active = WIDGET_INFO(/ACTIVE)
    WHILE (active NE 0) DO BEGIN
      IF(nbacks NE 0) THEN BEGIN			;Non-blocking BG task
        WHILE(nbacks NE 0) DO BEGIN
	  IF(backnumber GE nbacks) THEN backnumber = 0;
	  CALL_PROCEDURE, backroutines(validbacks(backnumber)), $
			  backids(validbacks(backnumber))
	  backnumber = backnumber + 1
	  newevent = WIDGET_EVENT(/NOWAIT)
        ENDWHILE
      ENDIF ELSE $					;Blocking mode
        newevent = WIDGET_EVENT()
      active = WIDGET_INFO(/ACTIVE)
    ENDWHILE
    nummanaged = 0
  ENDWHILE
  inuseflag = 0						;done so unset the 
ENDIF ;doeventloop NE 0					;inuse flag

END ;xmanager.pro
;----------------------- end of procedure XManager ----------------------------
