; R.S. Mallozzi, UAH/NASA, 1996 September

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;+
; NAME:
;     GET_PID
;
; PURPOSE:
;     Returns the current process ID; useful for generating unique filenames
;
; CALLING SEQUENCE:
;     PID = GET_PID()
;
; INPUTS:
;     NONE
; 
; KEYWORD PARAMETERS:
;     NONE
;
; OUTPUTS:
;     PID: The current process ID
;
; COMMON BLOCKS:
;     NONE
;
; SIDE EFFECTS:
;     NONE
;
; RESTRICTIONS:
;     VMS and some UNIX OS only
;
; DEPENDENCIES:
;     NONE
;
; MODIFICATION HISTORY:
;	Written: mallozzi@gibson.msfc.nasa.gov, July, 1996
;-                                           

FUNCTION GET_PID 


IF (!VERSION.RELEASE GE 4.0) THEN BEGIN
   OS_TYPE = STRTRIM(STRUPCASE(!VERSION.OS_FAMILY), 2)
ENDIF ELSE BEGIN
   OS = STRTRIM(STRUPCASE(!VERSION.OS), 2)
   CASE OS OF
        'VMS':   OS_TYPE = 'VMS'
        'IRIX':  OS_TYPE = 'UNIX'
        'LINUX': OS_TYPE = 'UNIX'
        'SUNOS': OS_TYPE = 'UNIX'
        ELSE: BEGIN
              PRINT, 'ERROR: GET_PID: Operating system unsupported'
              RETURN, -1
              END
   ENDCASE
ENDELSE

CASE OS_TYPE OF
     'VMS':  SPAWN, 'WRITE SYS$OUTPUT F$GETJPI("", "PID")', PID
     'UNIX': SPAWN, /SH, 'echo $$', PID
ENDCASE
            
PID = PID(0)
RETURN, PID



END 
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *



; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;+
; NAME:
;      W_BFITS
;
; PURPOSE:
;       Widget interface to BFITS
;
; CATEGORY:
;       BATSE Software
;
; CALLING SEQUENCE:
;       W_BFITS
;
; INPUTS:
;       NONE
; 
; KEYWORD PARAMETERS:
;       NONE
;
; OUTPUTS:
;       NONE
;
; COMMON BLOCKS:
;       COMMON W_BFITS_DATA, _WIDS, _HIDS
;
; SIDE EFFECTS:
;       NONE
;
; RESTRICTIONS:
;	Requires IDL widgets
;
; DEPENDENCIES:
;       BFITS fortran executable (compiled with no TAE support)
;       GET_PID.PRO
;
; MODIFICATION HISTORY:
;	v0.9: Written: mallozzi@gibson.msfc.nasa.gov,  1996 September
;-

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; This is the event handler 
PRO W_BFITS_EVENT, EVENT

COMMON W_BFITS_DATA, _WIDS, _HIDS


; Get current available datatypes
HANDLE_VALUE, _HIDS.AVAIL_DATATYPES, CURRENT_AVAIL_DATATYPES

TYPE = TAG_NAMES(EVENT, /STRUCTURE)
CASE TYPE OF
 
 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ; The button widget events
 'WIDGET_BUTTON': BEGIN
    WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE
                   
    VALUE = STRUPCASE(VALUE)
    CASE 1 OF 

         VALUE EQ 'QUIT': BEGIN
               WIDGET_CONTROL, EVENT.TOP, /DESTROY
               END

         VALUE EQ 'HELP': BEGIN
               XDISPLAYFILE, STRLOWCASE('W_BFITS.HLP')
               END

         ; Pulldown menu for OBJECT
         VALUE EQ 'GRB': BEGIN
               WIDGET_CONTROL, _WIDS.OBJECT_LABEL, SET_VALUE = VALUE(0)
               END
         VALUE EQ 'SFL': BEGIN
               WIDGET_CONTROL, _WIDS.OBJECT_LABEL, SET_VALUE = VALUE(0)
               END
         VALUE EQ 'TGF': BEGIN
               WIDGET_CONTROL, _WIDS.OBJECT_LABEL, SET_VALUE = VALUE(0)
               END
         VALUE EQ 'SGR': BEGIN
               WIDGET_CONTROL, _WIDS.OBJECT_LABEL, SET_VALUE = VALUE(0)
               END
         VALUE EQ 'CYG': BEGIN
               WIDGET_CONTROL, _WIDS.OBJECT_LABEL, SET_VALUE = VALUE(0)
               END
         VALUE EQ 'GCB': BEGIN
               WIDGET_CONTROL, _WIDS.OBJECT_LABEL, SET_VALUE = VALUE(0)
               END
         VALUE EQ 'UNK': BEGIN
               WIDGET_CONTROL, _WIDS.OBJECT_LABEL, SET_VALUE = VALUE(0)
               END

         VALUE EQ 'CHANGE DATATYPE': BEGIN
               WIDGET_CONTROL, EVENT.TOP, /DESTROY
               W_BFITS
               END

         VALUE EQ 'SAVE': BEGIN
               OPENW, FL_OUT, 'SYS$LOGIN:W_BFITS.SAV', /GET_LUN

               PRINTF, FL_OUT, _HIDS.SUBCOMMAND

               WIDGET_CONTROL, _WIDS.TRIGGER_NUMBER, GET_VALUE = TRIGGER_NUM
               PRINTF, FL_OUT, TRIGGER_NUM

               WIDGET_CONTROL, _WIDS.OBJECT_LABEL, GET_VALUE = OBJECT_CLASS
               PRINTF, FL_OUT, OBJECT_CLASS

               WIDGET_CONTROL, _WIDS.IBDB_PATH, GET_VALUE = IBDBPATH
               PRINTF, FL_OUT, IBDBPATH

               WIDGET_CONTROL, _WIDS.FITS_FILENAME, GET_VALUE = FITSFILE
               PRINTF, FL_OUT, FITSFILE

               WIDGET_CONTROL, _WIDS.TIME_RANGE_LO, GET_VALUE = TRL
               WIDGET_CONTROL, _WIDS.TIME_RANGE_HI, GET_VALUE = TRH
               PRINTF, FL_OUT, TRL, TRH

               WIDGET_CONTROL, _WIDS.TIME_RES, GET_VALUE = MIN_TIMERES
               PRINTF, FL_OUT, MIN_TIMERES

               WIDGET_CONTROL, _WIDS.DETECTORS, GET_VALUE = DET_INDEX
               PRINTF, FL_OUT, DET_INDEX

               WIDGET_CONTROL, _WIDS.BRIGHT_DETECTORS, GET_VALUE = DET_INDEX
               PRINTF, FL_OUT, DET_INDEX

               WIDGET_CONTROL, _WIDS.DATATYPES, GET_VALUE = DATATYPES
               PRINTF, FL_OUT, DATATYPES

               WIDGET_CONTROL, _WIDS.CHANNEL_LO, GET_VALUE = CHL
               WIDGET_CONTROL, _WIDS.CHANNEL_HI, GET_VALUE = CHH
               PRINTF, FL_OUT, CHL, CHH

               IF (_HIDS.SUBCOMMAND EQ 'DISC') THEN BEGIN
                  WIDGET_CONTROL, _WIDS.CORTIME, GET_VALUE = TEMP
                  PRINTF, FL_OUT, TEMP
               ENDIF

               WIDGET_CONTROL, _WIDS.DEADTIME, GET_VALUE = TEMP
               PRINTF, FL_OUT, TEMP

               WIDGET_CONTROL, _WIDS.CALIBRATE, GET_VALUE = TEMP
               PRINTF, FL_OUT, TEMP

               WIDGET_CONTROL, _WIDS.SCRATCH_DIR, GET_VALUE = SCRATCH_DIR
               PRINTF, FL_OUT, SCRATCH_DIR

               WIDGET_CONTROL, _WIDS.QMASK, GET_VALUE = QMASK
               PRINTF, FL_OUT, QMASK

               FREE_LUN, FL_OUT

               WIDGET_CONTROL, _WIDS.STATUS, SET_VALUE = 'Saved ' + $
                               'SYS$LOGIN:W_BFITS.SAV'
               END

         VALUE EQ 'RESTORE': BEGIN
               WIDGET_CONTROL, EVENT.TOP, /DESTROY
               OPENR, FL_IN, 'SYS$LOGIN:W_BFITS.SAV', /GET_LUN
               SUBCMD = ' '
               READF, FL_IN, SUBCMD
               FREE_LUN, FL_IN
               W_BFITS, SUBCMD
               END

         VALUE EQ 'CREATE FITS FILE': BEGIN

               WIDGET_CONTROL, _WIDS.STATUS, SET_VALUE = 'Writing NML File.'
               SUBCOMMAND = _HIDS.SUBCOMMAND

               WIDGET_CONTROL, _WIDS.TRIGGER_NUMBER, GET_VALUE = TRIGGER_NUM
               IF (STRCOMPRESS(TRIGGER_NUM(0), /REMOVE_ALL) EQ '') THEN BEGIN
                  ERR = WIDMENU(['Please Enter Trigger Number', $
                                 'Ok'], TITLE = 0)
                  RETURN
               ENDIF 
               TRIGGER_NUM = FIX(TRIGGER_NUM)

               WIDGET_CONTROL, _WIDS.OBJECT_LABEL, GET_VALUE = OBJECT_CLASS

               WIDGET_CONTROL, _WIDS.IBDB_PATH, GET_VALUE = IBDBPATH
               IF (IBDBPATH(0) EQ '') THEN BEGIN
                  ERR = WIDMENU(['Please Enter IBDB Path', $
                                 'Ok'], TITLE = 0)
                  RETURN
               ENDIF 

               WIDGET_CONTROL, _WIDS.FITS_FILENAME, GET_VALUE = FITSFILE
               IF (FITSFILE(0) EQ '') THEN BEGIN
                  ERR = WIDMENU(['Please Enter FITS Filename', $
                                 'Ok'], TITLE = 0)
                  RETURN
               ENDIF 

               WIDGET_CONTROL, _WIDS.TIME_RANGE_LO, GET_VALUE = TRL
               WIDGET_CONTROL, _WIDS.TIME_RANGE_HI, GET_VALUE = TRH
               IF ((TRL(0) EQ '') OR (TRL(0) EQ '')) THEN BEGIN
                  ERR = WIDMENU(['Please Enter Time Range', $
                                 'Ok'], TITLE = 0)
                  RETURN
               ENDIF 
               TIMERANGE = [FLOAT(TRL), FLOAT(TRH)]

               WIDGET_CONTROL, _WIDS.TIME_RES, GET_VALUE = MIN_TIMERES
               IF (MIN_TIMERES(0) EQ '') THEN BEGIN
                  ERR = WIDMENU(['Please Enter Minimum Time Resolution', $
                                 'Ok'], TITLE = 0)
                  RETURN
               ENDIF 
               MIN_TIMERES = FLOAT(MIN_TIMERES)

               WIDGET_CONTROL, _WIDS.DETECTORS, GET_VALUE = DET_INDEX
               DET_SEL = WHERE(DET_INDEX, CNT)
               IF (CNT NE 0) THEN BEGIN
                  DETS = STRING(DET_SEL)
               ENDIF ELSE BEGIN
                  WIDGET_CONTROL, _WIDS.BRIGHT_DETECTORS, GET_VALUE = DET_INDEX
                  DET_ARR = '-1'
                  IF (DET_INDEX(0) EQ 1) THEN DET_ARR = [DET_ARR, 'A']
                  IF (DET_INDEX(1) EQ 1) THEN DET_ARR = [DET_ARR, 'B']
                  IF (DET_INDEX(2) EQ 1) THEN DET_ARR = [DET_ARR, 'C']
                  IF (DET_INDEX(3) EQ 1) THEN DET_ARR = [DET_ARR, 'D']
                  DETS = DET_ARR(1:*)
               ENDELSE

               WIDGET_CONTROL, _WIDS.DATATYPES, GET_VALUE = DATATYPES
               INDEX = WHERE(DATATYPES, CNT)
               IF (CNT EQ 0) THEN BEGIN
                  ERR = WIDMENU(['Please Select a Datatype', $
                                 'Ok'], TITLE = 0)
                  RETURN
               ENDIF 
               DATATYPES = CURRENT_AVAIL_DATATYPES(INDEX)

               WIDGET_CONTROL, _WIDS.CHANNEL_LO, GET_VALUE = CHL
               WIDGET_CONTROL, _WIDS.CHANNEL_HI, GET_VALUE = CHH
               IF ((CHL(0) EQ '') OR (CHH(0) EQ '')) THEN BEGIN
                  ERR = WIDMENU(['Please Enter Channel Range', $
                                 'Ok'], TITLE = 0)
                  RETURN
               ENDIF 
               CHANNELS = [FIX(CHL), FIX(CHH)]

               IF (_HIDS.SUBCOMMAND EQ 'DISC') THEN BEGIN
                  WIDGET_CONTROL, _WIDS.CORTIME, GET_VALUE = TEMP
                  IF (TEMP(0) EQ 0) THEN CORTIME = 'N' ELSE CORTIME = 'Y'
               ENDIF ELSE BEGIN
                  CORTIME = 'N'
               ENDELSE

               WIDGET_CONTROL, _WIDS.DEADTIME, GET_VALUE = TEMP
               IF (TEMP(0) EQ 0) THEN DEADTIME = 'N' ELSE DEADTIME = 'Y'

               WIDGET_CONTROL, _WIDS.CALIBRATE, GET_VALUE = TEMP
               IF (TEMP(0) EQ 0) THEN CALIBRATE = 'N' ELSE CALIBRATE = 'Y'

               WIDGET_CONTROL, _WIDS.SCRATCH_DIR, GET_VALUE = SCRATCH_DIR
               IF (STRCOMPRESS(SCRATCH_DIR(0), /REMOVE_ALL) EQ '') THEN BEGIN
                  ERR = WIDMENU(['Please Enter a Scratch Directory', $
                                 'Ok'], TITLE = 0)
                  RETURN
               ENDIF 

               WIDGET_CONTROL, _WIDS.QMASK, GET_VALUE = QMASK
               IF (STRCOMPRESS(QMASK(0), /REMOVE_ALL) EQ '') THEN BEGIN
                  ERR = WIDMENU(['Please Enter Quality Mask', $
                                 'Ok'], TITLE = 0)
                  RETURN
               ENDIF 

               ; All data retrieved; write the BFITS namelist file, then
               ; execute BFITS
               PID = GET_PID()
               OUTFILE = STRLOWCASE( SCRATCH_DIR(0) + 'BFITS_' + $
                                     STRTRIM(PID, 2) + '.NML' )
               WIDGET_CONTROL, _WIDS.STATUS, SET_VALUE = 'Writing ' + OUTFILE
               OPENW, FL, OUTFILE, /GET_LUN

                      PRINTF, FL, " &BFITS_NAMELIST"
                      PRINTF, FL, " "
                      PRINTF, FL, " RNML_SUBCOMMAND = '" + SUBCOMMAND + "'"
                      PRINTF, FL, " RNML_IBDBPATH = '" + IBDBPATH + "'"
                      PRINTF, FL, " RNML_NUM_DATATYPES = " + $
                                    STRTRIM(N_ELEMENTS(DATATYPES), 2)

                      FOR I=0, N_ELEMENTS(DATATYPES)-1 DO BEGIN
                          PRINTF, FL, " RNML_DATATYPES(" + STRTRIM(I + 1, 2) + $
                                      ") = '" + STRTRIM(DATATYPES(I), 2) + "'"
                      ENDFOR

                      PRINTF, FL, " RNML_QMASK = '" + QMASK + "'"
                      PRINTF, FL, " RNML_FITSFILE = '" + FITSFILE + "'"
                      PRINTF, FL, " RNML_USE_DEADTIME = '" + DEADTIME + "'"
                      PRINTF, FL, " RNML_BURST_NUM = " + STRTRIM(TRIGGER_NUM, 2)
                      PRINTF, FL, " RNML_NUM_DETS = " + $
                                    STRTRIM(N_ELEMENTS(DETS), 2)

                      FOR I=0, N_ELEMENTS(DETS)-1 DO BEGIN
                          PRINTF, FL, " RNML_DETS(" + STRTRIM(I + 1, 2) + $
                                      ") = '" + STRTRIM(DETS(I), 2) + "'"
                      ENDFOR

                      FOR I=0, N_ELEMENTS(CHANNELS)-1 DO BEGIN
                          PRINTF, FL, " RNML_CHANNELS(" + STRTRIM(I + 1, 2) + $
                                      ") = " + STRTRIM(CHANNELS(I), 2)
                      ENDFOR

                      PRINTF, FL, " RNML_MIN_TIMERES = " + $
                              STRTRIM(MIN_TIMERES, 2)

                      FOR I=0, N_ELEMENTS(TIMERANGE)-1 DO BEGIN
                          PRINTF, FL, " RNML_TIMERANGE(" + STRTRIM(I + 1, 2) + $
                                      ") = " + STRTRIM(TIMERANGE(I), 2)
                      ENDFOR

                      PRINTF, FL, " RNML_USE_CORRECT_TIME = '" + CORTIME + "'"
                      PRINTF, FL, " RNML_USE_CALIBRATION = '" + CALIBRATE + "'"
                      PRINTF, FL, " RNML_OBJECT_CLASS = '" + OBJECT_CLASS + "'"
                      PRINTF, FL, " "
                      PRINTF, FL, " &END"

               FREE_LUN, FL
 
               ; That's it, call BFITS
               WIDGET_CONTROL, _WIDS.STATUS, SET_VALUE = 'Writing ' + FITSFILE
               SETLOG, 'BFITS$NAMELIST', OUTFILE
               SPAWN, 'RUN ' + _HIDS.BFITS_APPLICATION
               N = WIDMENU(['Wrote ' + FITSFILE, 'OK'], TITLE = 0)
               WIDGET_CONTROL, _WIDS.STATUS, SET_VALUE = 'Wrote ' + FITSFILE
               END

         ELSE:

    ENDCASE ; for VALUE
    END

 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ; The radio button widget events
 '': BEGIN
    WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE, GET_UVALUE = UVALUE

    CASE UVALUE OF 

         'DETECTOR_BUTTONS': BEGIN
              WIDGET_CONTROL, _WIDS.BRIGHT_DETECTORS, SET_VALUE = [0, 0, 0, 0]
              END
         'BRIGHT_DETECTOR_BUTTONS': BEGIN
              WIDGET_CONTROL, _WIDS.DETECTORS, SET_VALUE = [0, 0, 0, 0, 0, 0, 0, 0]
              END
         ELSE:

    ENDCASE ; for UVALUE
    END

 ELSE:

ENDCASE ; for TYPE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
PRO W_BFITS, SUBCMD
 

COMMON W_BFITS_DATA, _WIDS, _HIDS


VERSION = '0.9'
BFITS_APPLICATION = 'USRDK11:[BDAS.TEMP_EXECUTABLES.BFITS]BFITS_NOTAE.EXE'

IF (XREGISTERED('W_BFITS')) THEN BEGIN
   ; Because of COMMON BLOCKS, only one instance is allowed
   INFORM, 'W_BFITS already running.'
   RETURN
ENDIF

DEVICE, GET_SCREEN_SIZE = CURRENT_SCREEN

DATATYPES = ['DISC', 'SDISC', 'MER', 'HER', 'SHER', 'LIST']
IF (N_ELEMENTS(SUBCMD) EQ 0) THEN BEGIN
   INDEX      = WIDMENU(['Select BFITS Subcommand', DATATYPES], TITLE = 0)
   SUBCOMMAND = DATATYPES(INDEX - 1)
ENDIF ELSE BEGIN
   SUBCOMMAND = STRTRIM(SUBCMD, 2)
ENDELSE

CASE SUBCOMMAND OF
     'DISC':  BEGIN
              DEFAULT_DETECTORS        = [1, 1, 1, 1, 1, 1, 1, 1]
              DEFAULT_BRIGHT_DETECTORS = [0, 0, 0, 0]
              DEFAULT_DATATYPES        = ['DISCSC', 'PREB', 'DISCLA']
              AVAIL_DATATYPES          = ['DISCSC', 'PREB', 'DISCLA', $
                                          'DISCLB', 'TTE', 'TTS']
              DEFAULT_CHANNELS         = [1, 4]
              AVAIL_CHANNELS           = '[1, 4]'
              END
     'SDISC': BEGIN
              DEFAULT_DETECTORS        = [0, 0, 0, 0, 0, 0, 0, 0]
              DEFAULT_BRIGHT_DETECTORS = [1, 0, 0, 0]
              DEFAULT_DATATYPES        = ['SHERB', 'DISCSP']
              AVAIL_DATATYPES          = ['SHERB', 'DISCSP']
              DEFAULT_CHANNELS         = [1, 4]
              AVAIL_CHANNELS           = '[1, 4]'
              END
     'MER':   BEGIN
              DEFAULT_DETECTORS        = [1, 1, 1, 1, 1, 1, 1, 1]
              DEFAULT_BRIGHT_DETECTORS = [0, 0, 0, 0]
              DEFAULT_DATATYPES        = ['MER', 'CONT']
              AVAIL_DATATYPES          = ['MER', 'CONT']
              DEFAULT_CHANNELS         = [0, 15]
              AVAIL_CHANNELS           = '[0, 15]'
              END
     'HER':   BEGIN
              DEFAULT_DETECTORS        = [0, 0, 0, 0, 0, 0, 0, 0]
              DEFAULT_BRIGHT_DETECTORS = [1, 0, 0, 0]
              DEFAULT_DATATYPES        = ['HER', 'HERB']
              AVAIL_DATATYPES          = ['HER', 'HERB']
              DEFAULT_CHANNELS         = [0, 127]
              AVAIL_CHANNELS           = '[0, 127]'
              END
     'SHER':  BEGIN
              DEFAULT_DETECTORS        = [0, 0, 0, 0, 0, 0, 0, 0]
              DEFAULT_BRIGHT_DETECTORS = [1, 0, 0, 0]
              DEFAULT_DATATYPES        = ['SHER', 'SHERB']
              AVAIL_DATATYPES          = ['SHER', 'SHERB', 'STTE']
              DEFAULT_CHANNELS         = [4, 255]
              AVAIL_CHANNELS           = '[4, 255]'
              END
     'LIST':  BEGIN
              DEFAULT_DETECTORS        = [1, 1, 1, 1, 1, 1, 1, 1]
              DEFAULT_BRIGHT_DETECTORS = [0, 0, 0, 0]
              DEFAULT_DATATYPES        = ['STTE']
              AVAIL_DATATYPES          = ['STTE']
              DEFAULT_CHANNELS         = [4, 255]
              AVAIL_CHANNELS           = '[4, 255]'
              END
ENDCASE

_WIDS = {WIDGET_IDS, $
         STATUS: 0L, $
         TRIGGER_NUMBER: 0L, $
         OBJECT_LABEL: 0L, $
         IBDB_PATH: 0L, $
         FITS_FILENAME: 0L, $
         TIME_RANGE_LO: 0L, $
         TIME_RANGE_HI: 0L, $
         TIME_RES: 0L, $
         DETECTORS: 0L, $
         BRIGHT_DETECTORS: 0L, $
         DATATYPES: 0L, $
         CHANNEL_LO: 0L, $
         CHANNEL_HI: 0L, $
         CORTIME: 0L, $
         DEADTIME: 0L, $
         CALIBRATE: 0L, $
         SCRATCH_DIR: 0L, $
         QMASK: 0L}

_HIDS = {HANDLE_IDS, $
         SUBCOMMAND: SUBCOMMAND, $
         BFITS_APPLICATION: BFITS_APPLICATION, $

         ; Use a HANDLE here, since the array AVAIL_DATATYPES is variable length
         AVAIL_DATATYPES: HANDLE_CREATE()}

HANDLE_VALUE, _HIDS.AVAIL_DATATYPES, AVAIL_DATATYPES, /SET


; Parent
MTITLE   = STRUPCASE('W_BFITS V' + STRTRIM(VERSION, 2))
TOP_BASE = WIDGET_BASE(TITLE = MTITLE, /COLUMN, MAP = 0)

_WIDS.STATUS = WIDGET_TEXT(TOP_BASE, VALUE = ' ')


; Base for the rest of the menu
BASE1 = WIDGET_BASE(TOP_BASE, /COLUMN, /FRAME)

TEMP_BASE = WIDGET_BASE(BASE1, /ROW)
BUTTON    = WIDGET_BUTTON(TEMP_BASE, VALUE = 'QUIT')
BUTTON    = WIDGET_BUTTON(TEMP_BASE, VALUE = 'Help')
BUTTON    = WIDGET_BUTTON(TEMP_BASE, VALUE = 'Change Datatype')
BUTTON    = WIDGET_BUTTON(TEMP_BASE, VALUE = 'Save')
BUTTON    = WIDGET_BUTTON(TEMP_BASE, VALUE = 'Restore')

TEMP_BASE            = WIDGET_BASE(BASE1, /ROW)
LABEL                = WIDGET_LABEL(TEMP_BASE, VALUE = 'Trigger Number')
_WIDS.TRIGGER_NUMBER = WIDGET_TEXT(TEMP_BASE, VALUE = ' ', XSIZE = 5, /EDIT)

LABEL         = WIDGET_BUTTON(TEMP_BASE, VALUE = 'Object Class', MENU = 1)
BUTTON_LABELS = ['GRB', 'SFL', 'TGF', 'SGR', 'CYG', 'GCB', 'UNK']
FOR I=0, N_ELEMENTS(BUTTON_LABELS)-1 DO BEGIN
    PULLDOWN  = WIDGET_BUTTON(LABEL, VALUE = BUTTON_LABELS(I))
ENDFOR
_WIDS.OBJECT_LABEL = WIDGET_LABEL(TEMP_BASE, VALUE = BUTTON_LABELS(0))

TEMP_BASE       = WIDGET_BASE(BASE1, /ROW)
LABEL           = WIDGET_LABEL(TEMP_BASE, VALUE = 'IBDB Path')
_WIDS.IBDB_PATH = WIDGET_TEXT(TEMP_BASE, VALUE = 'BFITS$IBDB:', /EDIT)

;TEMP_BASE           = WIDGET_BASE(BASE1, /ROW)
LABEL               = WIDGET_LABEL(TEMP_BASE, VALUE = 'Output FITS Filename')
_WIDS.FITS_FILENAME = WIDGET_TEXT(TEMP_BASE, VALUE = 'DATA.FITS', /EDIT)

TEMP_BASE           = WIDGET_BASE(BASE1, /ROW)
LABEL               = WIDGET_LABEL(TEMP_BASE, VALUE = 'Time Range')
_WIDS.TIME_RANGE_LO = WIDGET_TEXT(TEMP_BASE, VALUE = '-3600.0', $
                                  XSIZE = 10, /EDIT)
_WIDS.TIME_RANGE_HI = WIDGET_TEXT(TEMP_BASE, VALUE = '+3600.0', $
                                  XSIZE = 10, /EDIT)

LABEL          = WIDGET_LABEL(TEMP_BASE, VALUE = 'Minimum Time Resolution')
_WIDS.TIME_RES = WIDGET_TEXT(TEMP_BASE, VALUE = '1.00e-02', $
                             XSIZE = 10, /EDIT)
 
TEMP_BASE       = WIDGET_BASE(BASE1, /ROW)
LABEL           = WIDGET_LABEL(TEMP_BASE, VALUE = 'Detectors')
BUTTON_LABELS   = ['0', '1', '2', '3', '4', '5', '6', '7']
_WIDS.DETECTORS = CW_BGROUP(TEMP_BASE, BUTTON_LABELS, /NONEXCLUSIVE, $
                           /ROW, UVALUE = 'DETECTOR_BUTTONS')
WIDGET_CONTROL, _WIDS.DETECTORS, SET_VALUE = DEFAULT_DETECTORS

TEMP_BASE              = WIDGET_BASE(BASE1, /ROW)
LABEL                  = WIDGET_LABEL(TEMP_BASE, VALUE = 'Bright Detectors')
BUTTON_LABELS          = ['A', 'B', 'C', 'D']
_WIDS.BRIGHT_DETECTORS = CW_BGROUP(TEMP_BASE, BUTTON_LABELS, /NONEXCLUSIVE, $
                                  /ROW, UVALUE = 'BRIGHT_DETECTOR_BUTTONS')
WIDGET_CONTROL, _WIDS.BRIGHT_DETECTORS, SET_VALUE = DEFAULT_BRIGHT_DETECTORS

TEMP_BASE       = WIDGET_BASE(BASE1, /ROW)
LABEL           = WIDGET_LABEL(TEMP_BASE, VALUE = 'Datatypes')
BUTTON_LABELS   = AVAIL_DATATYPES
_WIDS.DATATYPES = CW_BGROUP(TEMP_BASE, BUTTON_LABELS, /NONEXCLUSIVE, $
                           /ROW, UVALUE = 'DATA_BUTTONS')
WIDGET_CONTROL, _WIDS.DATATYPES, $
                SET_VALUE = [INTARR(N_ELEMENTS(DEFAULT_DATATYPES)) + 1]

TEMP_BASE = WIDGET_BASE(BASE1, /ROW)
LABEL     = WIDGET_LABEL(TEMP_BASE, VALUE = 'Channels')
_WIDS.CHANNEL_LO = WIDGET_TEXT(TEMP_BASE, $
                     VALUE = STRTRIM(DEFAULT_CHANNELS(0), 2), XSIZE = 4, /EDIT)
_WIDS.CHANNEL_HI = WIDGET_TEXT(TEMP_BASE, $
                     VALUE = STRTRIM(DEFAULT_CHANNELS(1), 2), XSIZE = 4, /EDIT)
LABEL = WIDGET_LABEL(TEMP_BASE, VALUE = AVAIL_CHANNELS)

T_BASE = WIDGET_BASE(BASE1, /ROW)
IF (SUBCOMMAND EQ 'DISC') THEN BEGIN
   TEMP_BASE     = WIDGET_BASE(T_BASE, /ROW)
   LABEL         = WIDGET_LABEL(TEMP_BASE, VALUE = 'Cortime')
   BUTTON_LABELS = ['No', 'Yes']
   _WIDS.CORTIME = CW_BGROUP(TEMP_BASE, BUTTON_LABELS, /EXCLUSIVE, /FRAME, $
                              /ROW, /NO_RELEASE, UVALUE = 'CORTIME_BUTTONS')
   WIDGET_CONTROL, _WIDS.CORTIME, SET_VALUE = 0
ENDIF ELSE BEGIN
   _WIDS.CORTIME = -1L
ENDELSE

TEMP_BASE      = WIDGET_BASE(T_BASE, /ROW)
LABEL          = WIDGET_LABEL(TEMP_BASE, VALUE = 'Deadtime')
BUTTON_LABELS  = ['No', 'Yes']
_WIDS.DEADTIME = CW_BGROUP(TEMP_BASE, BUTTON_LABELS, /EXCLUSIVE, /FRAME, $
                           /ROW, /NO_RELEASE, UVALUE = 'DEADTIME_BUTTONS')
WIDGET_CONTROL, _WIDS.DEADTIME, SET_VALUE = 0

TEMP_BASE     = WIDGET_BASE(T_BASE, /ROW)
LABEL         = WIDGET_LABEL(TEMP_BASE, VALUE = 'Calibrate')
BUTTON_LABELS = ['No', 'Yes']
_WIDS.CALIBRATE = CW_BGROUP(TEMP_BASE, BUTTON_LABELS, /EXCLUSIVE, /FRAME, $
                           /ROW, /NO_RELEASE, UVALUE = 'CALIBRATE_BUTTONS')
WIDGET_CONTROL, _WIDS.CALIBRATE, SET_VALUE = 1

TEMP_BASE         = WIDGET_BASE(BASE1, /ROW)
LABEL             = WIDGET_LABEL(TEMP_BASE, VALUE = 'Scratch Directory')
_WIDS.SCRATCH_DIR = WIDGET_TEXT(TEMP_BASE, VALUE = 'PSCRATCH:', /EDIT)

;TEMP_BASE   = WIDGET_BASE(BASE1, /ROW)
LABEL       = WIDGET_LABEL(TEMP_BASE, VALUE = 'Quality Mask')
_WIDS.QMASK = WIDGET_TEXT(TEMP_BASE, VALUE = 'IBDB0', /EDIT)

BUTTON = WIDGET_BUTTON(BASE1, VALUE = 'Create FITS File')

WIDGET_CONTROL, TOP_BASE, /REALIZE
WIDGET_CONTROL, TOP_BASE, TLB_GET_SIZE = MENU_SIZE
MENU_CENTER = [(CURRENT_SCREEN(0) / 2.0) - (MENU_SIZE(0) / 2.0), $
               (CURRENT_SCREEN(1) / 2.0) - (MENU_SIZE(1) / 2.0)]
WIDGET_CONTROL, TOP_BASE, $
                TLB_SET_XOFFSET = MENU_CENTER(0), $
                TLB_SET_YOFFSET = MENU_CENTER(1)
WIDGET_CONTROL, TOP_BASE, MAP = 1



IF (N_ELEMENTS(SUBCMD) NE 0) THEN BEGIN

   OPENR, FL_IN, 'SYS$LOGIN:W_BFITS.SAV', /GET_LUN

   TEMP = ' '
   READF, FL_IN, TEMP
   _HIDS.SUBCOMMAND = TEMP

   TEMP = ' '
   READF, FL_IN, TEMP
   WIDGET_CONTROL, _WIDS.TRIGGER_NUMBER, SET_VALUE = TEMP

   READF, FL_IN, TEMP
   WIDGET_CONTROL, _WIDS.OBJECT_LABEL, SET_VALUE = TEMP

   READF, FL_IN, TEMP
   WIDGET_CONTROL, _WIDS.IBDB_PATH, SET_VALUE = TEMP

   READF, FL_IN, TEMP
   WIDGET_CONTROL, _WIDS.FITS_FILENAME, SET_VALUE = TEMP

   FLOAT_1 = 0.0 & FLOAT_2 = 0.0
   READF, FL_IN, FLOAT_1, FLOAT_2
   WIDGET_CONTROL, _WIDS.TIME_RANGE_LO, SET_VALUE = STRTRIM(FLOAT_1, 2)
   WIDGET_CONTROL, _WIDS.TIME_RANGE_HI, SET_VALUE = STRTRIM(FLOAT_2, 2)

   TEMP = 0.0
   READF, FL_IN, FLOAT_1
   WIDGET_CONTROL, _WIDS.TIME_RES, SET_VALUE = STRTRIM(FLOAT_1, 2)

   TEMP = INTARR(8)
   READF, FL_IN, TEMP
   WIDGET_CONTROL, _WIDS.DETECTORS, SET_VALUE = STRTRIM(TEMP, 2)

   TEMP = INTARR(4)
   READF, FL_IN, TEMP
   WIDGET_CONTROL, _WIDS.BRIGHT_DETECTORS, SET_VALUE = STRTRIM(TEMP, 2)

   TEMP = INTARR(N_ELEMENTS(AVAIL_DATATYPES))
   READF, FL_IN, TEMP
   WIDGET_CONTROL, _WIDS.DATATYPES, SET_VALUE = STRTRIM(TEMP, 2)

   INT_1 = 0 & INT_2 = 0
   READF, FL_IN, INT_1, INT_2
   WIDGET_CONTROL, _WIDS.CHANNEL_LO, SET_VALUE = STRTRIM(INT_1, 2)
   WIDGET_CONTROL, _WIDS.CHANNEL_HI, SET_VALUE = STRTRIM(INT_2, 2)

   IF (_HIDS.SUBCOMMAND EQ 'DISC') THEN BEGIN
      READF, FL_IN, INT_1
      WIDGET_CONTROL, _WIDS.CORTIME, SET_VALUE = STRTRIM(INT_1, 2)
   ENDIF

   READF, FL_IN, INT_1
   WIDGET_CONTROL, _WIDS.DEADTIME, SET_VALUE = STRTRIM(INT_1, 2)

   READF, FL_IN, INT_1
   WIDGET_CONTROL, _WIDS.CALIBRATE, SET_VALUE = STRTRIM(INT_1, 2)

   TEMP = ' '
   READF, FL_IN, TEMP
   WIDGET_CONTROL, _WIDS.SCRATCH_DIR, SET_VALUE = TEMP

   READF, FL_IN, TEMP
   WIDGET_CONTROL, _WIDS.QMASK, SET_VALUE = TEMP

   FREE_LUN, FL_IN

   WIDGET_CONTROL, _WIDS.STATUS, SET_VALUE = 'Restored ' + $
                   'SYS$LOGIN:W_BFITS.SAV'

ENDIF ELSE BEGIN
  
   WIDGET_CONTROL, _WIDS.STATUS, SET_VALUE = 'Using ' + BFITS_APPLICATION

ENDELSE

; Register with XMANAGER
XMANAGER, 'W_BFITS', TOP_BASE, /MODAL




END
