; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; Creates a dialog for setting control variables

FUNCTION CV_DIALOG, INPUT_TITLE, INPUT_LIST, STARTING_VALUE, $
         RADIO = RADIO, TEXT = TEXT

; Parent
BASE = WIDGET_BASE(TITLE = ' ', /COLUMN, MAP = 0)

BUTTON = WIDGET_BUTTON(BASE, VALUE = 'Ok')


IF (N_ELEMENTS(RADIO) NE 0) THEN BEGIN

   ; Create a radio button selection widget
   RESULT = STARTING_VALUE
   
   LABEL = WIDGET_LABEL(BASE, VALUE = INPUT_TITLE)
  
   TEMP_BASE = WIDGET_BASE(BASE, /ROW)
    
   BUTTONS = CW_BGROUP(TEMP_BASE, INPUT_LIST, /EXCLUSIVE, $
                /NO_RELEASE, COLUMN = 1)
   
   WIDGET_CONTROL, BUTTONS, SET_VALUE = STARTING_VALUE
    
ENDIF

IF (N_ELEMENTS(TEXT) NE 0) THEN BEGIN

   ; Create a text input widget
   RESULT = INPUT_LIST
   LABEL = WIDGET_LABEL(BASE, VALUE = INPUT_TITLE)
   TEXT = WIDGET_TEXT(BASE, VALUE = INPUT_LIST, YSIZE = 1, /EDIT)
      
ENDIF


; Realize the BASE widget and the children
WIDGET_CONTROL, BASE, /REALIZE, SHOW = 1 

; Place the menu in center of screen
DEVICE, GET_SCREEN_SIZE = CURRENT_SCREEN
WIDGET_CONTROL, BASE, TLB_GET_SIZE = MENU_SIZE
MENU_CENTER = [(CURRENT_SCREEN(0) / 2.0) - (MENU_SIZE(0) / 2.0), $ 
               (CURRENT_SCREEN(1) / 2.0) - (MENU_SIZE(1) / 2.0)] 
WIDGET_CONTROL, BASE, $
                TLB_SET_XOFFSET = MENU_CENTER(0), $
                TLB_SET_YOFFSET = MENU_CENTER(1)
WIDGET_CONTROL, BASE, MAP = 1

; Place the cursor 
WIDGET_CONTROL, LABEL, /INPUT_FOCUS


EVENT_FLAG = 0
REPEAT BEGIN

      ; Get the event, without using XMANAGER
      EVENT = WIDGET_EVENT(BASE)

      ; Process the event
      TYPE = TAG_NAMES(EVENT, /STRUCTURE)

      CASE TYPE OF

      ; The text input events
      'WIDGET_TEXT_CH': BEGIN
          WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE
          RESULT = VALUE(0)
          END
        
      ; The radio button events
      '': BEGIN
          RESULT = EVENT.VALUE
          END

      ; The widget button events
      'WIDGET_BUTTON': BEGIN
          WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE
          IF (STRUPCASE(VALUE(0)) EQ 'OK') THEN EVENT_FLAG = 1
          END

     ENDCASE

ENDREP UNTIL (EVENT_FLAG NE 0)


WIDGET_CONTROL, EVENT.TOP, /DESTROY


RETURN, RESULT
END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 


; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; This is the event handler for CV Select - PLOTTING
PRO CV_SELECT_PLOTTING_EVENT, EVENT

@wgsp_cb.pro
 
 
NO_OR_YES = ['No', 'Yes']

TYPE = TAG_NAMES(EVENT, /STRUCTURE)
CASE TYPE OF

 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ; The list widget events
 
 ; Calling CV_DIALOG:  
 ;  To create a dialog with a label and an editable
 ;  text entry field, call CV_DIALOG with two arguments: a string for the
 ;  widget label, and a string for the starting text field.  Also set the
 ;  keyword /TEXT.
 ;
 ;  To create a dialog with a label and radio buttons, call CV_DIALOG with
 ;  three arguments: a string for the widget label, a string array of the 
 ;  labels for each of the radio buttons, and the index of the string array
 ;  which sets the initial selection of the radio buttons.  Also set the
 ;  keyword /RADIO.
  
 'WIDGET_LIST': BEGIN
     WIDGET_CONTROL, EVENT.ID, GET_UVALUE = UVALUE
     CASE STRUPCASE(UVALUE(EVENT.INDEX)) OF

          'PRINTER LANGUAGE': BEGIN
                   RESULT = CV_DIALOG('CV_PRINTER_LANG', $
                            STRTRIM(CV_PRINTER_LANG, 2), /TEXT) 
                   STRPUT, CV_PRINTER_LANG, STRUPCASE(RESULT)
                   END                   
                   
          'PLOT EXTRAPOLATED PHOTON MODEL': BEGIN
                   IF (CV_PLOT_MODEL_EXTRAP) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_PLOT_MODEL_EXTRAP', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_PLOT_MODEL_EXTRAP = LONG(RESULT)
                   END              
          
          'PLOT INDIVIDUAL TERMS OF PHOTON MODEL': BEGIN
                   IF (CV_PLOT_EACH_TERM) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_PLOT_EACH_TERM', $
                            NO_OR_YES, TEMP, /RADIO)
                   CV_PLOT_EACH_TERM = LONG(RESULT)
                   END               

          'PLOT ABSOLUTE VALUE OF INDIVIDUAL TERMS': BEGIN
                   IF (CV_ABS_MODEL) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_ABS_MODEL', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_ABS_MODEL = LONG(RESULT)
                   END          

          'PLOT DATA ONLY': BEGIN
                   IF (CV_PLOT_ONLY_DATA) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_PLOT_ONLY_DATA', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_PLOT_ONLY_DATA = LONG(RESULT)
                   END          

          'SUPPRESS SECONDARY PLOT LABELS': BEGIN
                   IF (CV_SUPPRESS_2ND_LABELS) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_SUPPRESS_2ND_LABELS', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_SUPPRESS_2ND_LABELS = LONG(RESULT)
                   END          

          'MULTI-DETECTOR PLOT MODE': BEGIN
                   CASE STRTRIM(CV_MULTI_DET_PLOT_MODE, 2) OF 
                        'S': TEMP = 0 
                        'T': TEMP = 1
                        'C': TEMP = 2
                         ELSE: BEGIN
                               PRINT, 'ERROR: W_CVSELECTOR'
                               HELP, CV_MULTI_DET_PLOT_MODE
                               END
                   ENDCASE
                   SELECTIONS = ['S', 'T', 'C']
                   W_SELECTIONS = ['Separate Plots (S)', 'Single Plot (T)', $
                                   'Separate Count Rate Plot (C)']
                   RESULT = CV_DIALOG('CV_MULTI_DET_PLOT_MODE', $
                            W_SELECTIONS, TEMP, /RADIO) 
                   STRPUT, CV_MULTI_DET_PLOT_MODE, STRUPCASE(SELECTIONS(RESULT))
                   END          
                   
          'PLOT AXIS LIMIT MODE': BEGIN
                   CASE STRTRIM(CV_GRAPH_LIMIT_MODE, 2) OF 
                        'P': TEMP = 0 
                        'M': TEMP = 1
                        'B': TEMP = 2
                         ELSE: BEGIN
                               PRINT, 'ERROR: W_CVSELECTOR'
                               HELP, CV_GRAPH_LIMIT_MODE
                               END
                   ENDCASE
                   SELECTIONS = ['P', 'M', 'B']
                   W_SELECTIONS = ['Power of Ten (P)', 'Minor Tickmarks (M)', $
                                   'Both (B)']
                   RESULT = CV_DIALOG('CV_GRAPH_LIMIT_MODE', $
                            W_SELECTIONS, TEMP, /RADIO) 
                   STRPUT, CV_GRAPH_LIMIT_MODE, STRUPCASE(SELECTIONS(RESULT))
                   END        
                   
          'HARDCOPY FILE STATUS': BEGIN
                   CASE STRTRIM(CV_HCOPY_FILE_STATUS, 2) OF 
                        'DELETE': TEMP = 0 
                        'KEEP': TEMP = 1
                         ELSE: BEGIN
                               PRINT, 'ERROR: W_CVSELECTOR'
                               HELP, CV_HCOPY_FILE_STATUS
                               END
                   ENDCASE
                   SELECTIONS = ['Delete', 'Keep']
                   RESULT = CV_DIALOG('CV_HCOPY_FILE_STATUS', $
                            SELECTIONS, TEMP, /RADIO) 
                   CV_HCOPY_FILE_STATUS = STRING(REPLICATE(32B, 25))
                   STRPUT, CV_HCOPY_FILE_STATUS, STRUPCASE(SELECTIONS(RESULT))
                   END        
                   
          'CREATE ENCAPSULATED PS FILES': BEGIN
                   IF (CV_ENCAPSULATED_PS) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_ENCAPSULATED_PS', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_ENCAPSULATED_PS = LONG(RESULT)
                   END  
                   
          'PLOT STYLE FOR COUNT RATE MODEL': BEGIN
                   RESULT = CV_DIALOG('CV_SPEC_STYLE', $
                            STRTRIM(CV_SPEC_STYLE, 2), /TEXT) 
                   CV_SPEC_STYLE = LONG(RESULT)
                   END   

          'AXIS RATE MODE': BEGIN
                   CASE STRTRIM(CV_RATE_MODE, 2) OF 
                        'R': TEMP = 0 
                        'E': TEMP = 1
                         ELSE: BEGIN
                               PRINT, 'ERROR: W_CVSELECTOR'
                               HELP, CV_RATE_MODE
                               END
                   ENDCASE
                   SELECTIONS = ['R', 'E']
                   W_SELECTIONS = ['Counts/s (R)', 'Counts/s-keV (E)']
                   RESULT = CV_DIALOG('CV_RATE_MODE', $
                            W_SELECTIONS, TEMP, /RADIO) 
                   STRPUT, CV_RATE_MODE, STRUPCASE(SELECTIONS(RESULT))
                   END        
                   
          'TRACK MOUSE POINTER': BEGIN
                   IF (CV_TRACK_MOUSE) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_TRACK_MOUSE', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_TRACK_MOUSE = LONG(RESULT)
                   END  
                   
          'PLOT UNDETERMINED PARAMETERS': BEGIN
                   IF (CV_PLOT_UNDETERMINED) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_PLOT_UNDETERMINED', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_PLOT_UNDETERMINED = LONG(RESULT)
                   END  
                   
          'PLOT INTERVAL SELECTIONS ON HARDCOPY': BEGIN
                   IF (CV_PLOT_CURSOR) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_PLOT_CURSOR', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_PLOT_CURSOR = LONG(RESULT)
                   END  
                   
          'WINDOW BACKING STORE': BEGIN
                   TEMP = CV_IDL_RETAIN
                   RESULT = CV_DIALOG('CV_IDL_RETAIN', $
                            ['0', '1', '2'], TEMP, /RADIO) 
                   CV_IDL_RETAIN = LONG(RESULT)
                   END        
                   
          'PLOT TITLE DEFAULT': BEGIN
                   CASE STRTRIM(CV_TITLE_STYLE, 2) OF 
                        'N': TEMP = 0 
                        'T': TEMP = 1
                         ELSE: BEGIN
                               PRINT, 'ERROR: W_CVSELECTOR'
                               HELP, CV_TITLE_STYLE
                               END
                   ENDCASE
                   SELECTIONS = ['N', 'T']
                   W_SELECTIONS = ['Object Name (N)', 'BATSE Trigger (T)']
                   RESULT = CV_DIALOG('CV_TITLE_STYLE', $
                            W_SELECTIONS, TEMP, /RADIO) 
                   STRPUT, CV_TITLE_STYLE, STRUPCASE(SELECTIONS(RESULT))
                   END        
          
     ENDCASE ; for UVALUE
     END

 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ; The button widget events
 'WIDGET_BUTTON': BEGIN
    WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE
                   
    VALUE = STRUPCASE(VALUE)
    CASE 1 OF 

         VALUE EQ 'HELP': WM_XDISPLAYFILE, $
                          WGSP_SOURCE + STRLOWCASE('MFIT_CONTROL_DECLARE.INCL')

         VALUE EQ 'DONE': BEGIN
 
               IF (STRTRIM(STRLOWCASE(MFIT_CONTROL_FILENAME), 2) EQ $
                  STRTRIM(STRLOWCASE(WGSP_SOURCE + 'MFIT_CONTROL.DAT'), 2)) $
                  THEN BEGIN
   
                  FILE_ERR = WIDMENU(['Create mfit_control.dat in WING_SCRATCH?', $
                                     'Yes', 'No'], TITLE = 0)
                  IF (FILE_ERR EQ 1) THEN BEGIN
                     MFIT_CONTROL_FILENAME = STRING(REPLICATE(32B, 128)) 
                     STRPUT, MFIT_CONTROL_FILENAME, $ 
                             WGSP_SCRATCH + STRLOWCASE('MFIT_CONTROL.DAT')
                  
                     OPENW, FL_U, STRTRIM(WGSP_SCRATCH + $
                            STRLOWCASE('MFIT_CONTROL.DAT'), 2), /GET_LUN
                     FREE_LUN, FL_U
                  ENDIF ELSE BEGIN
                     WIDGET_CONTROL, EVENT.TOP, /DESTROY
                     RETURN
                  ENDELSE
               ENDIF

               WRITE_CONTROL, STRTRIM(MFIT_CONTROL_FILENAME, 2)
               INFORM, 'Wrote ' + STRTRIM(MFIT_CONTROL_FILENAME, 2)
               GET_CONTROL
               WIDGET_CONTROL, EVENT.TOP, /DESTROY
               END

         VALUE EQ 'APPLY': BEGIN
 
               IF (STRTRIM(STRLOWCASE(MFIT_CONTROL_FILENAME), 2) EQ $
                  STRTRIM(STRLOWCASE(WGSP_SOURCE + 'MFIT_CONTROL.DAT'), 2)) $
                  THEN BEGIN
   
                  FILE_ERR = WIDMENU(['Create mfit_control.dat in WING_SCRATCH?', $
                                     'Yes', 'No'], TITLE = 0)
                  IF (FILE_ERR EQ 1) THEN BEGIN
                     MFIT_CONTROL_FILENAME = STRING(REPLICATE(32B, 128)) 
                     STRPUT, MFIT_CONTROL_FILENAME, $ 
                             WGSP_SCRATCH + STRLOWCASE('MFIT_CONTROL.DAT')
                  
                     OPENW, FL_U, STRTRIM(WGSP_SCRATCH + $
                            STRLOWCASE('MFIT_CONTROL.DAT'), 2), /GET_LUN
                     FREE_LUN, FL_U
                  ENDIF ELSE BEGIN
                     WIDGET_CONTROL, EVENT.TOP, /DESTROY
                     RETURN
                  ENDELSE
               ENDIF

               WRITE_CONTROL, STRTRIM(MFIT_CONTROL_FILENAME, 2)
               INFORM, 'Wrote ' + STRTRIM(MFIT_CONTROL_FILENAME, 2)

               IF (CV_RATE_MODE NE RATE_MODE_SAVE) THEN BEGIN
                  WM_UPDATE
                  IF (N_ELEMENTS(MFIT_WINDOW) NE 0) THEN BEGIN
                     WINDOW_SAVE = !D.WINDOW
                     WSET, MFIT_WINDOW
                     WW_UPDATE, 'HISTORY'
                     WSET, WINDOW_SAVE
                  ENDIF
               ENDIF

               GET_CONTROL
               END
 
    ENDCASE ; for VALUE
    END

 
ENDCASE ; for TYPE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
PRO CV_SELECT_PLOTTING

@wgsp_cb.pro


GET_CONTROL

IF (STRTRIM(STRLOWCASE(WGSP_SCRATCH), 2) EQ $
   STRTRIM(STRLOWCASE(WGSP_SOURCE), 2)) $
   THEN BEGIN
   FILE_ERR = WIDMENU(['Cannot write mfit_control.dat in WING_SOURCE', $
                       'OK'], TITLE = 0)
   RETURN
ENDIF


; Parent
MTITLE = 'CV Selector - PLOTTING'
CV_BASE = WIDGET_BASE(TITLE = MTITLE, /COLUMN, MAP = 0)

DBUTTON = WIDGET_BUTTON(CV_BASE, VALUE = 'DONE')
ABUTTON = WIDGET_BUTTON(CV_BASE, VALUE = 'Apply')
HBUTTON = WIDGET_BUTTON(CV_BASE, VALUE = 'Help')
LABEL   = WIDGET_LABEL(CV_BASE, VALUE = 'Use <enter> After Text Entries')

CONTROL_VAR = ['Printer Language', $
               'Plot Extrapolated Photon Model', $
               'Plot Individual Terms of Photon Model', $
               'Plot Absolute Value of Individual Terms', $
               'Plot Data Only', $
               'Suppress Secondary Plot Labels', $
               'Multi-detector Plot Mode', $
               'Plot Axis Limit Mode', $
               'Hardcopy File Status', $
               'Create Encapsulated PS Files', $
               'Plot Style for Count Rate Model', $
               'Axis Rate Mode', $
               'Track Mouse Pointer', $
               'Plot Undetermined Parameters', $
               'Plot Interval Selections on Hardcopy', $
               'Window Backing Store', $
               'Plot Title Default']

CONTROL_VAR = CONTROL_VAR(SORT(CONTROL_VAR))

LIST = WIDGET_LIST(CV_BASE, VALUE = CONTROL_VAR, $
                  YSIZE = 10, UVALUE = CONTROL_VAR)

; Realize the BASE widget and the children
WIDGET_CONTROL, CV_BASE, /REALIZE, SHOW = 1 

; Place the widget in center of screen
DEVICE, GET_SCREEN_SIZE = CURRENT_SCREEN
WIDGET_CONTROL, CV_BASE, TLB_GET_SIZE = MENU_SIZE
MENU_CENTER = [(CURRENT_SCREEN(0) / 2.0) - (MENU_SIZE(0) / 2.0), $ 
               (CURRENT_SCREEN(1) / 2.0) - (MENU_SIZE(1) / 2.0)] 
WIDGET_CONTROL, CV_BASE, $
                TLB_SET_XOFFSET = MENU_CENTER(0), $
                TLB_SET_YOFFSET = MENU_CENTER(1)
WIDGET_CONTROL, CV_BASE, MAP = 1

; Place the cursor 
WIDGET_CONTROL, DBUTTON, /INPUT_FOCUS

; Register with XMANAGER
XMANAGER, 'CV_SELECT_PLOTTING', CV_BASE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

 

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; This is the event handler for CV Select - FILES
PRO CV_SELECT_FILES_EVENT, EVENT

@wgsp_cb.pro


NO_OR_YES = ['No', 'Yes']

TYPE = TAG_NAMES(EVENT, /STRUCTURE)
CASE TYPE OF

 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ; The list widget events
 'WIDGET_LIST': BEGIN
     WIDGET_CONTROL, EVENT.ID, GET_UVALUE = UVALUE

     CASE STRUPCASE(UVALUE(EVENT.INDEX)) OF

          'KEEP ASCII FILE OPEN': BEGIN
                   IF (CV_KEEP_ASCII_OPEN) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_KEEP_ASCII_OPEN', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_KEEP_ASCII_OPEN = LONG(RESULT)
                   END  
           
          'ASCII FILENAME': BEGIN
                   RESULT = CV_DIALOG('CV_ASCII_OUT_FILE', $
                            STRTRIM(CV_ASCII_OUT_FILE, 2), /TEXT) 
                   STRPUT, CV_ASCII_OUT_FILE, STRUPCASE(RESULT)
                   END   
          
          'STARTUP TEMPLATE FILENAME': BEGIN
                   RESULT = CV_DIALOG('CV_STARTUP_TEMPLATE_FILE', $
                            STRTRIM(CV_STARTUP_TEMPLATE_FILE, 2), /TEXT) 
                   STRPUT, CV_STARTUP_TEMPLATE_FILE, STRLOWCASE(RESULT)
                   END 
          
          'WRITE ADDITIONAL FIT INFO TO BINARY FILE': BEGIN
                   IF (CV_FULL_BATCH_OUT) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_FULL_BATCH_OUT', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_FULL_BATCH_OUT = LONG(RESULT)
                   END  
          
          'SUPPRESS BINARY FILE OUTPUT': BEGIN
                   IF (CV_SUPPRESS_BATCH_OUT) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_SUPPRESS_BATCH_OUT', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_SUPPRESS_BATCH_OUT = LONG(RESULT)
                   END            
          
          'BINARY FILENAME': BEGIN
                   RESULT = CV_DIALOG('CV_BATCH_OUT_FILE', $
                            STRTRIM(CV_BATCH_OUT_FILE, 2), /TEXT) 
                   STRPUT, CV_BATCH_OUT_FILE, STRLOWCASE(RESULT)
                   END 
                             
          'ALWAYS WRITE ASCII FILE': BEGIN
                  IF (CV_ALWAYS_WA) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_ALWAYS_WA', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_ALWAYS_WA = LONG(RESULT)
                   END 

          
          'ALWAYS WRITE BINARY FILE': BEGIN
                  IF (CV_ALWAYS_WB) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_ALWAYS_WB', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_ALWAYS_WB = LONG(RESULT)
                   END                
               
     ENDCASE ; for UVALUE
     END               
  
 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ; The button widget events
 'WIDGET_BUTTON': BEGIN
    WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE
                   
    VALUE = STRUPCASE(VALUE)
    CASE 1 OF 

         VALUE EQ 'HELP': WM_XDISPLAYFILE, $
                          WGSP_SOURCE + STRLOWCASE('MFIT_CONTROL_DECLARE.INCL')

         VALUE EQ 'DONE': BEGIN

               IF (STRTRIM(STRLOWCASE(MFIT_CONTROL_FILENAME), 2) EQ $
                  STRTRIM(STRLOWCASE(WGSP_SOURCE + 'MFIT_CONTROL.DAT'), 2)) $
                  THEN BEGIN
   
                  FILE_ERR = WIDMENU(['Create mfit_control.dat in WING_SCRATCH?', $
                                     'Yes', 'No'], TITLE = 0)
                  IF (FILE_ERR EQ 1) THEN BEGIN
                     MFIT_CONTROL_FILENAME = STRING(REPLICATE(32B, 128)) 
                     STRPUT, MFIT_CONTROL_FILENAME, $ 
                             WGSP_SCRATCH + STRLOWCASE('MFIT_CONTROL.DAT')
                  
                     OPENW, FL_U, STRTRIM(WGSP_SCRATCH + $
                            STRLOWCASE('MFIT_CONTROL.DAT'), 2), /GET_LUN
                     FREE_LUN, FL_U
                  ENDIF ELSE BEGIN
                     WIDGET_CONTROL, EVENT.TOP, /DESTROY
                     RETURN
                  ENDELSE
               ENDIF

               WRITE_CONTROL, STRTRIM(MFIT_CONTROL_FILENAME, 2)
               INFORM, 'Wrote ' + STRTRIM(MFIT_CONTROL_FILENAME, 2)
               GET_CONTROL
               WIDGET_CONTROL, EVENT.TOP, /DESTROY
               END

         VALUE EQ 'APPLY': BEGIN

               IF (STRTRIM(STRLOWCASE(MFIT_CONTROL_FILENAME), 2) EQ $
                  STRTRIM(STRLOWCASE(WGSP_SOURCE + 'MFIT_CONTROL.DAT'), 2)) $
                  THEN BEGIN
   
                  FILE_ERR = WIDMENU(['Create mfit_control.dat in WING_SCRATCH?', $
                                     'Yes', 'No'], TITLE = 0)
                  IF (FILE_ERR EQ 1) THEN BEGIN
                     MFIT_CONTROL_FILENAME = STRING(REPLICATE(32B, 128)) 
                     STRPUT, MFIT_CONTROL_FILENAME, $ 
                             WGSP_SCRATCH + STRLOWCASE('MFIT_CONTROL.DAT')
                  
                     OPENW, FL_U, STRTRIM(WGSP_SCRATCH + $
                            STRLOWCASE('MFIT_CONTROL.DAT'), 2), /GET_LUN
                     FREE_LUN, FL_U
                  ENDIF ELSE BEGIN
                     WIDGET_CONTROL, EVENT.TOP, /DESTROY
                     RETURN
                  ENDELSE
               ENDIF

               WRITE_CONTROL, STRTRIM(MFIT_CONTROL_FILENAME, 2)
               INFORM, 'Wrote ' + STRTRIM(MFIT_CONTROL_FILENAME, 2)
               GET_CONTROL
               END
 
    ENDCASE ; for VALUE
    END
 
 
ENDCASE ; for TYPE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
PRO CV_SELECT_FILES

@wgsp_cb.pro
 

GET_CONTROL

IF (STRTRIM(STRLOWCASE(WGSP_SCRATCH), 2) EQ $
   STRTRIM(STRLOWCASE(WGSP_SOURCE), 2)) $
   THEN BEGIN
   FILE_ERR = WIDMENU(['Cannot write mfit_control.dat in WING_SOURCE', $
                       'OK'], TITLE = 0)
   RETURN
ENDIF

; Parent
MTITLE = 'CV Selector - FILES'
CV_BASE = WIDGET_BASE(TITLE = MTITLE, /COLUMN, MAP = 0)

DBUTTON = WIDGET_BUTTON(CV_BASE, VALUE = 'DONE')
ABUTTON = WIDGET_BUTTON(CV_BASE, VALUE = 'Apply')
HBUTTON = WIDGET_BUTTON(CV_BASE, VALUE = 'Help')
LABEL   = WIDGET_LABEL(CV_BASE, VALUE = 'Use <enter> After Text Entries')

CONTROL_VAR = ['Keep ASCII File Open', $
               'ASCII Filename', $
               'Startup Template Filename', $
               'Write Additional Fit Info to Binary File', $
               'Suppress Binary File Output', $
               'Binary Filename', $
               'ALWAYS Write ASCII File', $
               'ALWAYS Write Binary File']

CONTROL_VAR = CONTROL_VAR(SORT(CONTROL_VAR))
 

LIST = WIDGET_LIST(CV_BASE, VALUE = CONTROL_VAR, $
                  YSIZE = 10, UVALUE = CONTROL_VAR)
 

; Realize the BASE widget and the children
WIDGET_CONTROL, CV_BASE, /REALIZE, SHOW = 1 

; Place the widget in center of screen
DEVICE, GET_SCREEN_SIZE = CURRENT_SCREEN
WIDGET_CONTROL, CV_BASE, TLB_GET_SIZE = MENU_SIZE
MENU_CENTER = [(CURRENT_SCREEN(0) / 2.0) - (MENU_SIZE(0) / 2.0), $ 
               (CURRENT_SCREEN(1) / 2.0) - (MENU_SIZE(1) / 2.0)] 
WIDGET_CONTROL, CV_BASE, $
                TLB_SET_XOFFSET = MENU_CENTER(0), $
                TLB_SET_YOFFSET = MENU_CENTER(1)
WIDGET_CONTROL, CV_BASE, MAP = 1

; Place the cursor 
WIDGET_CONTROL, DBUTTON, /INPUT_FOCUS

; Register with XMANAGER
XMANAGER, 'CV_SELECT_FILES', CV_BASE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 



; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; This is the event handler for CV Select - MISCELLANEOUS
PRO CV_SELECT_MISC_EVENT, EVENT

@wgsp_cb.pro
 
 
NO_OR_YES = ['No', 'Yes']

TYPE = TAG_NAMES(EVENT, /STRUCTURE)
CASE TYPE OF

 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ; The list widget events
 'WIDGET_LIST': BEGIN
     WIDGET_CONTROL, EVENT.ID, GET_UVALUE = UVALUE
     CASE STRUPCASE(UVALUE(EVENT.INDEX)) OF

          'MAXIMUM ITERATIONS FOR CONVERGENCE': BEGIN
                   RESULT = CV_DIALOG('CV_MAX_TRIES', $
                            STRTRIM(CV_MAX_TRIES, 2), /TEXT) 
                   CV_MAX_TRIES = LONG(RESULT)
                   END 

          'MFIT DEBUG MODE': BEGIN
                   IF (CV_DEBUG) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_DEBUG', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_DEBUG = LONG(RESULT)
                   END  

          'MFIT VERBOSE MODE': BEGIN
                   TEMP = CV_TALKY
                   SELECTIONS = ['-1', '0', '1', '2', '3']
                   W_SELECTIONS = ['Suppress Warnings (-1)', $
                                   'No Extra Output (0)', $
                                   'ChiSq Value per Iteration (1)', $
                                   '+Parameter Values per Iteration (2)', $
                                   '+Extra Info After Each Fit (3)']
                   RESULT = CV_DIALOG('CV_TALKY', $
                            W_SELECTIONS, TEMP, /RADIO) 
                   CV_TALKY = LONG(SELECTIONS(RESULT))
                   END          
          
          'MFIT EXPERIENCED USER': BEGIN
                   IF (CV_EXP_USER) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_EXP_USER', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_EXP_USER = LONG(RESULT)
                   END  

          'COUTOUR LEVEL': BEGIN
                  CASE STRTRIM(CV_CONTOUR_CHOICE, 2) OF 
                        'A': TEMP = 0
                        'P': TEMP = 1
                        'S': TEMP = 2
                        ELSE: BEGIN
                              PRINT, 'ERROR: W_CVSELECTOR'
                              HELP, CV_CONTOUR_CHOICE
                              END
                   ENDCASE
                   SELECTIONS = ['A', 'P', 'S']
                   W_SELECTIONS = ['Ask (A)', 'Percentage (P)', 'Sigma (S)']
                   RESULT = CV_DIALOG('CV_CONTOUR_CHOICE', $
                            W_SELECTIONS, TEMP, /RADIO) 
                   STRPUT, CV_CONTOUR_CHOICE, STRUPCASE(SELECTIONS(RESULT))
                   END                    
          
          'PRINT CHI-SQUARE DURING FIT': BEGIN
                   IF (CV_WRITE_CONTOUR_CHI_STEPS) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_WRITE_CONTOUR_CHI_STEPS', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_WRITE_CONTOUR_CHI_STEPS = LONG(RESULT)
                   END  

          'AUTOMATICALLY GENERATE FIT COMMENTS': BEGIN
                   IF (CV_AUTO_COMMENT) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_AUTO_COMMENT', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_AUTO_COMMENT = LONG(RESULT)
                   END  

          'DRM LOWER INPUT PHOTON BIN': BEGIN
                   RESULT = CV_DIALOG('CV_FIRST_BIN_CALC', $
                            STRTRIM(CV_FIRST_BIN_CALC, 2), /TEXT) 
                   CV_FIRST_BIN_CALC = LONG(RESULT)
                   END 

          'DRM UPPER INPUT PHOTON BIN': BEGIN
                   RESULT = CV_DIALOG('CV_LAST_BIN_CALC', $
                            STRTRIM(CV_LAST_BIN_CALC, 2), /TEXT) 
                   CV_LAST_BIN_CALC = LONG(RESULT)
                   END 

          'BATCH FIT SUMMARY OUTPUT': BEGIN
                   TEMP = CV_BATCH_FIT_SUMMARY  
                   SELECTIONS = ['0', '1', '2']
                   W_SELECTIONS = ['ChiSq Only (0)', $
                                   '+Parameter Values (1)', $
                                   '+Convariance Info (2)']
                   RESULT = CV_DIALOG('CV_BATCH_FIT_SUMMARY', $
                            W_SELECTIONS, TEMP, /RADIO) 
                   CV_BATCH_FIT_SUMMARY = LONG(RESULT)
                   END       

          'GAUSS OFFSET': BEGIN
                   RESULT = CV_DIALOG('CV_GAUSS_OFFSET', $
                            STRTRIM(CV_GAUSS_OFFSET, 2), /TEXT) 
                   CV_GAUSS_OFFSET = LONG(RESULT)
                   END           
          
          'GAUSS START ENERGY (KEV)': BEGIN
                   RESULT = CV_DIALOG('CV_START_GAUSS_E', $
                            STRTRIM(CV_START_GAUSS_E, 2), /TEXT) 
                   CV_START_GAUSS_E = LONG(RESULT)
                   END           
          
          'GAUSS END ENERGY (KEV)': BEGIN
                   RESULT = CV_DIALOG('CV_END_GAUSS_E', $
                            STRTRIM(CV_END_GAUSS_E, 2), /TEXT) 
                   CV_END_GAUSS_E = LONG(RESULT)
                   END           
          
          'GAUSS ENERGY STEP SIZE': BEGIN
                   RESULT = CV_DIALOG('CV_GAUSS_STEP_SIZE', $
                            STRTRIM(CV_GAUSS_STEP_SIZE, 2), /TEXT) 
                   CV_GAUSS_STEP_SIZE = LONG(RESULT)
                   END 

          'VARY GAUSSIAN LINE WIDTH': BEGIN
                   IF (CV_GAUSS_VWIDTH) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_GAUSS_VWIDTH', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_GAUSS_VWIDTH = LONG(RESULT)
                   END  

          'AUTOMATICALLY FIX UNDETERMINED PARAMETERS': BEGIN
                  IF (CV_FIX_UNDETERMINED) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_FIX_UNDETERMINED', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_FIX_UNDETERMINED = LONG(RESULT)
                   END  

          'SAME AXIS LIMITS FOR ALL DETECTORS': BEGIN
                   IF (CV_AXES_SAME) THEN TEMP = 1 ELSE TEMP = 0
                   RESULT = CV_DIALOG('CV_AXES_SAME', $
                            NO_OR_YES, TEMP, /RADIO) 
                   CV_AXES_SAME = LONG(RESULT)
                   END  

     ENDCASE ; for UVALUE
     END        
 

 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ; The button widget events
 'WIDGET_BUTTON': BEGIN
    WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE
                   
    VALUE = STRUPCASE(VALUE)
    CASE 1 OF 

         VALUE EQ 'HELP': WM_XDISPLAYFILE, $
                          WGSP_SOURCE + STRLOWCASE('MFIT_CONTROL_DECLARE.INCL')

         VALUE EQ 'DONE': BEGIN

               IF (STRTRIM(STRLOWCASE(MFIT_CONTROL_FILENAME), 2) EQ $
                  STRTRIM(STRLOWCASE(WGSP_SOURCE + 'MFIT_CONTROL.DAT'), 2)) $
                  THEN BEGIN
   
                  FILE_ERR = WIDMENU(['Create mfit_control.dat in WING_SCRATCH?', $
                                     'Yes', 'No'], TITLE = 0)
                  IF (FILE_ERR EQ 1) THEN BEGIN
                     MFIT_CONTROL_FILENAME = STRING(REPLICATE(32B, 128)) 
                     STRPUT, MFIT_CONTROL_FILENAME, $ 
                             WGSP_SCRATCH + STRLOWCASE('MFIT_CONTROL.DAT')
                  
                     OPENW, FL_U, STRTRIM(WGSP_SCRATCH + $
                            STRLOWCASE('MFIT_CONTROL.DAT'), 2), /GET_LUN
                     FREE_LUN, FL_U
                  ENDIF ELSE BEGIN
                     WIDGET_CONTROL, EVENT.TOP, /DESTROY
                     RETURN
                  ENDELSE
               ENDIF

               WRITE_CONTROL, STRTRIM(MFIT_CONTROL_FILENAME, 2)
               INFORM, 'Wrote ' + STRTRIM(MFIT_CONTROL_FILENAME, 2)
               GET_CONTROL
               WIDGET_CONTROL, EVENT.TOP, /DESTROY
               END

         VALUE EQ 'APPLY': BEGIN

               IF (STRTRIM(STRLOWCASE(MFIT_CONTROL_FILENAME), 2) EQ $
                  STRTRIM(STRLOWCASE(WGSP_SOURCE + 'MFIT_CONTROL.DAT'), 2)) $
                  THEN BEGIN
   
                  FILE_ERR = WIDMENU(['Create mfit_control.dat in WING_SCRATCH?', $
                                     'Yes', 'No'], TITLE = 0)
                  IF (FILE_ERR EQ 1) THEN BEGIN
                     MFIT_CONTROL_FILENAME = STRING(REPLICATE(32B, 128)) 
                     STRPUT, MFIT_CONTROL_FILENAME, $ 
                             WGSP_SCRATCH + STRLOWCASE('MFIT_CONTROL.DAT')
                  
                     OPENW, FL_U, STRTRIM(WGSP_SCRATCH + $
                            STRLOWCASE('MFIT_CONTROL.DAT'), 2), /GET_LUN
                     FREE_LUN, FL_U
                  ENDIF ELSE BEGIN
                     WIDGET_CONTROL, EVENT.TOP, /DESTROY
                     RETURN
                  ENDELSE
               ENDIF

               WRITE_CONTROL, STRTRIM(MFIT_CONTROL_FILENAME, 2)
               INFORM, 'Wrote ' + STRTRIM(MFIT_CONTROL_FILENAME, 2)
               GET_CONTROL
               END
 
    ENDCASE ; for VALUE
    END


 
ENDCASE ; for TYPE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
PRO CV_SELECT_MISC

@wgsp_cb.pro
 
 
GET_CONTROL

IF (STRTRIM(STRLOWCASE(WGSP_SCRATCH), 2) EQ $
   STRTRIM(STRLOWCASE(WGSP_SOURCE), 2)) $
   THEN BEGIN
   FILE_ERR = WIDMENU(['Cannot write mfit_control.dat in WING_SOURCE', $
                       'OK'], TITLE = 0)
   RETURN
ENDIF


; Parent
MTITLE = 'CV Selector - MISCELLANEOUS'
CV_BASE = WIDGET_BASE(TITLE = MTITLE, /COLUMN, MAP = 0)

DBUTTON = WIDGET_BUTTON(CV_BASE, VALUE = 'DONE')
ABUTTON = WIDGET_BUTTON(CV_BASE, VALUE = 'Apply')
HBUTTON = WIDGET_BUTTON(CV_BASE, VALUE = 'Help')
LABEL   = WIDGET_LABEL(CV_BASE, VALUE = 'Use <enter> After Text Entries')

CONTROL_VAR = ['Maximum Iterations for Convergence', $
               'MFIT Debug Mode', $
               'MFIT Verbose Mode', $
               'MFIT Experienced User', $
               'Coutour Level', $
               'Print Chi-square During Fit', $
               'Automatically Generate Fit Comments', $
               'DRM Lower Input Photon Bin', $
               'DRM Upper Input Photon Bin', $
               'Batch Fit Summary Output', $
               'Gauss Offset', $
               'Gauss Start Energy (keV)', $
               'Gauss End Energy (keV)', $
               'Gauss Energy Step Size', $
               'Vary Gaussian Line Width', $
               'Automatically Fix Undetermined Parameters', $
               'Same Axis Limits for All Detectors']
 
CONTROL_VAR = CONTROL_VAR(SORT(CONTROL_VAR))

LIST = WIDGET_LIST(CV_BASE, VALUE = CONTROL_VAR, $
                  YSIZE = 10, UVALUE = CONTROL_VAR) 

; Realize the BASE widget and the children
WIDGET_CONTROL, CV_BASE, /REALIZE, SHOW = 1 

; Place the widget in center of screen
DEVICE, GET_SCREEN_SIZE = CURRENT_SCREEN
WIDGET_CONTROL, CV_BASE, TLB_GET_SIZE = MENU_SIZE
MENU_CENTER = [(CURRENT_SCREEN(0) / 2.0) - (MENU_SIZE(0) / 2.0), $ 
               (CURRENT_SCREEN(1) / 2.0) - (MENU_SIZE(1) / 2.0)] 
WIDGET_CONTROL, CV_BASE, $
                TLB_SET_XOFFSET = MENU_CENTER(0), $
                TLB_SET_YOFFSET = MENU_CENTER(1)
WIDGET_CONTROL, CV_BASE, MAP = 1

; Place the cursor 
WIDGET_CONTROL, DBUTTON, /INPUT_FOCUS

; Register with XMANAGER
XMANAGER, 'CV_SELECT_MISC', CV_BASE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
