; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; Reads MFIT_CONTROL.DAT and sets the default values of the control 
;  variables and any non-default values set in MFIT_CONTROL.DAT
;  If keyword GET_DEFAULTS is set, assigns the default values, else just
;  reads MFIT_CONTROL.DAT
;
PRO GET_CONTROL, GET_DEFAULTS = GET_DEFAULTS

COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

; WINGSPAN IDL source code location and other parameters:
COMMON SOURCE_LOC, WGSP_SOURCE, WGSP_PRINTER, WGSP_SCRATCH, $
       WGSP_FITS, WGSP_LOOKUP, WGSP_DRM, WGSP_NO_MFIT, $
       WGSP_SCALE, WGSP_PRINT_COMMAND, WGSP_CONFIRM, WGSP_HELP, $
       WGSP_WORK, MFIT_CONTROL_FILENAME, MFIT_FUNCINFO_FILENAME


; Trap file errors
ON_IOERROR, FILE_ERROR

; The default values
IF (N_ELEMENTS(GET_DEFAULTS) NE 0) THEN BEGIN

   CV_DEBUG = LONG(0)
   CV_TALKY = LONG(1)
   CV_EXP_USER = LONG(0)
   CV_CONTOUR_CHOICE = STRING(FORMAT= '(A1)', 'A')
   CV_WRITE_CONTOUR_CHI_STEPS = LONG(0)
   CV_AUTO_COMMENT = LONG(1)
   CV_FIRST_BIN_CALC = LONG(0)
   CV_LAST_BIN_CALC = LONG(9999)
   CV_BATCH_FIT_SUMMARY = LONG(0)
   CV_GAUSS_OFFSET = 1.5
   CV_START_GAUSS_E = 0.0
   CV_END_GAUSS_E = 100.0
   CV_GAUSS_STEP_SIZE = 0.333333
   CV_GAUSS_VWIDTH = LONG(0)
   CV_FIX_UNDETERMINED = LONG(1)
   CV_AXES_SAME = LONG(1)

   CV_PRINTER_LANG = STRING(REPLICATE(32B, 25))
   STRPUT, CV_PRINTER_LANG, 'PS'
   CV_PLOT_MODEL_EXTRAP = LONG(1)
   CV_PLOT_EACH_TERM = LONG(1)
   CV_ABS_MODEL = LONG(1)
   CV_PLOT_ONLY_DATA = LONG(0)
   CV_SUPPRESS_2ND_LABELS = LONG(0)
   CV_MULTI_DET_PLOT_MODE = STRING(FORMAT= '(A1)', 'C')
   CV_GRAPH_LIMIT_MODE = STRING(FORMAT= '(A1)', 'B')
   CV_HCOPY_FILE_STATUS = STRING(REPLICATE(32B, 25))
   STRPUT, CV_HCOPY_FILE_STATUS, 'DELETE'
   CV_ENCAPSULATED_PS = LONG(0)
   CV_SPEC_STYLE = LONG(40)

   CV_RATE_MODE = STRING(FORMAT= '(A1)', 'E')
   CV_TRACK_MOUSE = LONG(1)
   CV_PLOT_UNDETERMINED = LONG(1)
   CV_PLOT_CURSOR = LONG(0)
   CV_IDL_RETAIN = LONG(2)
   CV_TITLE_STYLE = STRING(FORMAT= '(A1)', 'N')
   CV_SPARE_1 = LONG(0)
   CV_SPARE_2 = LONG(0)
   CV_SPARE_3 = LONG(0)

   CV_KEEP_ASCII_OPEN = LONG(0)
   CV_ASCII_OUT_FILE = STRING(REPLICATE(32B, 80))
   CV_STARTUP_TEMPLATE_FILE = STRING(REPLICATE(32B, 80))
   CV_FULL_BATCH_OUT = LONG(1)
   CV_SUPPRESS_BATCH_OUT = LONG(1)
   CV_BATCH_OUT_FILE = STRING(REPLICATE(32B, 80))
   CV_ALWAYS_WA = LONG(0)
   CV_ALWAYS_WB = LONG(0)

   CV_REL_CONVERG = 0.001
   CV_ABS_CONVERG = 0.01
   CV_MAX_TRIES = LONG(90)

   RETURN
ENDIF


; Not reading default control variables; check to see if 
;  user has custom MFIT_CONTROL_FILE

IF (STRTRIM(STRLOWCASE(MFIT_CONTROL_FILENAME), 2) EQ $
   STRTRIM(STRLOWCASE(WGSP_SOURCE + 'MFIT_CONTROL.DAT'), 2)) THEN BEGIN
   GET_CONTROL, /GET_DEFAULTS
   RETURN
ENDIF ; Else using custom control file

FILE_CHECK = FINDFILE(STRTRIM(MFIT_CONTROL_FILENAME, 2))
IF (FILE_CHECK(0) EQ '') THEN BEGIN
   OPENW, FL_U, STRTRIM(MFIT_CONTROL_FILENAME, 2), /GET_LUN
   FREE_LUN, FL_U
   PRINT, '*** WARNING:  GET_CONTROL: Control file not found: '
   PRINT, '   ', STRTRIM(MFIT_CONTROL_FILENAME, 2)
   CREATE_FILE = WIDMENU(['Created mfit_control.dat in WING_SCRATCH', $
                         'OK'], TITLE = 0)
   WRITE_CONTROL, STRTRIM(MFIT_CONTROL_FILENAME, 2)
   RETURN
ENDIF

OPENR, FL, STRTRIM(MFIT_CONTROL_FILENAME, 2), /GET_LUN

       STR = '' & STR_ARR = ''
       WHILE NOT EOF(FL) DO BEGIN
             READF, FL, STR
             STR_ARR = [STR_ARR, STR]
       ENDWHILE
       STR_ARR = STR_ARR(1:*)
FREE_LUN, FL

BEGIN_INDEX = WHERE(STRTRIM(STR_ARR, 2) EQ '&MFIT_CONTROL', BEGIN_CNT)
END_INDEX = WHERE(STRTRIM(STR_ARR, 2) EQ '&END', END_CNT)

IF (BEGIN_CNT EQ 0 OR END_CNT EQ 0) THEN BEGIN
   PRINT, '*** ERROR: GET_CONTROL: Error reading ' + MFIT_CONTROL_FILE_NAME
   PRINT, '    Using default control variables.'
   GET_CONTROL, /GET_DEFAULTS
   RETURN
ENDIF
       
CONTROL_STR_ARR = STR_ARR(BEGIN_INDEX(0): END_INDEX(0))
       
; Now have the new control variables in the string 
;  array CONTROL_STR_ARR.  Extract them and set the new value.
NUM_CHANGES = N_ELEMENTS(CONTROL_STR_ARR) - 2
FOR I=0, NUM_CHANGES - 1 DO BEGIN

    COMMAND = CONTROL_STR_ARR(I + 1)
    EQUAL = STRPOS(COMMAND, '=')
    IF (EQUAL EQ -1 ) THEN GOTO, SKIP
    
    VARIABLE = STRTRIM(STRMID(COMMAND, 0, EQUAL - 1), 2)
    VALUE    = STRTRIM(STRMID(COMMAND, EQUAL + 1, 999), 2) 

    ; Strip off literal quotes
    QUOTE = STRPOS(VALUE, "'")
    IF (QUOTE NE -1) THEN VALUE = STRMID(VALUE, 1, STRLEN(VALUE) - 2)

    IF (STRUPCASE(VALUE) EQ '.TRUE.') THEN VALUE = '1'
    IF (STRUPCASE(VALUE) EQ '.FALSE.') THEN VALUE = '0'

    ; Now set the correct control variable
    CASE STRUPCASE(VARIABLE) OF
           
    'MCV_DEBUG': CV_DEBUG = LONG(VALUE)
    'MCV_TALKY': CV_TALKY = LONG(VALUE)
    'MCV_EXP_USER': CV_EXP_USER = LONG(VALUE)
    'MCV_CONTOUR_CHOICE': CV_CONTOUR_CHOICE = STRING(VALUE, FORMAT= '(A1)')
    'MCV_WRITE_CONTOUR_CHI_STEPS': CV_WRITE_CONTOUR_CHI_STEPS = LONG(VALUE)
    'MCV_AUTO_COMMENT': CV_AUTO_COMMENT = LONG(VALUE)
    'MCV_FIRST_BIN_CALC': CV_FIRST_BIN_CALC = LONG(VALUE)
    'MCV_LAST_BIN_CALC': CV_LAST_BIN_CALC = LONG(VALUE)
    'MCV_BATCH_FIT_SUMMARY': CV_BATCH_FIT_SUMMARY = LONG(VALUE)
    'MCV_GAUSS_OFFSET': CV_GAUSS_OFFSET = FLOAT(VALUE)
    'MCV_START_GAUSS_E': CV_START_GAUSS_E = FLOAT(VALUE)
    'MCV_END_GAUSS_E': CV_END_GAUSS_E = FLOAT(VALUE)
    'MCV_GAUSS_STEP_SIZE': CV_GAUSS_STEP_SIZE = FLOAT(VALUE)
    'MCV_GAUSS_VWIDTH': CV_GAUSS_VWIDTH = LONG(VALUE)
    'MCV_FIX_UNDETERMINED': CV_FIX_UNDETERMINED = LONG(VALUE)
    'MCV_AXES_SAME': CV_AXES_SAME = LONG(VALUE)
     
    'MCV_PRINTER_LANG': BEGIN
                        CV_PRINTER_LANG = STRING(REPLICATE(32B, 25))
                        STRPUT, CV_PRINTER_LANG, VALUE
                        END
    'MCV_PLOT_MODEL_EXTRAP': CV_PLOT_MODEL_EXTRAP = LONG(VALUE)
    'MCV_PLOT_EACH_TERM': CV_PLOT_EACH_TERM = LONG(VALUE)
    'MCV_ABS_MODEL': CV_ABS_MODEL = LONG(VALUE)
    'MCV_PLOT_ONLY_DATA': CV_PLOT_ONLY_DATA = LONG(VALUE)
    'MCV_SUPPRESS_2ND_LABELS': CV_SUPPRESS_2ND_LABELS = LONG(VALUE)
    'MCV_MULTI_DET_PLOT_MODE': CV_MULTI_DET_PLOT_MODE = STRING(VALUE, FORMAT= '(A1)')
    'MCV_GRAPH_LIMIT_MODE': CV_GRAPH_LIMIT_MODE = STRING(VALUE, FORMAT= '(A1)')
    'MCV_HCOPY_FILE_STATUS': BEGIN
                             CV_HCOPY_FILE_STATUS = STRING(REPLICATE(32B, 25))
                             STRPUT, CV_HCOPY_FILE_STATUS, VALUE
                             END
    'MCV_ENCAPSULATED_PS': CV_ENCAPSULATED_PS = LONG(VALUE)
    'MCV_SPEC_STYLE': CV_SPEC_STYLE = LONG(VALUE)

    'MCV_RATE_MODE': CV_RATE_MODE = STRING(VALUE, FORMAT= '(A1)')
    'MCV_TRACK_MOUSE': CV_TRACK_MOUSE = LONG(VALUE)
    'MCV_PLOT_UNDETERMINED': CV_PLOT_UNDETERMINED = LONG(VALUE)
    'MCV_PLOT_CURSOR': CV_PLOT_CURSOR = LONG(VALUE)
    'MCV_IDL_RETAIN': CV_IDL_RETAIN = LONG(VALUE)
    'MCV_TITLE_STYLE': CV_TITLE_STYLE = STRING(VALUE, FORMAT= '(A1)')
    'MCV_SPARE_1': CV_SPARE_1 = LONG(VALUE)
    'MCV_SPARE_2': CV_SPARE_2 = LONG(VALUE)
    'MCV_SPARE_3': CV_SPARE_3 = LONG(VALUE)
     
    'MCV_KEEP_ASCII_OPEN': CV_KEEP_ASCII_OPEN = LONG(VALUE)
    'MCV_ASCII_OUT_FILE': BEGIN
                          CV_ASCII_OUT_FILE = STRING(REPLICATE(32B, 80))
                          STRPUT, CV_ASCII_OUT_FILE, VALUE
                          END
    'MCV_STARTUP_TEMPLATE_FILE': BEGIN
                                 CV_STARTUP_TEMPLATE_FILE = STRING(REPLICATE(32B, 80))
                                 STRPUT, CV_STARTUP_TEMPLATE_FILE, VALUE
                                 END
    'MCV_FULL_BATCH_OUT': CV_FULL_BATCH_OUT = LONG(VALUE)
    'MCV_SUPPRESS_BATCH_OUT': CV_SUPPRESS_BATCH_OUT = LONG(VALUE)
    'MCV_BATCH_OUT_FILE': BEGIN
                          CV_BATCH_OUT_FILE = STRING(REPLICATE(32B, 80))
                          STRPUT, CV_BATCH_OUT_FILE, VALUE
                          END
    'MCV_ALWAYS_WA': CV_ALWAYS_WA = LONG(VALUE)
    'MCV_ALWAYS_WB': CV_ALWAYS_WB = LONG(VALUE)
     
    'MCV_REL_CONVERG': CV_REL_CONVERG = FLOAT(VALUE)
    'MCV_ABS_CONVERG': CV_ABS_CONVERG = FLOAT(VALUE)
    'MCV_MAX_TRIES': CV_MAX_TRIES = LONG(VALUE)
                  
    ELSE: BEGIN
          PRINT, '*** ERROR: GET_CONTROL: No such control variable: ' + VALUE
          PRINT, 'Skipping...(edit mfit_control.dat)'
          END

    ENDCASE

SKIP:
ENDFOR

RETURN

FILE_ERROR:
FILE_ERR = WIDMENU([!ERR_STRING + ' Using defaults.', 'OK'], TITLE = 0)
STRPUT, MFIT_CONTROL_FILENAME, WGSP_SOURCE + STRLOWCASE('MFIT_CONTROL.DAT')
GET_CONTROL, /GET_DEFAULTS

        

END ; PRO GET_CONTROL
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
