; MAIN WINGSPAN WIDGET AND EVENT PROCESSING PROCEDURES 
; R.S. Mallozzi, UAH/NASA, Dec, 1994



; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; This is the event handler for the main WINGSPAN widget
PRO TOP_BASE_EVENT, EVENT

@wgsp_cb.pro
; Following COMMON internal to the main widget routine and its event handler
COMMON NUMBER_DATA, NUM_ENTRIES, NUM_NOTES 



TYPE = TAG_NAMES(EVENT, /STRUCTURE)
CASE TYPE OF

 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ; The text widget events
 'WIDGET_TEXT_CH': BEGIN
     WIDGET_CONTROL, EVENT.ID, GET_UVALUE = UVALUE, GET_VALUE = VALUE
     CASE UVALUE OF 
         6: BEGIN
             ; Get the FITS file 
             WM_FILE_ENTRY, /FITS, VALUE, FILENAME
             IF (FILENAME EQ '') THEN BEGIN
                START_STATUS = 0
                GOTO, NFILE
             ENDIF

             BREAK_FILE, FILENAME, DEV, DIRECT, FNAME, FEXT, FVERSION

             ; Break up the user-supplied filter for the lookup file
             BREAK_FILE, WGSP_LOOKUP, LDEV, LDIRECT, LFNAME, LFEXT, LFVERSION
             LUFILE = DEV + DIRECT + STRLOWCASE(FNAME + LFEXT)

             ; Read FITS file and plot data
             WIDGET_CONTROL, GET_VALUE = H_WINDOW, WIDS.DRAW1
             WIDGET_CONTROL, GET_VALUE = S_WINDOW, WIDS.DRAW2
             INFORM, 'Reading ' + STRTRIM(FILENAME,2)
             WM_START_SESSION, FILENAME, H_WINDOW, S_WINDOW, START_STATUS
             IF (START_STATUS EQ 0) THEN BEGIN
                ; Cancel
                GOTO, NFILE                     
             ENDIF ELSE BEGIN
                IF (CV_RATE_MODE EQ 'R') THEN BEGIN
                   CURRENT_VALUES(5) = 'Counts s!U-1!N'
                ENDIF ELSE BEGIN
                   CURRENT_VALUES(5) = 'Counts s!U-1!N keV!U-1!N'
                ENDELSE
             ENDELSE

             ; Auto read LOOKUP file, if set
             CHECK = FINDFILE(LUFILE, COUNT = COUNT)
             IF (AUTO_READ EQ 1) THEN BEGIN
                IF (COUNT EQ 0) THEN BEGIN
                   INFORM, 'LOOKUP file not found.  Updating menu.'
                   INITIALIZE_PLOTS
                   INFORM, 'Ready.'
                ENDIF ELSE BEGIN
                   HAVE_LOOKUP = 1
                   READ_LOOKUPS, LUFILE, DUMMY, $
                                 CURRENT_VALUES(1), CURRENT_VALUES(2), BAD_TABLE
                   IF (BAD_TABLE NE 1) THEN BEGIN
                      IF CURRENT_VALUES(1) EQ 'LOG' THEN TVAL = 0 ELSE TVAL = 0
                      WIDGET_CONTROL, WIDS.EBUTTONS, SET_VALUE = TVAL
                      IF CURRENT_VALUES(2) EQ 'LOG' THEN TVAL = 0 ELSE TVAL = 0
                      WIDGET_CONTROL, WIDS.RBUTTONS, SET_VALUE = TVAL

                      INFORM, 'Lookup file is ' + STRTRIM(LUFILE, 2) + '.  Ready.'
                   ENDIF ELSE BEGIN
                    INITIALIZE_PLOTS
                   ENDELSE
                ENDELSE
             ENDIF ELSE INFORM, 'Ready.'

             ; Update menu 
             WIDGET_CONTROL, WIDS.LIST(2), SET_VALUE = FILENAME
             WIDGET_CONTROL, WIDS.LIST(3), SET_VALUE = LUFILE
             CURRENT_VALUES(6) = FILENAME
             IF (COUNT EQ 0) THEN BEGIN
                CURRENT_VALUES(7) = 'none'
             ENDIF ELSE BEGIN  
                CURRENT_VALUES(7) = LUFILE
             ENDELSE

             ; Update the object name
             BURST_NAME = STRTRIM(SXPAR(HEADER, 'OBJECT'), 2)
             IF (BURST_NAME EQ 'BB123') OR (CV_TITLE_STYLE EQ 'T') THEN BEGIN
                ; Handle old file
                II = GET_NOTE('Burst')
                BURST_NAME = STRTRIM(NOTES(II), 2)
             ENDIF 
             CURRENT_VALUES(8) = BURST_NAME

              ; Update the notes
             TEMP_NOTES    = STRARR(NUM_NOTES) 
             TEMP_NOTES(0) = NOTES(0)
             TEMP_NOTES(1) = NOTES(3) + '    ' + NOTES(4)
             TEMP_NOTES(2) = NOTES(2)
             TEMP_NOTES(3) = 'Detector type: ' + $
                  STRTRIM(SXPAR(DH, 'DET_MODE'), 2) + '    ' + NOTES(1)
             TEMP_NOTES(4) = 'Current printer: ' + STRTRIM(WGSP_PRINTER, 2)
             FOR I=0, NUM_NOTES-1 DO BEGIN
                 WIDGET_CONTROL, WIDS.M_NOTES(I), SET_VALUE = TEMP_NOTES(I)
             ENDFOR

NFILE:
            IF (FILENAME EQ '') OR (START_STATUS EQ 0) THEN $
               WIDGET_CONTROL, WIDS.LIST(2), /INPUT_FOCUS
            END
         7: BEGIN
             ; Get the LOOKUP file 
             WM_LOOKUP, VALUE, LFILENAME, BAD_TABLE
             IF (LFILENAME NE '') THEN BEGIN
                WIDGET_CONTROL, WIDS.LIST(3), SET_VALUE = LFILENAME
                CURRENT_VALUES(7) = LFILENAME
                BREAK_FILE, LFILENAME, LDEV, LDIRECT, LFNAME, LFEXT, LFVERSION
                IF (BAD_TABLE NE 1) THEN $
                   INFORM, 'Lookup file is ' + $
                           STRTRIM(LFNAME + LFEXT,2) + '.  Ready.'
             ENDIF
            END
     ENDCASE ; for UVALUE
     END
 
 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ; The button widget events
 'WIDGET_BUTTON': BEGIN
    WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE, GET_UVALUE = UVALUE
                   
    VALUE = STRUPCASE(VALUE)
    CASE 1 OF 
         ; Spectral History
         VALUE EQ 'STACKED HISTORIES': BEGIN
               ; Make sure data has been read
               WARN_NO_DATA, DATA_STATUS
               IF (DATA_STATUS EQ 'NO DATA') THEN RETURN

               CURRENT_VALUES(0) = VALUE
               COUNT_SPAN, COMBINED_ENERGIES, ENERGY_SPAN, PLOT_IND 
               WM_SPECTRAL_HISTORY, PLOT_IND, CURRENT_VALUES(0)
               END
         VALUE EQ 'STACKED SPECTRA': BEGIN
               ; Make sure data has been read
               WARN_NO_DATA, DATA_STATUS
               IF (DATA_STATUS EQ 'NO DATA') THEN RETURN

               CURRENT_VALUES(0) = VALUE
               COUNT_SPAN, COMBINED_TIMES, TIME_SPAN, PLOT_IND 
               WM_SPECTRAL_HISTORY, PLOT_IND, CURRENT_VALUES(0)
               END
         VALUE EQ 'CONTOUR': BEGIN
               ; Make sure data has been read
               WARN_NO_DATA, DATA_STATUS
               IF (DATA_STATUS EQ 'NO DATA') THEN RETURN

               CURRENT_VALUES(0) = VALUE
               PLOT_IND = 0   
               WM_SPECTRAL_HISTORY, PLOT_IND, CURRENT_VALUES(0)
               END
         VALUE EQ 'SURFACE': BEGIN
               ; Make sure data has been read
               WARN_NO_DATA, DATA_STATUS
               IF (DATA_STATUS EQ 'NO DATA') THEN RETURN

               CURRENT_VALUES(0) = VALUE
               PLOT_IND = 0   
               WM_SPECTRAL_HISTORY, PLOT_IND, CURRENT_VALUES(0)
               END
         VALUE EQ 'HIDDEN LINE': BEGIN
               ; Make sure data has been read
               WARN_NO_DATA, DATA_STATUS
               IF (DATA_STATUS EQ 'NO DATA') THEN RETURN

               CURRENT_VALUES(0) = VALUE
               PLOT_IND = 0   
               WM_SPECTRAL_HISTORY, PLOT_IND, CURRENT_VALUES(0)
               END

         VALUE EQ 'FITS FILENAME': BEGIN
               ; Get the FITS file 
               WM_FILE_ENTRY, /FITS, WGSP_FITS, FILENAME
               IF (FILENAME EQ '') THEN BEGIN
                  START_STATUS = 0
                  GOTO, NOFILE
               ENDIF

               BREAK_FILE, FILENAME, DEV, DIRECT, FNAME, FEXT, FVERSION

               ; Break up the user-supplied filter for the lookup file
               BREAK_FILE, WGSP_LOOKUP, LDEV, LDIRECT, LFNAME, LFEXT, LFVERSION
               LUFILE = DEV + DIRECT + STRLOWCASE(FNAME + LFEXT)

               ; Read FITS file and plot data
               WIDGET_CONTROL, GET_VALUE = H_WINDOW, WIDS.DRAW1
               WIDGET_CONTROL, GET_VALUE = S_WINDOW, WIDS.DRAW2
               INFORM, 'Reading ' + STRTRIM(FILENAME,2)
               WM_START_SESSION, FILENAME, H_WINDOW, S_WINDOW, START_STATUS
               IF (START_STATUS EQ 0) THEN BEGIN
                  ; Cancel
                  GOTO, NOFILE                     
               ENDIF ELSE BEGIN
                  IF (CV_RATE_MODE EQ 'R') THEN BEGIN
                     CURRENT_VALUES(5) = 'Counts s!U-1!N'
                  ENDIF ELSE BEGIN
                     CURRENT_VALUES(5) = 'Counts s!U-1!N keV!U-1!N'
                  ENDELSE
               ENDELSE

               ; Auto read LOOKUP file, if set
               CHECK = FINDFILE(LUFILE, COUNT = COUNT)
               IF (AUTO_READ EQ 1) THEN BEGIN
                  IF COUNT EQ 0 THEN BEGIN
                     INFORM, 'LOOKUP file not found.  Updating menu.'
                     INITIALIZE_PLOTS
                     INFORM, 'Ready.'
                  ENDIF ELSE BEGIN
                     HAVE_LOOKUP = 1
                     READ_LOOKUPS, LUFILE, DUMMY, $
                                 CURRENT_VALUES(1), CURRENT_VALUES(2), BAD_TABLE
                     IF (BAD_TABLE NE 1) THEN BEGIN
                        IF CURRENT_VALUES(1) EQ 'LOG' THEN TVAL = 0 ELSE TVAL = 0
                        WIDGET_CONTROL, WIDS.EBUTTONS, SET_VALUE = TVAL
                        IF CURRENT_VALUES(2) EQ 'LOG' THEN TVAL = 0 ELSE TVAL = 0
                        WIDGET_CONTROL, WIDS.RBUTTONS, SET_VALUE = TVAL

                        INFORM, 'Lookup file is ' + STRTRIM(LUFILE, 2) + '.  Ready.'
                     ENDIF ELSE BEGIN
                       INITIALIZE_PLOTS
                     ENDELSE
                  ENDELSE
               ENDIF ELSE INFORM, 'Ready.'

               ; Update menu 
               WIDGET_CONTROL, WIDS.LIST(2), SET_VALUE = FILENAME
               WIDGET_CONTROL, WIDS.LIST(3), SET_VALUE = LUFILE
               CURRENT_VALUES(6) = FILENAME
               IF (COUNT EQ 0) THEN BEGIN
                  CURRENT_VALUES(7) = 'none'
               ENDIF ELSE BEGIN  
                  CURRENT_VALUES(7) = LUFILE
               ENDELSE

               ; Update the object name
               BURST_NAME = STRTRIM(SXPAR(HEADER, 'OBJECT'), 2)
               IF (BURST_NAME EQ 'BB123') OR (CV_TITLE_STYLE EQ 'T') THEN BEGIN
                  ; Handle old file
                  II = GET_NOTE('Burst')
                  BURST_NAME = STRTRIM(NOTES(II), 2)
               ENDIF 
               CURRENT_VALUES(8) = BURST_NAME

               ; Update the notes
               TEMP_NOTES    = STRARR(NUM_NOTES) 
               TEMP_NOTES(0) = NOTES(0)
               TEMP_NOTES(1) = NOTES(3) + '    ' + NOTES(4)
               TEMP_NOTES(2) = NOTES(2)
               TEMP_NOTES(3) = 'Detector type: ' + $
                    STRTRIM(SXPAR(DH, 'DET_MODE'), 2) + '    ' + NOTES(1)
               TEMP_NOTES(4) = 'Current printer: ' + STRTRIM(WGSP_PRINTER, 2)
               FOR I=0, NUM_NOTES-1 DO BEGIN
                   WIDGET_CONTROL, WIDS.M_NOTES(I), SET_VALUE = TEMP_NOTES(I)
               ENDFOR

NOFILE:
              IF (FILENAME EQ '') OR (START_STATUS EQ 0) THEN $
                 WIDGET_CONTROL, WIDS.LIST(2), /INPUT_FOCUS
              END
         VALUE EQ 'LOOKUP FILENAME': BEGIN
              ; Get the LOOKUP file 
              WM_LOOKUP, WGSP_LOOKUP, LFILENAME, BAD_TABLE
              IF (LFILENAME NE '') THEN BEGIN
                 WIDGET_CONTROL, WIDS.LIST(3), SET_VALUE = LFILENAME
                 CURRENT_VALUES(7) = LFILENAME
                 BREAK_FILE, LFILENAME, LDEV, LDIRECT, LFNAME, LFEXT, LFVERSION
                 IF (BAD_TABLE NE 1) THEN $
                    INFORM, 'Lookup file is ' + $
                            STRTRIM(LFNAME + LFEXT,2) + '.  Ready.'
              ENDIF
              END

         ; Create a work window for Burst History
         VALUE EQ 'BURST HISTORY': BEGIN
               ; Make sure data has been read
               WARN_NO_DATA, DATA_STATUS
               IF (DATA_STATUS EQ 'NO DATA') THEN RETURN

               WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 0
               IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
                  WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 0
               ENDIF

               WGSP_WORK = 'HISTORY'
               W_WORK, 'HISTORY' 
               WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 1
               IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
                  WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 1
               ENDIF
               INFORM, 'Ready.'
               END
         ; Create a work window for Burst Spectrum
         VALUE EQ 'BURST SPECTRUM': BEGIN
               ; Make sure data has been read
               WARN_NO_DATA, DATA_STATUS
               IF (DATA_STATUS EQ 'NO DATA') THEN RETURN

               WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 0
               IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
                  WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 0
               ENDIF

               WGSP_WORK = 'SPECTRUM'
               W_WORK, 'SPECTRUM'
               WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 1
               IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
                  WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 0
               ENDIF

               INFORM, 'Ready.'
               END

         ; Intitiate the MFIT widget
         VALUE EQ 'FIT SPECTRUM / MFIT': BEGIN
               ; Make sure data has been read
               WARN_NO_DATA, DATA_STATUS
               IF (DATA_STATUS EQ 'NO DATA') THEN RETURN

               IF (WGSP_NO_MFIT EQ 1) THEN BEGIN
                  INFORM, 'MFIT currently disabled.' 
               ENDIF ELSE BEGIN
                  WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 0
                  IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
                     WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 0
                  ENDIF

                  RATE_SAVE = RATE_RANGE 
                  HIST_SAVE = HIST_RANGE 
                  ENRG_SAVE = ENERGY_RANGE 
                  INIT_TIME_RANGE = TIME_RANGE
                  INIT_HIST_RANGE = HIST_RANGE
                  W_MFIT
                  RATE_RANGE   = RATE_SAVE
                  HIST_RANGE   = HIST_SAVE
                  ENERGY_RANGE = ENRG_SAVE

                  WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 1
                  IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
                     WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 1
                  ENDIF
                  INFORM, 'Ready.'
               ENDELSE
               END

         VALUE EQ 'DISPLAY PARAMETERS': BEGIN
               ; Make sure data has been read
               WARN_NO_DATA, DATA_STATUS
               IF (DATA_STATUS EQ 'NO DATA') THEN RETURN

               WM_DISPLAY_PARAMETERS
               END

         ; HELP button
         VALUE EQ 'HELP': BEGIN
               IF (STRUPCASE(STRMID(WGSP_BROWSER, 0, 10)) EQ 'NO_BROWSER') $
                  THEN BEGIN
                  WM_XDISPLAYFILE, WGSP_SOURCE + STRLOWCASE('HELP.HLP'), $
                  TITLE = 'WINGSPAN Help'
               ENDIF ELSE BEGIN
                  INFORM, 'Spawning ' + WGSP_BROWSER
                  IF (STRUPCASE(!VERSION.OS) EQ 'VMS') THEN BEGIN
                     SPAWN, /NOWAIT, WGSP_BROWSER
                  ENDIF ELSE BEGIN
                     SPAWN, WGSP_BROWSER + ' &'
                  ENDELSE
                  WAIT, 3
                  INFORM, 'Ready.'
               ENDELSE
               END
 
         ; User external routines are defined in and initiated from
         ;  W_EXTERNAL_ROUTINES.PRO
         VALUE EQ STRUPCASE( 'EXTERNAL ROUTINES' ): BEGIN
               ; Make sure data has been read
               WARN_NO_DATA, DATA_STATUS, /NO_DIALOG
               IF (DATA_STATUS EQ 'NO DATA') THEN BEGIN
                  IF (WIDMENU(['Warning: No Data Read', 'CANCEL', 'OK'], $
                                TITLE = 0) EQ 1) THEN RETURN
               ENDIF

               WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 0
               IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
                  WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 0
               ENDIF
               W_EXTERNAL_ROUTINES
               WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 1
               IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
                  WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 1
               ENDIF
               END

         VALUE EQ 'QUIT': BEGIN
               IF (WGSP_CONFIRM NE 0) THEN BEGIN
                  STATUS = WID_DIALOG( 'QUIT Session?' )
                  IF (STATUS EQ 'OK') THEN BEGIN
                     WM_CLEANUP
                     WIDGET_CONTROL, EVENT.TOP, /DESTROY
                  ENDIF ELSE BEGIN
                     WIDGET_CONTROL, EVENT.TOP, MAP = 1
                     IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
                        WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 1
                     ENDIF
                  ENDELSE
               ENDIF ELSE BEGIN
                  WM_CLEANUP
                  WIDGET_CONTROL, EVENT.TOP, /DESTROY
               ENDELSE
               END

;         VALUE EQ 'COLOR': BEGIN
;               INFORM, 'Load a color table.  Select B-W LINEAR for Monochrome.'
;               XLOADCT, GROUP = EVENT.TOP  
;               INFORM, 'Ready.'
;               END
         VALUE EQ 'COLOR': BEGIN
               IF (!P.BACKGROUND EQ WGSP_COLOR_LO) THEN BEGIN
                  !P.BACKGROUND = WGSP_COLOR_HI
                  !P.COLOR = WGSP_COLOR_LO
                  WGSP_COLOR = 1
               ENDIF ELSE BEGIN
                  !P.BACKGROUND = WGSP_COLOR_LO
                  !P.COLOR = WGSP_COLOR_HI
                  WGSP_COLOR = 0
               ENDELSE
               WM_UPDATE
               INFORM, 'Ready.'
               END

         ; CV selector
         VALUE EQ 'PLOTTING': BEGIN
               CV_SELECT_PLOTTING 
               END
         VALUE EQ 'FILES': BEGIN
               CV_SELECT_FILES 
               END               
         VALUE EQ 'MISCELLANEOUS': BEGIN
               CV_SELECT_MISC 
               END               
         VALUE EQ 'PLOT LABELS / OPTIONS': BEGIN
               W_LABEL 
               END               

         VALUE EQ 'LINEAR': BEGIN
               CASE STRUPCASE(UVALUE) OF
                    'HIST_TIME_LIN': BEGIN
;                        STR = 'Time Axis: LINEAR'
;                        WIDGET_CONTROL, WIDS.HISTORY_XAXIS, SET_VALUE = STR
;                        CURRENT_VALUES(9) = 'LINEAR'
;                        INFORM, 'Time scale is LINEAR.  Ready.'
;                        WM_UPDATE
                        END
                    'HIST_RATE_LIN': BEGIN
                        STR = 'Rate Axis: LINEAR'
                        WIDGET_CONTROL, WIDS.HISTORY_YAXIS, SET_VALUE = STR
                        CURRENT_VALUES(10) = 'LINEAR'
                        INFORM, 'Rate scale is LINEAR.  Ready.'
                        WM_UPDATE
                        END
                    'SPEC_ENERGY_LIN': BEGIN
                        STR = 'Energy Axis: LINEAR'
                        WIDGET_CONTROL, WIDS.SPECTRUM_XAXIS, SET_VALUE = STR
                        CURRENT_VALUES(1) = 'LINEAR'
                        INFORM, 'Energy scale is LINEAR.  Ready.'
                        WM_UPDATE
                        END
                    'SPEC_RATE_LIN': BEGIN
                        STR = 'Rate Axis: LINEAR'
                        WIDGET_CONTROL, WIDS.SPECTRUM_YAXIS, SET_VALUE = STR
                        CURRENT_VALUES(2) = 'LINEAR'
                        INFORM, 'Rate scale is LINEAR.  Ready.'
                        WM_UPDATE
                        END
                    ELSE: PRINT, '* * * W_MAIN: ERROR: No handler for button '  + VALUE
               ENDCASE
               END
         VALUE EQ 'LOG': BEGIN
               CASE STRUPCASE(UVALUE) OF
                    'HIST_TIME_LOG': BEGIN
;                        STR = 'Time Axis: LOG'
;                        WIDGET_CONTROL, WIDS.HISTORY_XAXIS, SET_VALUE = STR
;                        CURRENT_VALUES(9) = 'LOG'
;                        INFORM, 'Time scale is LOG.  Ready.'
;                        WM_UPDATE
                        END
                    'HIST_RATE_LOG': BEGIN
                        STR = 'Rate Axis: LOG'
                        WIDGET_CONTROL, WIDS.HISTORY_YAXIS, SET_VALUE = STR
                        CURRENT_VALUES(10) = 'LOG'
                        INFORM, 'Rate scale is LOG.  Ready.'
                        WM_UPDATE
                        END
                    'SPEC_ENERGY_LOG': BEGIN
                        STR = 'Energy Axis: LOG'
                        WIDGET_CONTROL, WIDS.SPECTRUM_XAXIS, SET_VALUE = STR
                        CURRENT_VALUES(1) = 'LOG'
                        INFORM, 'Energy scale is LOG.  Ready.'
                        WM_UPDATE
                        END
                    'SPEC_RATE_LOG': BEGIN
                        STR = 'Rate Axis: LOG'
                        WIDGET_CONTROL, WIDS.SPECTRUM_YAXIS, SET_VALUE = STR
                        CURRENT_VALUES(2) = 'LOG'
                        INFORM, 'Rate scale is LOG.  Ready.'
                        WM_UPDATE
                        END
                    ELSE: PRINT, '* * * W_MAIN: ERROR: No handler for button '  + VALUE
               ENDCASE
               END

         ELSE:

    ENDCASE ; for VALUE
    END

 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ; The radio button widget events
 '': BEGIN
      WIDGET_CONTROL, EVENT.ID, GET_UVALUE = UVALUE
      CASE UVALUE OF 
       'HBUTTONS': BEGIN
        CASE EVENT.VALUE OF
         0: BEGIN
             ACTION = 'LOG'
             CURRENT_VALUES(10) = ACTION
             INFORM, 'Rate scale is ' + STRTRIM(ACTION,2) + '.  Ready.'
             WM_UPDATE
            END
         1: BEGIN
             ACTION = 'LINEAR'
             CURRENT_VALUES(10) = ACTION
             INFORM, 'Rate scale is ' + STRTRIM(ACTION,2) + '.  Ready.'
             WM_UPDATE
            END
        ENDCASE
        END
       'EBUTTONS': BEGIN
        CASE EVENT.VALUE OF
         0: BEGIN
             ACTION = 'LOG'
             CURRENT_VALUES(1) = ACTION
             INFORM, 'Energy scale is ' + STRTRIM(ACTION,2) + '.  Ready.'
             WM_UPDATE
            END
         1: BEGIN
             ACTION = 'LINEAR'
             CURRENT_VALUES(1) = ACTION
             INFORM, 'Energy scale is ' + STRTRIM(ACTION,2) + '.  Ready.'
             WM_UPDATE
            END
        ENDCASE
        END
       'RBUTTONS': BEGIN
        CASE EVENT.VALUE OF
         0: BEGIN
             ACTION = 'LOG'
             CURRENT_VALUES(2) = ACTION
             INFORM, 'Rate scale is ' + STRTRIM(ACTION,2) + '.  Ready.' 
             WM_UPDATE
            END
         1: BEGIN
             ACTION = 'LINEAR'
             CURRENT_VALUES(2) = ACTION
             INFORM, 'Rate scale is ' + STRTRIM(ACTION,2) + '.  Ready.'
             WM_UPDATE
            END
        ENDCASE
        END
       'ABUTTONS': BEGIN
        CASE EVENT.VALUE OF
         0: BEGIN
             ACTION = 'OFF'
             INFORM, 'Auto Read is ' + STRTRIM(ACTION,2) + '.  Ready.' 
             AUTO_READ = 0
            END
         1: BEGIN
             ACTION = 'ON'
             INFORM, 'Auto Read is ' + STRTRIM(ACTION,2) + '.  Ready.'
             AUTO_READ = 1
            END
        ENDCASE
        END
       'JBUTTONS': BEGIN
        CASE EVENT.VALUE OF
         0: BEGIN
             CDET = 0L
             NUM_DET = 1L
             JOINT_FIT = 0
             CHANGE_FILE = WIDMENU(['Change current FITS file or re-read?', $
                                    'CANCEL', 'Read New File', $
                                    'Read Current File'], TITLE = 0)
             CASE CHANGE_FILE OF
                  1: RETURN
                  2: BEGIN
                     WM_FILE_ENTRY, /FITS, WGSP_FITS, FILENAME
                     IF (FILENAME EQ '') THEN BEGIN
                        ; RSM, 28 March, if filename is NULL, 
                        ;  then don't do anything
                        RETURN
                     ENDIF
                     END
                  ELSE: BEGIN
                     FILENAME = CURRENT_VALUES(6)
                     END
             ENDCASE

             BREAK_FILE, FILENAME, DEV, DIRECT, FNAME, FEXT, FVERSION

             ; Break up the user-supplied filter for the lookup file
             BREAK_FILE, WGSP_LOOKUP, LDEV, LDIRECT, LFNAME, LFEXT, LFVERSION
             LUFILE = DEV + DIRECT + STRLOWCASE(FNAME + LFEXT)

             ; Read FITS file and plot data
             WIDGET_CONTROL, GET_VALUE = H_WINDOW, WIDS.DRAW1
             WIDGET_CONTROL, GET_VALUE = S_WINDOW, WIDS.DRAW2
             INFORM, 'Reading ' + STRTRIM(FILENAME,2)

             ; Must initialize file name array!
             DATA_FILE_NAME = STRARR (MAX_DET)
             FOR LDET=0, MAX_DET - 1 DO BEGIN
                 DATA_FILE_NAME (LDET) = STRING (REPLICATE (32B, 70))
             ENDFOR             

             WM_START_SESSION, FILENAME, H_WINDOW, S_WINDOW, START_STATUS
             IF (START_STATUS EQ 0) THEN GOTO, NFILE

             ; Auto read LOOKUP file, if set
             CHECK = FINDFILE(LUFILE, COUNT = COUNT)
             IF (AUTO_READ EQ 1) THEN BEGIN
                IF (COUNT EQ 0) THEN BEGIN
                   INFORM, 'LOOKUP file not found.  Updating menu.'
                   INITIALIZE_PLOTS
                   INFORM, 'Ready.'
                ENDIF ELSE BEGIN
                   HAVE_LOOKUP = 1
                   READ_LOOKUPS, LUFILE, DUMMY, $
                                 CURRENT_VALUES(1), CURRENT_VALUES(2), BAD_TABLE
                   IF (BAD_TABLE NE 1) THEN BEGIN
                      IF CURRENT_VALUES(1) EQ 'LOG' THEN TVAL = 0 ELSE TVAL = 0
                      WIDGET_CONTROL, WIDS.EBUTTONS, SET_VALUE = TVAL
                      IF CURRENT_VALUES(2) EQ 'LOG' THEN TVAL = 0 ELSE TVAL = 0
                      WIDGET_CONTROL, WIDS.RBUTTONS, SET_VALUE = TVAL

                      INFORM, 'Lookup file is ' + STRTRIM(LUFILE, 2) + '.  Ready.'
                   ENDIF ELSE BEGIN
                    INITIALIZE_PLOTS
                   ENDELSE
                ENDELSE
             ENDIF ELSE INFORM, 'Ready.'

             ; Update menu 
             WIDGET_CONTROL, WIDS.LIST(2), SET_VALUE = FILENAME
             WIDGET_CONTROL, WIDS.LIST(3), SET_VALUE = LUFILE
             CURRENT_VALUES(6) = FILENAME
             IF (COUNT EQ 0) THEN BEGIN
                CURRENT_VALUES(7) = 'none'
             ENDIF ELSE BEGIN  
                CURRENT_VALUES(7) = LUFILE
             ENDELSE

             ; Update the object name
             BURST_NAME = STRTRIM(SXPAR(HEADER, 'OBJECT'), 2)
             CURRENT_VALUES(8) = BURST_NAME

             ; Update the notes
             TEMP_NOTES    = STRARR(NUM_NOTES) 
             TEMP_NOTES(0) = NOTES(0)
             TEMP_NOTES(1) = NOTES(3) + '    ' + NOTES(4)
             TEMP_NOTES(2) = NOTES(2)
             TEMP_NOTES(3) = 'Detector type: ' + $
                  STRTRIM(SXPAR(DH, 'DET_MODE'), 2) + '    ' + NOTES(1)
             TEMP_NOTES(4) = 'Current printer: ' + STRTRIM(WGSP_PRINTER, 2)
             FOR I=0, NUM_NOTES-1 DO BEGIN
                 WIDGET_CONTROL, WIDS.M_NOTES(I), SET_VALUE = TEMP_NOTES(I)
             ENDFOR

             ACTION = 'OFF'
             INFORM, 'Joint Fitting is ' + STRTRIM(ACTION,2) + '.  Ready.' 
             JOINT_FIT = 0
            END
         1: BEGIN
             ACTION = 'ON'
             INFORM, 'Joint Fitting is ' + STRTRIM(ACTION,2) + '.  Ready.'
             JOINT_FIT = 1
            END
        ENDCASE
        END
      ENDCASE ; for UVALUE
     END

 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ; The draw widget events (second method of selecting work window)
 'WIDGET_DRAW': BEGIN
    ; Make sure data has been read
    WARN_NO_DATA, DATA_STATUS
    IF (DATA_STATUS EQ 'NO DATA') THEN RETURN

    WIDGET_CONTROL, EVENT.ID
    CASE EVENT.ID OF

     ; Burst history
     WIDS.DRAW1: BEGIN
          IF (EVENT.TYPE EQ 0) THEN BEGIN
             WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 0
             IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
                WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 0
             ENDIF
             WGSP_WORK = 'HISTORY'
             W_WORK, 'HISTORY'
             WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 1
             IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
                WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 1
             ENDIF
             INFORM, 'Ready.'
          ENDIF
          END
     ; Burst spectrum
     WIDS.DRAW2: BEGIN
          IF (EVENT.TYPE EQ 0) THEN BEGIN
             WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 0
             IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
                WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 0
             ENDIF
             WGSP_WORK = 'SPECTRUM'
             W_WORK, 'SPECTRUM'
             WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 1
             IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
                WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 1
             ENDIF
             INFORM, 'Ready.'
          ENDIF
          END

    ENDCASE
    END

ENDCASE ; for TYPE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
PRO W_MAIN

@wgsp_cb.pro
; Following COMMON internal to the main widget routine and its event handler
COMMON NUMBER_DATA, NUM_ENTRIES, NUM_NOTES


; Get the MFIT control variables 
; Set the variable types, and assign defaults
GET_CONTROL, /GET_DEFAULTS

DEVICE, RETAIN = CV_IDL_RETAIN


; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Parameter initialization

; Initialize MFIT input parameters
MAX_CHAN       = 252
MAX_EBINS      = 275
MAX_PLOT       = 6 * MAX_CHAN + 1
MAX_DET        = 8
MAX_TERMS      = 56
MAX_CHI_2D     = 50
MAX_CHI_1D     = MAX([500, MAX_CHI_2D])
MAX_PLOT_TYPE  = 7
MAX_PARAM      = 20
NUM_DET        = LONG(1) ; Start out with a single detector selected
NUM_CHAN       = LONARR(MAX_DET)
DET_NAMES      = REPLICATE(STRING(REPLICATE(32B, 12)), MAX_DET)
CHAN_OFFSET    = LONARR(MAX_DET)
CHAN_ENERGY    = FLTARR(MAX_CHAN, MAX_DET)
CHAN_WIDTH     = FLTARR(MAX_CHAN, MAX_DET)
OBS_CRATE      = FLTARR(MAX_CHAN, MAX_DET)
BACK_CRATE     = FLTARR(MAX_CHAN, MAX_DET)
BACK_CSIG      = FLTARR(MAX_CHAN, MAX_DET)
LIVE_TIME      = FLTARR(MAX_CHAN, MAX_DET)
NUM_EBINS      = LONARR(MAX_DET)
PHOT_ENERGY    = FLTARR(MAX_EBINS, MAX_DET)
PHOT_WIDTH     = FLTARR(MAX_EBINS, MAX_DET)
RES_FRAC_AT511 = FLTARR(MAX_DET)
RES_EXP        = FLTARR(MAX_DET)
DATA_FILE_NAME = REPLICATE(STRING(REPLICATE(32B, 70)), MAX_DET)
USED_REBIN     = LONARR(MAX_DET)
REBIN_CHAN     = LONARR(MAX_CHAN + 1, MAX_DET)
 
NUM_ENTRIES      = 4  
NUM_WORK_BUTTONS = 15 ; Buttons on WORK widget
NUM_MFIT_BUTTONS = 30 ; Buttons on MFIT widget
NUM_NOTES        = 5

; End parameter initialization
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


; Structure that holds the WIDGET IDs
WIDS = { $

        ; Main menu ID
        TOP_BASE: 0L, $       
        
        ; If main menu is two separate base, set by WGSP_SEPARATE_BASE, this 
        ; is the 2nd base ID
        DRAW_SUB_BASE: 0L, $  

        LIST: LONARR(NUM_ENTRIES), $
        DRAW1: 0L, DRAW2: 0L, $
        M_STATUS: 0L, $
        HBUTTONS: 0L, EBUTTONS: 0L, RBUTTONS: 0L, $
        ABUTTONS: 0L, $
        JBUTTONS: 0L, $

        ; Work area for BURST HISTORY/SPECTRUM
        WORK_BASE: 0L, $
        MW_STATUS: 0L, MW_DATA: 0L, $
        WORK_DRAW: 0L, $
        SELECT_BKGD: 0L, $

        ; MFIT menu
        MFIT_BASE: 0L, $
        MM_STATUS: 0L, MM_DATA: 0L, $

        ; Plot widget
        HC_BASE: 0L, HC_DRAW: 0L, HC_STATUS: 0L, $

        ; Object (e.g., burst, flare) notes
        M_NOTES: LONARR(NUM_NOTES)}


IF (XREGISTERED('W_MAIN')) THEN BEGIN
   ; Because of COMMON BLOCKS, only one instance of the WINGSPAN widget
   ; is allowed
   INFORM, 'W_MAIN already running.'
   RETURN
ENDIF

; In case user made changes
GET_CONTROL

; First Initialization of the saved rate mode:
RATE_MODE_SAVE = CV_RATE_MODE  

; Parent
MTITLE = STRUPCASE(STRTRIM(CALLING_PROG, 2))
WIDS.TOP_BASE = WIDGET_BASE(TITLE = MTITLE, /COLUMN)

LTEXT_LIST = [ $
    'Spectrum Energy scale', $
    'Spectrum Rate scale', $
    'Time label', $
    'Energy label', $
    'Rate label', $
    'FITS filename', $
    'Lookup filename', $
    'Plot title', $
    'Auto Read Lookup', $
    'Joint Fitting']

CASE CV_RATE_MODE OF 
    'R': RTEXT_LIST = [ $
          '   ', $
          'LOG', $                             ; SPECTRUM Energy scale
          'LOG', $                             ; SPECTRUM Rate scale
          'Seconds since burst trigger', $
          'Energy (keV)', $
          'Counts s!U-1!N', $
          'FITS FILE', $
          'LOOKUP FILE', $
          '   ', $
          'LINEAR', $                          ; HISTORY Time scale
          'LINEAR']                            ; HISTORY Rate scale
    ELSE: RTEXT_LIST = [ $
          '   ', $
          'LOG', $                             ; SPECTRUM Energy scale
          'LOG', $                             ; SPECTRUM Rate scale
          'Seconds since burst trigger', $
          'Energy (keV)', $
          'Counts s!U-1!N keV!U-1!N', $
          'FITS FILE', $
          'LOOKUP FILE', $
          '   ', $
          'LINEAR', $                          ; HISTORY Time scale
          'LINEAR']                            ; HISTORY Rate scale
ENDCASE

TEXT_LIST = [LTEXT_LIST, RTEXT_LIST]
MAX_XLEN  = MAX( STRLEN(TEXT_LIST) )     
MAX_YLEN  = N_ELEMENTS(TEXT_LIST) 

; Defaults
MENU_VALUES    = RTEXT_LIST
MENU_VALUES(6) = WGSP_FITS
MENU_VALUES(7) = WGSP_LOOKUP


; STATUS bar spans top of widget
WIDS.M_STATUS = WIDGET_TEXT(WIDS.TOP_BASE, VALUE = ' ', $
                            UVALUE = 'STATUS', YSIZE = 1)

; Base for the rest of the menu
SUB_TOP_BASE = WIDGET_BASE(WIDS.TOP_BASE, /ROW)

; Begin left side of main menu - - - - - - - - - - - - - - - - - - - - - - - - -

BASE1 = WIDGET_BASE(SUB_TOP_BASE, /COLUMN, /FRAME)

 ; QUIT and HELP buttons
 SUB_BASE0 = WIDGET_BASE(BASE1, /ROW)

  QBUTTON = WIDGET_BUTTON(SUB_BASE0, VALUE = 'QUIT')

  CBUTTON = WIDGET_BUTTON(SUB_BASE0, MENU = 1, VALUE = 'Preferences')
  CBUTTON_LABELS = ['Plotting', 'Files','Miscellaneous','Plot Labels / Options']

       FOR I=0, N_ELEMENTS(CBUTTON_LABELS)-1 DO BEGIN
           PULLDOWN = WIDGET_BUTTON(CBUTTON, VALUE = CBUTTON_LABELS(I))
       ENDFOR
     
  BUTTON  = WIDGET_BUTTON(SUB_BASE0, VALUE = 'Color')
  HBUTTON = WIDGET_BUTTON(SUB_BASE0, VALUE = 'Help')

 TBASE1 = WIDGET_BASE(BASE1, /COLUMN)
 
 ; HISTORY Rate scale
 TEMP_BASE = WIDGET_BASE(TBASE1, /ROW)
 LABEL     = WIDGET_LABEL(TEMP_BASE, VALUE = 'History Rate Scale')
 BUTTON_LABELS = ['LOG', 'LINEAR']
 WIDS.HBUTTONS = CW_BGROUP(TEMP_BASE, BUTTON_LABELS, $
                    /FRAME, /ROW, /EXCLUSIVE, /NO_RELEASE, UVALUE = 'HBUTTONS')

 ; SPECTRUM Energy scale
 TEMP_BASE = WIDGET_BASE(TBASE1, /ROW)
 LABEL     = WIDGET_LABEL(TEMP_BASE, VALUE = LTEXT_LIST(0))
 BUTTON_LABELS = ['LOG', 'LINEAR']
 WIDS.EBUTTONS = CW_BGROUP(TEMP_BASE, BUTTON_LABELS, $
                    /FRAME, /ROW, /EXCLUSIVE, /NO_RELEASE, UVALUE = 'EBUTTONS')

 ; SPECTRUM Rate scale
 TEMP_BASE = WIDGET_BASE(TBASE1, /ROW)
 LABEL     = WIDGET_LABEL(TEMP_BASE, VALUE = LTEXT_LIST(1))
 BUTTON_LABELS = ['LOG', 'LINEAR']
 WIDS.RBUTTONS = CW_BGROUP(TEMP_BASE, BUTTON_LABELS, $
                    /FRAME, /ROW, /EXCLUSIVE, /NO_RELEASE, UVALUE = 'RBUTTONS')

 ; Auto Read Lookup
 TEMP_BASE = WIDGET_BASE(TBASE1, /ROW)
 LABEL     = WIDGET_LABEL(TEMP_BASE, VALUE = LTEXT_LIST(8))
 BUTTON_LABELS = ['OFF', 'ON']
 WIDS.ABUTTONS = CW_BGROUP(TEMP_BASE, BUTTON_LABELS, $
                    /FRAME, /ROW, /EXCLUSIVE, /NO_RELEASE, UVALUE = 'ABUTTONS')

 ; Joint Fitting
 TEMP_BASE = WIDGET_BASE(TBASE1, /ROW)
 LABEL     = WIDGET_LABEL(TEMP_BASE, VALUE = LTEXT_LIST(9))
 BUTTON_LABELS = ['OFF', 'ON']
 WIDS.JBUTTONS = CW_BGROUP(TEMP_BASE, BUTTON_LABELS, $
                    /FRAME, /ROW, /EXCLUSIVE, /NO_RELEASE, UVALUE = 'JBUTTONS')

 ; FITS filename
 TEMP_BASE = WIDGET_BASE(TBASE1, /ROW)
 LABEL     = WIDGET_BUTTON(TEMP_BASE, VALUE = LTEXT_LIST(5))
 WIDS.LIST(2) = WIDGET_TEXT(TEMP_BASE, VALUE = MENU_VALUES(6), /EDIT, $
                            UVALUE = 6, XSIZE = MAX_XLEN, YSIZE = 1)
 
 ; Lookup filename
 TEMP_BASE = WIDGET_BASE(TBASE1, /ROW)
 LABEL     = WIDGET_BUTTON(TEMP_BASE, VALUE = LTEXT_LIST(6))
 WIDS.LIST(3) = WIDGET_TEXT(TEMP_BASE, VALUE = MENU_VALUES(7), /EDIT, $
                            UVALUE = 7, XSIZE = MAX_XLEN, YSIZE = 1)

 ; Selection buttons
 SUB_BASE1 = WIDGET_BASE(BASE1, /COLUMN)

 BUTTON = WIDGET_BUTTON(SUB_BASE1, VALUE = 'Display Parameters')

 ; Only if we are running MFIT
 IF (WGSP_NO_MFIT EQ 0) THEN BEGIN
    BUTTON = WIDGET_BUTTON(SUB_BASE1, VALUE = 'Fit Spectrum / MFIT')
 ENDIF 

 ; Pull-down menu for Spectral History
 BUTTON = WIDGET_BUTTON(SUB_BASE1, MENU = 1, VALUE = 'Spectral History')
 
 BUTTON_LABELS = ['Stacked Histories', 'Stacked Spectra', 'Contour', $
                  'Surface', 'Hidden Line']

       PULLDOWN = WIDGET_BUTTON(BUTTON, VALUE = BUTTON_LABELS(0))
       PULLDOWN = WIDGET_BUTTON(BUTTON, VALUE = BUTTON_LABELS(1))
       PULLDOWN = WIDGET_BUTTON(BUTTON, VALUE = BUTTON_LABELS(2))
       PULLDOWN = WIDGET_BUTTON(BUTTON, VALUE = BUTTON_LABELS(3))
       PULLDOWN = WIDGET_BUTTON(BUTTON, VALUE = BUTTON_LABELS(4))

 ; Pops up a menu that holds user-defined routines.  These routines are
 ;  defined in the file W_EXTERNAL_ROUTINES.PRO
 BUTTON = WIDGET_BUTTON(SUB_BASE1, VALUE = 'External Routines')
 
 ; Add some information notes
 TBASE3 = WIDGET_BASE(BASE1, /COLUMN)
 IF (!VERSION.RELEASE GE '4.0.1') THEN BEGIN
    FOR I=0, NUM_NOTES-1 DO BEGIN
        WIDS.M_NOTES(I) = WIDGET_LABEL(TBASE3, VALUE = ' ', /DYNAMIC_RESIZE)
    ENDFOR
 ENDIF ELSE BEGIN
    FOR I=0, NUM_NOTES-1 DO BEGIN
        WIDS.M_NOTES(I) = WIDGET_LABEL(TBASE3, VALUE = ' ')
    ENDFOR
 ENDELSE


; End left side of main menu - - - - - - - - - - - - - - - - - - - - - - - - - -


; Begin right side of main menu - - - - - - - - - - - - - - - - - - - - - - - -

; Base for the two draw widgets and buttons
IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
   WIDS.DRAW_SUB_BASE = WIDGET_BASE(/COLUMN, /FRAME)
ENDIF ELSE BEGIN
   WIDS.DRAW_SUB_BASE = WIDGET_BASE(SUB_TOP_BASE, /COLUMN, /FRAME)
ENDELSE


 ; BURST HISTORY widget 
 BUTTON = WIDGET_BUTTON(WIDS.DRAW_SUB_BASE, VALUE = 'Burst History')
 WIDS.DRAW1 = WIDGET_DRAW(WIDS.DRAW_SUB_BASE, $
                         XSIZE = CURRENT_SCREEN(0)*0.45*WGSP_SCALE, $
                         YSIZE = CURRENT_SCREEN(1)*0.40, /FRAME, /BUTTON_EVENTS)

 ; BURST SPECTRUM widget 
 BUTTON = WIDGET_BUTTON(WIDS.DRAW_SUB_BASE, VALUE = 'Burst Spectrum')
 WIDS.DRAW2 = WIDGET_DRAW(WIDS.DRAW_SUB_BASE, $
                         XSIZE = CURRENT_SCREEN(0)*0.45*WGSP_SCALE, $
                         YSIZE = CURRENT_SCREEN(1)*0.40, /FRAME, /BUTTON_EVENTS)

; End right side of main menu - - - - - - - - - - - - - - - - - - - - - - - - -


; Set the current detector
CDET = 0L                  

; Initialize plotting parameters set in wing.pro
!P.CHARSIZE  = GLOBAL_CHARSIZE
!P.CHARTHICK = GLOBAL_CHARTHICK
!X.TICKLEN   = GLOBAL_TICKLEN
!Y.TICKLEN   = GLOBAL_TICKLEN
!P.THICK     = GLOBAL_LINETHICK
!X.THICK     = GLOBAL_LINETHICK
!Y.THICK     = GLOBAL_LINETHICK

WLO_X = -3000.0 & WHI_X = 3000.0 ; Temporary initial values for GET_RANGE widget
OPLOT_TIME_HIST = 1              ; ON
WGSP_COLOR_LO = !P.BACKGROUND    ; Window color
WGSP_COLOR_HI = !P.COLOR         ; Drawing color
WGSP_COLOR    = 0                ; Default color setup; WGSP_COLOR = 1 for
                                 ; inverted colors

CURRENT_VALUES = MENU_VALUES

WIDGET_CONTROL, WIDS.HBUTTONS, SET_VALUE = WGSP_HIST_RATE_SCALE
WIDGET_CONTROL, WIDS.EBUTTONS, SET_VALUE = WGSP_ENERGY_SCALE
WIDGET_CONTROL, WIDS.RBUTTONS, SET_VALUE = WGSP_RATE_SCALE
WIDGET_CONTROL, WIDS.ABUTTONS, SET_VALUE = AUTO_READ 
WIDGET_CONTROL, WIDS.JBUTTONS, SET_VALUE = JOINT_FIT
 
; Realize the BASE widget(s) and the children
IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
   WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, /REALIZE, SHOW = 1 
ENDIF

WIDGET_CONTROL, WIDS.TOP_BASE, /REALIZE, SHOW = 1 


; Try to read the FITS file automatically based on WGSP_FITS
;  This is just the event handler for FITS filename: event

; Get the FITS file 
WM_FILE_ENTRY, /FITS, WGSP_FITS, FILENAME
IF (FILENAME EQ '') THEN BEGIN
   START_STATUS = 0
   GOTO, NFILE
ENDIF

BREAK_FILE, FILENAME, DEV, DIRECT, FNAME, FEXT, FVERSION

; Break up the user-supplied filter for the lookup file
BREAK_FILE, WGSP_LOOKUP, LDEV, LDIRECT, LFNAME, LFEXT, LFVERSION
LUFILE = DEV + DIRECT + STRLOWCASE(FNAME + LFEXT)

; Read FITS file and plot data
WIDGET_CONTROL, GET_VALUE = H_WINDOW, WIDS.DRAW1
WIDGET_CONTROL, GET_VALUE = S_WINDOW, WIDS.DRAW2
INFORM, 'Reading ' + STRTRIM(FILENAME,2)
WM_START_SESSION, FILENAME, H_WINDOW, S_WINDOW, START_STATUS
IF (START_STATUS EQ 0) THEN GOTO, NFILE

; Auto read LOOKUP file, if set
CHECK = FINDFILE(LUFILE, COUNT = COUNT)
IF (AUTO_READ EQ 1) THEN BEGIN
   IF (COUNT EQ 0) THEN BEGIN
      INFORM, 'LOOKUP file not found.  Updating menu.'
      INITIALIZE_PLOTS
      INFORM, 'Ready.'
   ENDIF ELSE BEGIN
      HAVE_LOOKUP = 1
      READ_LOOKUPS, LUFILE, DUMMY, CURRENT_VALUES(1),CURRENT_VALUES(2),BAD_TABLE
      IF (BAD_TABLE NE 1) THEN BEGIN
         IF CURRENT_VALUES(1) EQ 'LOG' THEN TVAL = 0 ELSE TVAL = 0
         WIDGET_CONTROL, WIDS.EBUTTONS, SET_VALUE = TVAL
         IF CURRENT_VALUES(2) EQ 'LOG' THEN TVAL = 0 ELSE TVAL = 0
         WIDGET_CONTROL, WIDS.RBUTTONS, SET_VALUE = TVAL

         INFORM, 'Lookup file is ' + STRTRIM(LUFILE, 2) + '.  Ready.'
      ENDIF ELSE BEGIN
         INITIALIZE_PLOTS
      ENDELSE
   ENDELSE
ENDIF ELSE INFORM, 'Ready.'

; Update menu 
WIDGET_CONTROL, WIDS.LIST(2), SET_VALUE = FILENAME
WIDGET_CONTROL, WIDS.LIST(3), SET_VALUE = LUFILE
CURRENT_VALUES(6) = FILENAME
IF (COUNT EQ 0) THEN BEGIN
   CURRENT_VALUES(7) = 'none'
ENDIF ELSE BEGIN  
   CURRENT_VALUES(7) = LUFILE
ENDELSE

; Update the object name
BURST_NAME = STRTRIM(SXPAR(HEADER, 'OBJECT'), 2)
IF (BURST_NAME EQ 'BB123') OR (CV_TITLE_STYLE EQ 'T') THEN BEGIN
   ; Handle old file
   II = GET_NOTE('Burst')
   BURST_NAME = STRTRIM(NOTES(II), 2)
ENDIF 
CURRENT_VALUES(8) = BURST_NAME

; Update the notes
TEMP_NOTES    = STRARR(NUM_NOTES) 
TEMP_NOTES(0) = NOTES(0)
TEMP_NOTES(1) = NOTES(3) + '    ' + NOTES(4)
TEMP_NOTES(2) = NOTES(2)
TEMP_NOTES(3) = 'Detector type: ' + $
     STRTRIM(SXPAR(DH, 'DET_MODE'), 2) + '    ' + NOTES(1)
TEMP_NOTES(4) = 'Current printer: ' + STRTRIM(WGSP_PRINTER, 2)
FOR I=0, NUM_NOTES-1 DO BEGIN
    WIDGET_CONTROL, WIDS.M_NOTES(I), SET_VALUE = TEMP_NOTES(I)
ENDFOR

; Set up the labels for hardcopy
N_ANNOTES = 0
THE_PAGE_POS = [0.1, 0.28, 0.86, 0.86]

BREAK_FILE, CURRENT_VALUES(6), D_FDEV, D_FDIR, D_FNAME, D_FEXT, D_FVER
A_NOTE = 'FITS File: ' + D_FNAME + D_FEXT
ANNOTE, A_NOTE, THE_PAGE_POS

BREAK_FILE, CURRENT_VALUES(7), D_FDEV, D_FDIR, D_FNAME, D_FEXT, D_FVER
A_NOTE = 'Lookup File: ' + D_FNAME + D_FEXT
ANNOTE, A_NOTE, THE_PAGE_POS

A_NOTE = 'Calib. Name: ' + STRTRIM(CTOE_LABEL, 2)
ANNOTE, A_NOTE, THE_PAGE_POS

A_NOTE = STRTRIM(CALLING_PROG, 2)
ANNOTE, A_NOTE, THE_PAGE_POS

NFILE:

IF (FILENAME EQ '') OR (START_STATUS EQ 0) THEN BEGIN
   WIDGET_CONTROL, WIDS.LIST(2), /INPUT_FOCUS
   INFORM, 'Enter a FITS filename or a file filter.'
ENDIF 

; Register with XMANAGER
IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
   XMANAGER, 'TOP_BASE', WIDS.DRAW_SUB_BASE, $
             /JUST_REG, GROUP_LEADER = WIDS.TOP_BASE
ENDIF
XMANAGER, 'TOP_BASE', WIDS.TOP_BASE




END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


; BEGIN W_MAIN_PROCS 

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
PRO WM_START_SESSION, ACTION, H_WINDOW, S_WINDOW, IO_STATUS, BURST_NAME


@wgsp_cb.pro

GET_ANOTHER = 2
PICK_OR_FIT = 0

IF (N_ELEMENTS(CDET) NE 0) THEN BEGIN
   IF (NUM_DET LE MAX_DET) THEN BEGIN
      IF (JOINT_FIT EQ 1) THEN BEGIN
         IF (NUM_DET LT 2L) THEN BEGIN ; No data to sum into, so...
            PICK_OR_FIT = $
               WIDMENU(['What should be done with the current data?', $
               'Pick One Spectrum for Joint Fit', $
               'Sum Selection for Joint Fit'], TITLE = 0)
            GET_ANOTHER = 1
         ENDIF ELSE BEGIN            ; Allow adding data as well:
            PICK_OR_FIT = $
               WIDMENU(['What should be done with the current data?', $
               'Pick One Spectrum for Joint Fit', $
               'Sum Selection for Joint Fit', $
               'Add Selection to Existing Data'], TITLE = 0)
            GET_ANOTHER = 1
         ENDELSE
      ENDIF ELSE BEGIN
         GET_ANOTHER = 2
      ENDELSE
   ENDIF
   IF (PICK_OR_FIT EQ 1) THEN BEGIN ;Pick
      WM_UPDATE
      HISTORY_WINDOW  = H_WINDOW
      WSET, HISTORY_WINDOW
      INFORM, 'Click on a spectrum of current file to fit.'
      CURSOR, X, Y, /DOWN, /DATA

      T_IND = SEARCH(COMBINED_TIMES(0, *), X)

      IF (X LT !X.CRANGE(0)) THEN BEGIN        ; Bail out mode: bad selection
         GET_ANOTHER = 2
         GOTO, DONE
      ENDIF

      RUN_MODE = STRING(FORMAT='(A1)', 'B') ; Batch mode; not that it matters
      TEMP_SPAN = TRANSPOSE(COMBINED_TIMES(*,T_IND(0)))
      TEMP_SPAN(0,1) = TOTAL(TEMP_SPAN) / 2.0  ; Shorten a bit....
      SPEC_TITLE = CREATE_LABEL(COMBINED_TIMES, TEMP_SPAN, ' s')

      DATA_TITL = READ_MENU(8)  ;FREAD(PLOT_MENU, 9)  
      DATA_COMM = SPEC_TITLE + ' of ' + DATA_TITL 
      PASS_COMMAND = STRING(FORMAT='(A2)', 'F ')       ; Fit regardless
      NEW_DATA = LONG(1)                     ; Passing in new data to MFIT
      WRITE_SPEC, T_IND(0), DATA_TITL, DATA_COMM, THE_PP, IO_ERR, /STUFF
      IF (IO_ERR LT 0) THEN GET_ANOTHER = 2    ; Handle IO errors

      INFORM, 'Reading ' + STRTRIM(ACTION, 2)
      ENDIF $
   ELSE IF (PICK_OR_FIT EQ 2) THEN BEGIN      ; "Sum"
      RUN_MODE = STRING(FORMAT='(A1)', 'B') ; Batch mode; not that it matters
      DATA_TITL = READ_MENU(8)  ;FREAD(PLOT_MENU, 9)  
      COUNT_SPAN, COMBINED_TIMES, TIME_SPAN, T_IND
      SPEC_TITLE = CREATE_LABEL(COMBINED_TIMES, TIME_SPAN(0, *), ' s')
      DATA_COMM = SPEC_TITLE + ' of ' + DATA_TITL 
      PASS_COMMAND = STRING(FORMAT='(A2)', 'F ')       ; Fit regardless
      NEW_DATA = LONG(1)                     ; Passing in new data to MFIT
      WRITE_SPEC, T_IND, DATA_TITL, DATA_COMM, THE_PP, IO_ERR, /STUFF
      IF IO_ERR LT 0 THEN GET_ANOTHER = 2    ; Handle IO errors

      INFORM, 'Reading ' + STRTRIM(ACTION, 2)
      ENDIF $
   ELSE IF (PICK_OR_FIT EQ 3) THEN BEGIN      ; "Add"
      CHOICES = ['Choose a detector to add into:', DET_NAMES(0: NUM_DET - 2)]
      THE_DET = WIDMENU(CHOICES, TITLE = 0) - 1
      RUN_MODE = STRING(FORMAT='(A1)', 'B') ; Batch mode; not that it matters
      DATA_TITL = READ_MENU(8)  ;FREAD(PLOT_MENU, 9)  
      COUNT_SPAN, COMBINED_TIMES, TIME_SPAN, T_IND
      SPEC_TITLE = CREATE_LABEL(COMBINED_TIMES, TIME_SPAN(0, *), ' s')
      DATA_COMM = SPEC_TITLE + ' of ' + DATA_TITL 
      PASS_COMMAND = STRING(FORMAT='(A2)', 'F ')       ; Fit regardless
      NEW_DATA = LONG(1)                     ; Passing in new data to MFIT
      SUM_SPEC, T_IND, DATA_TITL, DATA_COMM, THE_PP, IO_ERR, THE_DET
      IF IO_ERR LT 0 THEN GET_ANOTHER = 2    ; Handle IO errors

      ; Need to replace current data set completely:
      CDET = CDET - 1L
      NUM_DET = NUM_DET - 1L
      INFORM, 'Reading ' + STRTRIM(ACTION, 2)
   ENDIF 

ENDIF

DONE:
IO_STATUS = 1

; Turn off buttons during read
WIDGET_CONTROL, WIDS.TOP_BASE, SENSITIVE = 0, /HOURGLASS
GET_FITS_STUFF, ACTION, TIMES, THRESHOLDS, SPECTRA, SPECTRA_ERRORS, $
                IO_STATUS, GET_ANOTHER
WIDGET_CONTROL, WIDS.TOP_BASE, SENSITIVE = 1

IF (IO_STATUS EQ 0) THEN RETURN

HISTORY_WINDOW  = H_WINDOW
SPECTRUM_WINDOW = S_WINDOW

; Don't display background model before it's calculated.
HAVE_BACK = 0     

; First time through, initialize variables in MFIT_SUPER.
IF N_ELEMENTS(HAVE_INITTED) EQ 0 THEN BEGIN
   HAVE_INITTED = LONG(0)
   HAVE_DRM     = LONG(0)
   HAVE_FIT     = LONG(0)
   FIT_ERR      = LONG(0)
ENDIF

; Reset plot annotations to NULL
ANNOTATE_BH = ''
ANNOTATE_BS = ''
;ANNOTATE_BS = STRTRIM(CTOE_LABEL, 2)

IF (AUTO_READ EQ 1) THEN $
   INITIALIZE_PLOTS, /NOPLOT $
ELSE $
   INITIALIZE_PLOTS



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
PRO WM_FILE_ENTRY, TEST_FILE, FILENAME, FITS = FITS, LU = LU
; TEST_FILE is either a complete filename or a file filter specification.
; If the file is found, it is stored in FILENAME, else FILENAME = ''

@wgsp_cb.pro
; Following COMMON internal to WM_LOOKUP, WM_FILE_ENTRY to track the 
;  status of the LOOKUP file
COMMON LFILE_STAT, READING


STAR = STRPOS(TEST_FILE(0),'*')

IF (STAR EQ -1) THEN BEGIN
   FILENAME = TEST_FILE(0)
   BREAK_FILE, FILENAME, FDEV, FDIR, FNAME, FEXT, FVERSION
   IF (FDIR EQ '') THEN $
      CHECK = FINDFILE(WGSP_SCRATCH + FNAME + FEXT, COUNT = FCOUNT) $
   ELSE $
      CHECK = FINDFILE(FILENAME, COUNT = FCOUNT) 
   IF (FCOUNT NE 0) THEN BEGIN
      GOTO, HAVE_FILE  
   ENDIF ELSE BEGIN   
      INFORM, 'File not found.  Ready.' 
      FILENAME = ''
      RETURN
   ENDELSE
ENDIF ELSE BEGIN
   UFILTER  = TEST_FILE(0)

   ; First check to see if the user has supplied a path
   BREAK_FILE, UFILTER, UDEV, UDIR, UNAME, UEXT, UVER
   IF UDIR NE '' THEN BEGIN
      UPATH   = UDEV + UDIR
      UFILTER = UNAME + UEXT + UVER
   ENDIF ELSE BEGIN
      UPATH = WGSP_SCRATCH
   ENDELSE  

   IF (KEYWORD_SET(FITS)) THEN BEGIN
      FTYPE = 'FITS'
      DEFAULT_FILE = WGSP_SOURCE + STRLOWCASE('HER_HERB_143_6.FITS')
      FILENAME = PICKFILE(FILTER=UFILTER, PATH=UPATH, $
       TITLE='Select a ' + FTYPE + ' file to read or CANCEL to make changes.', $
                          FILE=DEFAULT_FILE, /NOCONFIRM)
   ENDIF
   IF (KEYWORD_SET(LU)) THEN BEGIN
      FTYPE = 'LOOKUP'
      DEFAULT_FILE = WGSP_SOURCE + STRLOWCASE('HER_HERB_143_6.WGSP_LOOKUP')
      IF READING EQ 1 THEN $
      FILENAME = PICKFILE(FILTER=UFILTER, PATH=UPATH, $
                          TITLE='Select a ' + FTYPE + ' file to read.', $
                          FILE=DEFAULT_FILE, /NOCONFIRM)
      IF READING EQ 0 THEN $
      FILENAME = PICKFILE(FILTER=UFILTER, PATH=UPATH, $
                          TITLE='Select a ' + FTYPE + ' file to save.', $
                          FILE=DEFAULT_FILE)
   ENDIF
   IF (FILENAME EQ '') THEN BEGIN
      INFORM, 'File not found.  Ready.' 
      RETURN
   ENDIF
ENDELSE

HAVE_FILE:



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
PRO WM_LOOKUP, TEST_FILE, LFILENAME, BAD_TABLE
; TEST_FILE is either a complete filename or a file filter specification.

@wgsp_cb.pro
; Following COMMON internal to WM_LOOKUP, WM_FILE_ENTRY to track the 
;  status of the LOOKUP file
COMMON LFILE_STAT, READING


BAD_TABLE = 0 ; No error to start
LOOKUP_FILENAME = TEST_FILE(0)

READIT = WIDMENU(['Select Action for Lookup File', $
                  'CANCEL', 'Read', 'Save'], TITLE = 0)

CASE READIT OF
   1: BEGIN
      INFORM, 'Action cancelled.  Ready.'
      LFILENAME = ''
      END
   2: BEGIN
      READING = 1
      WM_FILE_ENTRY, /LU, LOOKUP_FILENAME, LFILENAME
      IF (LFILENAME EQ '') THEN RETURN ELSE CURRENT_VALUES(7) = LFILENAME

      INFORM, 'Reading lookup file ' + STRTRIM(LFILENAME,2)
      READ_LOOKUPS, LFILENAME, DUMMY, $
                    CURRENT_VALUES(1), CURRENT_VALUES(2), BAD_TABLE
      IF (BAD_TABLE NE 1) THEN BEGIN
         IF CURRENT_VALUES(1) EQ 'LOG' THEN TVAL = 0 ELSE TVAL = 0
         WIDGET_CONTROL, WIDS.EBUTTONS, SET_VALUE = TVAL
         IF CURRENT_VALUES(2) EQ 'LOG' THEN TVAL = 0 ELSE TVAL = 0
         WIDGET_CONTROL, WIDS.RBUTTONS, SET_VALUE = TVAL
      ENDIF
      END

   3: BEGIN
      READING = 0
      IF STRPOS(TEST_FILE(0),'*') NE -1 THEN BEGIN
         WM_FILE_ENTRY, /LU, LOOKUP_FILENAME, LFILENAME
         IF LFILENAME EQ '' THEN RETURN ELSE CURRENT_VALUES(7) = LFILENAME
      ENDIF ELSE LFILENAME = TEST_FILE(0)
      INFORM, 'Saving lookup file ' + STRTRIM(LFILENAME,2)
      SAVE_LOOKUPS, LFILENAME
      INFORM, 'Ready.'
      END

ENDCASE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; Spectral History
PRO WM_SPECTRAL_HISTORY, PLOT_IND, TYPE

@wgsp_cb.pro


N_PLOTS = N_ELEMENTS(PLOT_IND)

IF (N_PLOTS GT 25) THEN BEGIN

   INFORM, 'Cannot display more than 25 ' + TYPE + '.  Ready.'

ENDIF ELSE BEGIN

   REBIN_SPECTRA
   WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 0
   IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
      WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 0
   ENDIF
   
   PLOT_SPECTRA, [0.1, 0.28, 0.86, 0.86]
   INFORM, 'Ready.'
   
   WIDGET_CONTROL, WIDS.TOP_BASE, MAP = 1
   IF (WGSP_SEPARATE_BASE NE 0) THEN BEGIN
      WIDGET_CONTROL, WIDS.DRAW_SUB_BASE, MAP = 1
   ENDIF

ENDELSE

   

END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; Updates the Burst History and Spectrum windows in main menu
PRO WM_UPDATE

@wgsp_cb.pro


ENERGY_SCALE = READ_MENU(1)
RATE_SCALE   = READ_MENU(2)
!X.TITLE     = READ_MENU(4)
!Y.TITLE     = READ_MENU(5)
TITLE        = CREATE_LABEL(COMBINED_TIMES, TIME_SPAN, ' s')

IF (CV_RATE_MODE NE RATE_MODE_SAVE) THEN BEGIN
   INTEGRATE_SPECTRUM
   INTEGRATE_HISTORY

   COMBINE, SPECTRA_T, SPECTRA_T_ERR, THRESHOLDS, ENERGY_LOOKUP, SPECTRUM, $
            SPECTRUM_ERR, COMBINED_ENERGIES
   NIND = WHERE(SPECTRUM, COUNT)
   IF COUNT EQ 0 THEN BEGIN
      RATE_RANGE = [MIN(SPECTRUM - SPECTRUM_ERR), MAX(SPECTRUM + SPECTRUM_ERR)]
   ENDIF ELSE BEGIN
      RATE_RANGE = [MIN(SPECTRUM(NIND) - SPECTRUM_ERR(NIND)), $
                    MAX(SPECTRUM + SPECTRUM_ERR)]
   ENDELSE

   COMBINE, SPECTRA_E, SPECTRA_E_ERR, TIMES, TIME_LOOKUP, HISTORY, HISTORY_ERR, $
         COMBINED_TIMES
   HIST_RANGE = [MIN(HISTORY), MAX(HISTORY)]

   RATE_MODE_SAVE = CV_RATE_MODE
ENDIF

WSET, SPECTRUM_WINDOW
STD_DISPLAY, SPECTRUM, SPECTRUM_ERR, COMBINED_ENERGIES, $
             ENERGY_RANGE, RATE_RANGE, ENERGY_SPAN, TITLE, $
             ENERGY_SCALE, RATE_SCALE, 2, 'SPECTRUM'

!X.TITLE = READ_MENU(3)
!Y.TITLE = READ_MENU(5)
IF (CV_RATE_MODE EQ 'R') THEN BEGIN       
   !Y.TITLE = STRMID(!Y.TITLE, 0, STRPOS(!Y.TITLE, ' keV'))
ENDIF
TITLE = CREATE_LABEL(COMBINED_ENERGIES, ENERGY_SPAN, ' keV')

WSET, HISTORY_WINDOW
STD_DISPLAY, HISTORY, HISTORY_ERR, COMBINED_TIMES, $
             TIME_RANGE, HIST_RANGE, TIME_SPAN, TITLE, $
;             'LINEAR', 'LINEAR', 1, 'HISTORY'    
             READ_MENU(9), READ_MENU(10), 1, 'HISTORY'    


END 
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
PRO WM_DISPLAY_PARAMETERS, FLUN, WRITE_ONLY = WRITE_ONLY

@wgsp_cb.pro


; Text String
TS = ''  

IF KEYWORD_SET(WRITE_ONLY) THEN WRITING = 1 ELSE WRITING = 0

THE_HEADER = 'Current lookup parameters:'
TS = [TS, THE_HEADER] 
TS = [TS, '   ' + NOTES(0)]
TS = [TS, '   ' + NOTES(3) + '   ' + NOTES(4)]
TS = [TS, '   ' + NOTES(2)]
TS = [TS, '   ' + NOTES(1)]

TS = [TS, ' ']
TS = [TS, 'Observed selection start and stop times:']
N_REGIONS = SIZE(TIME_SPAN)

IF (N_REGIONS(1) - 1) GT 0 THEN BEGIN
   LOWER = 1
   UPPER = N_REGIONS(1) - 1
ENDIF ELSE BEGIN
   LOWER = 0
   UPPER = 0
ENDELSE
      
RANGE = INTARR(2)

FOR HH = LOWER, UPPER DO BEGIN
    ER = NEAREST_EDGE(COMBINED_TIMES, TIME_SPAN(HH, *), RANGE)
    EB = NEAREST_EDGE(TIMES, TIME_SPAN(HH, *), RANGE)
    TS = [TS, '   OBS_START:  ' + STRTRIM(ER(0), 2) + $
              '   OBS_STOP:  ' + STRTRIM(ER(1), 2) + $
              '   ' + STRTRIM((RANGE(1) - RANGE(0) + 1), 2) + ' Spectra']
ENDFOR

TS = [TS, '   Time Range:  (' + $
              STRTRIM(TIME_RANGE(0), 2) + ', ' + $
              STRTRIM(TIME_RANGE(1), 2) + ') seconds', $
          '   Rate Range:  (' + $
              STRTRIM(HIST_RANGE(0), 2) + ', ' + $
              STRTRIM(HIST_RANGE(1), 2) + ') counts/s-keV']

TS = [TS, ' ']
TS = [TS, 'Selected Energy bins lower and upper thresholds (keV):']
N_REGIONS = SIZE(ENERGY_SPAN)

IF (N_REGIONS(1) - 1) GT 0 THEN BEGIN
   LOWER = 1
   UPPER = N_REGIONS(1) - 1
ENDIF ELSE BEGIN
   LOWER = 0
   UPPER = 0
ENDELSE

RANGE = INTARR(2)
      
FOR HH = LOWER, UPPER DO BEGIN
    ER = NEAREST_EDGE(COMBINED_ENERGIES, ENERGY_SPAN(HH, *), RANGE)
    EB = NEAREST_EDGE(THRESHOLDS, ENERGY_SPAN(HH, *), RANGE)
    TS = [TS, '   LOWER_THRESH:  ' + STRTRIM(ER(0), 2) + $
              '   UPPER_THRESH:  ' + STRTRIM(ER(1), 2) + $
              '   ' + STRTRIM((RANGE(1) - RANGE(0) + 1), 2) + ' Bins']
ENDFOR
  
TS = [TS, '   Energy Range:  (' + $
              STRTRIM(ENERGY_RANGE(0), 2) + ', ' + $
              STRTRIM(ENERGY_RANGE(1), 2) + ') keV', $
          '   Rate Range:  (' + $
              STRTRIM(RATE_RANGE(0), 2) + ', ' + $
              STRTRIM(RATE_RANGE(1), 2) + ') counts/s-keV']
TS = [TS, ' ']

IF HAVE_BACK THEN BEGIN
   TS = [TS, 'Background selection start and stop times:']
   N_SPEC = SIZE(SPECTRA)
   N_CHAN = N_SPEC(1)
   N_TIME = N_SPEC(2)
   TR = [TRANSPOSE(TIMES(0,*)), TIMES(1, N_TIME - 1)]
   N_BACK = SIZE(BACK_SPAN)
   BACK_REGN = N_BACK(1)
   BACK_START = FLTARR(5)
   BACK_END = FLTARR(5)
      
   FOR BB = 0, BACK_REGN - 1 DO BEGIN
       I0 = SEARCH(TR, BACK_SPAN(BB, 0))
       IF I0 EQ -1 THEN I0 = 0 ELSE IF I0 EQ N_TIME THEN I0 = N_TIME - 1
       I1 = SEARCH(TR, BACK_SPAN(BB, 1))
       IF I1 EQ -1 THEN I1 = 0 ELSE IF I1 EQ N_TIME THEN I1 = N_TIME - 1
       BACK_START(BB) = TIMES(0, I0)
       BACK_END(BB) = TIMES(1, I1)
       TS = [TS, '   BACK_START:  ' + STRTRIM(BACK_START(BB), 2) + $
                 '   BACK_END:  ' + STRTRIM(BACK_END(BB), 2) + $
                 '   ' + STRTRIM(I1 - I0 + 1, 2) + ' Spectra']
   ENDFOR
         
   TS = [TS, 'Order of the background model:  ' + STRTRIM(FIX(NDEGREE), 2)]
ENDIF

; Display the text
IF (N_ELEMENTS(WRITE_ONLY) EQ 0) THEN $
   W_TEXT, TS, TITLE = 'Current Lookup Parameters'

IF NOT WRITING THEN BEGIN
   GO_ON = WIDMENU(['Write current lookup parameters to file?', 'Yes', 'No'], $
                  TITLE = 0)
   IF GO_ON THEN BEGIN
      NAME = READ_MENU(6)
      BREAK_FILE, NAME, PAR_DEV, PAR_DIR, PAR_NAME, PAR_EXT, PAR_VER
      OUT_FILE = WGSP_SCRATCH + STRLOWCASE(PAR_NAME + '.PAR')
      INFORM, 'Parameter file is ' + OUT_FILE + '.  Ready.', /NO_STATUS
      OPENW, FLUN, OUT_FILE, /GET_LUN
             WRITING = 1
             FOR I=0, N_ELEMENTS(TS)-1 DO PRINTF, FLUN, TS(I)
      FREE_LUN, FLUN
   ENDIF ELSE INFORM, 'Ready.'
ENDIF ELSE BEGIN
   FOR I=0, N_ELEMENTS(TS)-1 DO PRINTF, FLUN, TS(I)
ENDELSE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
PRO WM_WRITE_LINEFIT

@wgsp_cb.pro


NCHAN = NUM_CHAN(CDET)
FILENAME = READ_MENU(6)
BREAK_FILE, FILENAME, FDEV, FDIR, FNAME, FEXT
LINE_XFER_FILE = WGSP_SCRATCH + STRLOWCASE(FNAME + '.LXF')

INFORM, 'Writing output to Line Search file ' + $
        WGSP_SCRATCH + STRLOWCASE(FNAME + '.LXF') 

TRIG_NUM = LONG(SXPAR(HEADER, 'BATSE_TR'))
PRINT, 'TRIG_NUM: ', TRIG_NUM
DET_TYPE = STRMID(SXPAR(DH, 'DET_MODE'), 0, 1)
PRINT, 'DET_TYPE: ', DET_TYPE

FXBOPEN, UNIT, FILENAME, 1
 FXBREAD, UNIT, DATA, 1
 DET_NUM = LONG(DATA)
 PRINT, 'DET_NUM: ', DET_NUM 
 FXBREAD, UNIT, DATA, 8
 BRST_ANG = DATA
 PRINT, 'BRST_ANG: ', BRST_ANG 
 FXBREAD, UNIT, DATA, 9
 GEO_ANG = DATA
 PRINT, 'GEO_ANG: ', GEO_ANG 
CLOSE, UNIT

PRINT, 'CTOE_LABEL: ', CTOE_LABEL

PRINT, 'CALLING_PROG: ', CALLING_PROG

DATE_TIME = SYSTIME()
PRINT, 'DATE_TIME: ', DATE_TIME

NBACK_COEFF = LONG(NDEGREE + 1)
PRINT, 'NBACK_COEFF: ', NBACK_COEFF 

N_SPEC = SIZE(SPECTRA)
N_CHAN = N_SPEC(1)
N_TIME = N_SPEC(2)
TR = [TRANSPOSE(TIMES(0,*)), TIMES(1, N_TIME - 1)]
BACK_ARR = INTARR(N_TIME)
BACK_ARR(INDGEN(N_TIME)) = 0
BACK_IND = INDGEN(N_TIME)
N_BACK = SIZE(BACK_SPAN)
BACK_REGN = N_BACK(1)
BACK_START = FLTARR(5)
BACK_END = FLTARR(5)

FOR BB = 0, BACK_REGN - 1 DO BEGIN
    I0 = SEARCH(TR, BACK_SPAN(BB, 0))
    IF I0 EQ -1 THEN I0 = 0 ELSE IF I0 EQ N_TIME THEN I0 = N_TIME - 1
    I1 = SEARCH(TR, BACK_SPAN(BB, 1))
    IF I1 EQ -1 THEN I1 = 0 ELSE IF I1 EQ N_TIME THEN I1 = N_TIME - 1
    FOR KK = I0, I1 DO BACK_ARR(KK) = 1
    BACK_START(BB) = TIMES(0, I0)
    BACK_END(BB) = TIMES(1, I1)
    PRINT, 'BACK_START: ', BACK_START(BB), '  BACK_END: ', BACK_END(BB)
ENDFOR

BACK_IND = BACK_IND(WHERE(BACK_ARR))
; Done getting BACK_IND array. We can now access elements of the TIMES array 
;in the selection by using BACK_IND.

TIME_IND = INTARR(10)
COUNT_SPAN, TIMES, TIME_SPAN, TIME_IND
NUM_SHERB_RECS = LONG(N_ELEMENTS(TIME_IND))
PRINT, 'NUM_SHERB_RECS: ', NUM_SHERB_RECS 

; B_T: Background times: center of selected time intervals.
BT_BEG = TRANSPOSE(TIMES(0, TIME_IND ))     
BT_END = TRANSPOSE(TIMES(1, TIME_IND ))
 
WIDTH = THRESHOLDS(1, *) - THRESHOLDS(0, *)
GIND = WHERE(WIDTH GT 0.0)

Y_CURVES = SPECTRA(*, TIME_IND)
DY_CURVES = SPECTRA_ERRORS(*, TIME_IND)

NUM_TIMES = LONG(N_ELEMENTS(TIME_IND))
IF WGSP_NO_MFIT THEN BEGIN
   BACK_SPECS = TRANSPOSE(BACK_EVAL_MODEL(BT_BEG, BT_END))
ENDIF ELSE BEGIN
   TEMP_FIT = FLTARR(NUM_TIMES, MAX_CHAN)
   STATUS = CALL_EXTERNAL('MFIT_SHARE', 'BACK_EVAL_MODEL', NUM_TIMES, $
                     BT_BEG, BT_END, TEMP_FIT)
   BACK_SPECS = TRANSPOSE(TEMP_FIT(*, 0: NCHAN - 1))
ENDELSE

IF WGSP_NO_MFIT THEN BEGIN
   BACK_ERR_SPECS = TRANSPOSE(BACK_EVAL_MSIG(BT_BEG, BT_END))
ENDIF ELSE BEGIN
   TEMP_ERRS = FLTARR(NUM_TIMES, MAX_CHAN)
   AFUNC = FLTARR(5, NUM_TIMES)
   STATUS = CALL_EXTERNAL('MFIT_SHARE', 'BACK_EVAL_MSIG', NUM_TIMES, $
               BT_BEG, BT_END, AFUNC, TEMP_ERRS)
   BACK_ERR_SPECS = TRANSPOSE(TEMP_ERRS(*, 0: NCHAN - 1))
ENDELSE

LIVE_TIME = FLTARR(N_CHAN, NUM_SHERB_RECS)
FOR I = 0, NUM_SHERB_RECS - 1 DO BEGIN
   NIND = WHERE(DY_CURVES(*, I))
   TEMP_LIVE_TIME = Y_CURVES(NIND, I) / $
                       (DY_CURVES(NIND, I) * DY_CURVES(NIND, I))
   LIVE_TIME(*, I) = TOTAL(TEMP_LIVE_TIME) / N_ELEMENTS(NIND)
   Y_CURVES(GIND, I) = Y_CURVES(GIND, I) / WIDTH(GIND)
   BACK_SPECS(GIND, I) = BACK_SPECS(GIND, I) / WIDTH(GIND) 
   BACK_ERR_SPECS(GIND, I) = BACK_ERR_SPECS(GIND, I) / WIDTH(GIND) 
   ENDFOR

OPENW, UNIT2, LINE_XFER_FILE, /GET_LUN, /SEGMENTED
 WRITEU, UNIT2, TRIG_NUM, DET_TYPE, DET_NUM, BRST_ANG, GEO_ANG, CTOE_LABEL, $
                CALLING_PROG, DATE_TIME
 WRITEU, UNIT2, NBACK_COEFF, BACK_START, BACK_END, NUM_SHERB_RECS
 WRITEU, UNIT2, TIMES(0, TIME_IND), TIMES(1, TIME_IND)
 FOR I = 0, NUM_SHERB_RECS - 1 DO BEGIN
    WRITEU, UNIT2, Y_CURVES(*, I), BACK_SPECS(*, I), BACK_ERR_SPECS(*, I), $
                   LIVE_TIME(*, I)
    ENDFOR
FREE_LUN, UNIT2


END; PRO WRITE_LINEFIT
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
PRO WM_CLEANUP

@wgsp_cb.pro


IF N_ELEMENTS(LAYOUT_WINDOW)      NE 0 THEN WDELETE, LAYOUT_WINDOW
IF N_ELEMENTS(WORK_WINDOW)        NE 0 THEN WDELETE, WORK_WINDOW
IF N_ELEMENTS(PLOT_WINDOW)        NE 0 THEN WDELETE, PLOT_WINDOW
IF N_ELEMENTS(PROCESSED_HISTORY)  NE 0 THEN WDELETE, PROCESSED_HISTORY
IF N_ELEMENTS(PROCESSED_SPECTRUM) NE 0 THEN WDELETE, PROCESSED_SPECTRUM



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
PRO WARN_NO_DATA, STATUS, NO_DIALOG = NO_DIALOG

COMMON FITS_C, HEADER, CH, DH


; Make sure data has been read
IF (N_ELEMENTS(HEADER) EQ 0) THEN BEGIN
   IF (N_ELEMENTS(NO_DIALOG) EQ 0) THEN BEGIN
      DATA_ERR = WIDMENU(['Please enter a data file name', 'OK'], TITLE = 0)
   ENDIF
   STATUS = 'NO DATA'
ENDIF ELSE BEGIN
   STATUS = 'OK'
ENDELSE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; END W_MAIN_PROCS 
