; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
FUNCTION M_LOG_RND_DOWN, IN_VALUE, AXIS_TYPE
;C
;C     Function output M_LOG_RND_DOWN is input value IN_VALUE rounded down
;C     to either a power of 10 or to a minor tickmark (e.g., 10, 20, 30,..).
;C
;C     If control variable CV_GRAPH_LIMIT_MODE = 'P', we round down to
;C     a power of 10.
;C     If control variable CV_GRAPH_LIMIT_MODE = 'M', we round down to a
;C     minor tickmark.
;C     If control variable CV_GRAPH_LIMIT_MODE = 'B', then
;C        if AXIS_TYPE = 'E' we round down to a minor tickmark,
;C        otherwise we round down to a power of 10.
;C
;C     Minor tickmark limits are found as follows:
;C     1) M_LOG_RND_DOWN is of the form D x 10**P where D is a postive digit
;C        from 1 to 9 and P is any integer, and
;C     2) M_LOG_RND_DOWN is the largest such number <= IN_VALUE.
;C 
;C     Michael S. Briggs, UAH / MSFC ES-64, Sept 1993.
;C
;C     Input arguments:
;      REAL*4 IN_VALUE                                 ! should be positive
;      CHARACTER*1 AXIS_TYPE
;C
;C     Output via function value M_LOG_RND_DOWN.
;C
;C     declare the control variables:
COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

;C     Internal variables:
;      REAL*4 VALUE
;      INTEGER*4 P
;      INTEGER*4 D
;C
;C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;C
;C
;C     Obtain the values of the control variables, esp. CV_GRAPH_LIMIT_MODE:
;      INCLUDE 'mfit_SRC_dir:MGET_CONTROL.FOR_FRAG'
;C
;C
      VALUE = MAX ([IN_VALUE, 1.E-37])
;C
; ARRRGH - RSM 6 Feb, 1995  
;          D, P MUST be an integers; ALOG10 automatically converts to FLOAT
;      P = ALOG10 (VALUE)
      P = FIX(ALOG10 (VALUE))
      IF (VALUE LT 1.0) THEN P = P - 1
;C
      IF (CV_GRAPH_LIMIT_MODE EQ 'P' OR   $
         (CV_GRAPH_LIMIT_MODE EQ 'B' AND AXIS_TYPE NE 'E')) THEN BEGIN
         M_LOG_RND_DOWN = 10.^P
      ENDIF ELSE BEGIN
;         D = VALUE / 10.^P 
         D = FIX(VALUE / 10.^P) 
         M_LOG_RND_DOWN = D * 10.^P
         ENDELSE
;C
      RETURN, M_LOG_RND_DOWN
      END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
FUNCTION M_LOG_RND_UP, IN_VALUE, AXIS_TYPE
;C
;C     Function output M_LOG_RND_UP is the input value IN_VALUE rounded up
;C     to either a power of 10 or to a minor tickmark (e.g., 10, 20, 30, ...)
;C
;C     If control variable CV_GRAPH_LIMIT_MODE ='P',we round up to a power of 10,
;C     otherwise we round up to a minor tickmark.
;C
;C     Minor tickmark limits are found as follows:
;C     1) M_LOG_RND_UP is of the form D x 10**P where D is a postive digit
;C        from 1 to 9 and P is any integer, and
;C     2) M_LOG_RND_UP is the smallest such number >= IN_VALUE.
;C 
;C     Michael S. Briggs, UAH / MSFC ES-64, September 1993.
;C
;C     Input arguments:
;      REAL*4 IN_VALUE                               ! should be positive
;      CHARACTER*1 AXIS_TYPE                              ! not used
;C
;C     Output via function value M_LOG_RND_UP.
;C
;C     declare the control variables:
;      INCLUDE 'mfit_dir:MFIT_CONTROL_DECLARE.INCL'
;C
;C     Internal variables:
;      REAL*4 VALUE
;      INTEGER*4 P
;      INTEGER*4 D
;C
;C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;C
;C   
;C     Obtain the values of the control variables, esp. CV_GRAPH_LIMIT_MODE:
COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

;C
      VALUE = MAX ([IN_VALUE, 1.E-37])
;C
; ARRRGH - RSM 6 Feb, 1995  
;          D, P MUST be an integers; ALOG10 automatically converts to FLOAT
;      P = ALOG10 (VALUE)
      P = FIX(ALOG10 (VALUE))
      IF (VALUE LT 1.0) THEN P = P - 1
;C
      IF (CV_GRAPH_LIMIT_MODE EQ 'P') THEN BEGIN
         M_LOG_RND_UP = 10.^P
      ENDIF ELSE BEGIN
;         D = VALUE / 10.^P + 1.
         D = FIX(VALUE / 10.^P + 1.)
         M_LOG_RND_UP = D * 10.^P
         ENDELSE
;C
      RETURN, M_LOG_RND_UP
      END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
FUNCTION M_ROUND_BEG, VALUE, LOG_FLAG, MODE
;C
;C     Function to round begining axis limit.
;C     Michael S. Briggs, 8 Dec 1993, UAH / MSFC ES-66.
;C     Output solely via function return.
;C
;C     Input arguments:
;      REAL*4 VALUE                               ! value to round
;      LOGICAL*4 LOG_FLAG                         ! if TRUE, log axis
;      CHARACTER*1 MODE
;C           
;C     Internal variables:
;      REAL*4 M_LOG_RND_UP, M_LOG_RND_DOWN              ! functions for rounding
;C
;C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;C                                         
      IF (LOG_FLAG) THEN BEGIN
;C        log axis: round the positive number down:
         M_ROUND_BEG = M_LOG_RND_DOWN (MAX ([VALUE, 1.E-9]), MODE)
      ENDIF ELSE BEGIN
         IF (VALUE GT 0.0) THEN BEGIN
;C           linear axis: round positive number down:
            M_ROUND_BEG = M_LOG_RND_DOWN (VALUE, MODE)
         ENDIF ELSE BEGIN
;C           linear axis: round negative number down by rounding its opposite up:
            M_ROUND_BEG = -M_LOG_RND_UP (-VALUE, MODE)
         ENDELSE
;C        if linear limit is just above zero, convert it to zero:
         IF (M_ROUND_BEG GT 0.0 AND M_ROUND_BEG LE 1.E-10) THEN BEGIN
            M_ROUND_BEG = 0.0
            ENDIF
      ENDELSE
;C
      RETURN, M_ROUND_BEG
      END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
FUNCTION M_ROUND_END, VALUE, LOG_FLAG, MODE
;C
;C     Function to round ending axis limit.
;C     Michael S. Briggs, 8 Dec 1993, UAH / MSFC ES-66.
;C
;C     Input arguments:
;      REAL*4 VALUE                               ! value to round
;      LOGICAL*4 LOG_FLAG                         ! if TRUE, log axis
;      CHARACTER*1 MODE
;C           
;C     Internal variables:
;      REAL*4 M_LOG_RND_UP                    ! function for rounding
;C
;C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;C     
      M_ROUND_END = M_LOG_RND_UP (VALUE, MODE)
;C
      RETURN, M_ROUND_END
      END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      PRO M_REBIN_ROUND, LDET, CHAN_OFFSET, USED_REBIN, REBIN_CHAN, $
                             FIRST_CHAN, LAST_CHAN
; Defines parameters MAX_ & AMAX_
;COMMON PARAM_C, MAX_CHAN, MAX_EBINS, MAX_PLOT, MAX_TERMS, MAX_CHI_1D, $
;          MAX_CHI_2D, MAX_PLOT_TYPE, MAX_PARAM, MAX_DET

;     This subroutine "rounds" the channel (instrument units) until they
;     match the boundaries of the rebinned channels.    The channels may
;     be rebinned into larger channels in order to improve the error bars, etc.
;     Channels are rounded inwards, i.e., FIRST_CHAN is rounded up and
;     LAST_CHAN is rounded down.
;     The variables FIRST_CHAN and LAST_CHAN in raw or instrument channels.
;     The REBIN_CHAN lists the starting raw channel numbers of the rebinned
;     channels, plus one extra element at the end = number of channels plus
;     one.    The variable CHAN_OFFSET which lists the instrument channel
;     number of the first instrument channel being processed applies to
;     FIRST_CHAN and LAST_CHAN, but not to REBIN_CHAN, which is really a 
;     list of indices.
;     Typically, FIRST_CHAN will be FIRST_CHAN_FIT or FIRST_CHAN_PLOT, etc.
; 
;     Michael S. Briggs, 13 December 1993, UAH / MSFC ES-66.
; 
;     RSM, Nov, 1994 - Converted from FORTRAN to IDL
;
;     Examples are all for 16 channel data:
;
;     Ex 1: 
;     CHAN_OFFSET = 0 so channels 0-15 are available.
;     No rebinning: 
;     USED_REBIN = NUM_CHAN + 1 = 17
;     REBIN_CHAN = 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16
;     No legal values of FIRST_CHAN nor LAST_CHAN would be altered.
;
;     Ex 2:
;     CHAN_OFFSET = 0 so channels 0-15 are available.
;     Rebinned 4 raw channels per rebinned channel:
;     i.e., 0-3, 4-7, 8-11, 12-15
;     USED_REBIN = 5
;     REBIN_CHAN = 0, 4, 8, 12, 16
;           old FIRST    LAST,     new FIRST   LAST
;                 4       11             4      11
;                 4        7             4       7
;                 2        9             4       7
;                 0       15             0      15
;                10       15            12      15
;                 
;     Ex 3:
;     16 channel data omitting first 4 channels:
;     CHAN_OFFSET = 4 so channels 4-15 are available.
;     Rebinned 4 raw channels per rebinned channel:
;     i.e., 4-7, 8-11, 12-15
;     USED_REBIN = 4
;     REBIN_CHAN = 0, 4, 8, 12
;           old FIRST    LAST,     new FIRST   LAST
;                 4       11             4      11
;                 4        7             4       7
;                 5       14             8      11
;


; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

;     Increase FIRST_CHAN until it matches the starting channel of one of
;     the super channels:

      INDEX = 0
      WHILE $
        (REBIN_CHAN (INDEX, LDET) LT FIRST_CHAN (LDET) - CHAN_OFFSET (LDET) $
                 AND INDEX LT USED_REBIN (LDET) - 2) DO BEGIN
         INDEX = INDEX + 1
      ENDWHILE
      FIRST_CHAN (LDET) = REBIN_CHAN (INDEX, LDET) + CHAN_OFFSET (LDET)
      FIRST_INDEX = INDEX

;     Decrease LAST_CHAN until it matches the ending channel of one of the
;     super channels:

      INDEX = USED_REBIN (LDET) - 1
      WHILE $
        (REBIN_CHAN (INDEX, LDET) - 1 GT LAST_CHAN (LDET) - CHAN_OFFSET (LDET) $
                  AND INDEX GT FIRST_INDEX) DO BEGIN
         INDEX = INDEX - 1
      ENDWHILE
      LAST_CHAN (LDET) = REBIN_CHAN (INDEX, LDET) + CHAN_OFFSET (LDET) - 1


      RETURN
      END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; The two routine convert string arrays between IDL's internal representation
; and DEC FORTRAN internal representation.   The routine IDL_TO_FOR is used
; to convert a string array before calling a FORTRAN routine via CALL_EXTERNAL,
; while the routine FOR_TO_IDL is used after the CALL_EXTERNAL.
; There is no incompatibility between IDL and DEC FORTRAN for scaler strings,
; but string arrays are incompatible.
; IDL string arrays are arrays of strings, each string preceded by a descriptor
; of being of variable length.   FORTRAN character arrays are stored as a
; single character variable with only one descriptor--the compiler must find
; the individual elements as substrings--this is possible because the compiler
; reserves storage for the entire array using the declared (=maximum) size of
; the string.   The FORTRAN compiler uses blanks for undefined or unused
; portions of strings (scaler or array).
; Michael S. Briggs, UAH/ES-84, 27 June 1994.
;
FUNCTION IDL_TO_FOR, IDL_ARRAY, NUM_ELEM, ELEM_LEN

; check legality of arguments:
IF (NUM_ELEM NE N_ELEMENTS (IDL_ARRAY)) THEN   $
   PRINT, 'IDL_TO_FOR: BAD ARGUMENTS RE NUMBER OF ELEMENTS'
IF (MAX (STRLEN (IDL_ARRAY)) GT ELEM_LEN) THEN   $
   PRINT, 'IDL_TO_FOR: BAD ARGUMENTS RE MAXIMUM STRING LENGTH'

; Create FORTRAN "array" as a scalar of the correct total size:
FORTRAN_ARRAY = STRING (REPLICATE (32B, NUM_ELEM * ELEM_LEN))

; Load the elements into the array:
FOR I=0, NUM_ELEM - 1 DO BEGIN
    STRPUT, FORTRAN_ARRAY, IDL_ARRAY (I), I*ELEM_LEN
ENDFOR

RETURN, FORTRAN_ARRAY
END;   IDL_TO_FOR


FUNCTION FOR_TO_IDL, FORTRAN_ARRAY, NUM_ELEM, ELEM_LEN

; check arguments for legality:
IF (STRLEN (FORTRAN_ARRAY) NE NUM_ELEM * ELEM_LEN) THEN   $
   PRINT, 'FOR_TO_IDL: BAD ARGUMENTS'

; create the IDL string array:
IDL_ARRAY      = REPLICATE(STRING(REPLICATE(32B, ELEM_LEN)), NUM_ELEM)
IDL_ARRAY_HOLD = STRING(REPLICATE(32B, ELEM_LEN))

; extract the FORTRAN array elements as substrings and make into IDL array
; elements:
FOR I=0, NUM_ELEM - 1 DO BEGIN
   ELEMENT = STRMID (FORTRAN_ARRAY, I*ELEM_LEN, ELEM_LEN)
   STRPUT, IDL_ARRAY_HOLD, ELEMENT
   IDL_ARRAY (I) = IDL_ARRAY_HOLD
ENDFOR


RETURN, IDL_ARRAY
END;   FOR_TO_IDL
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; Widget version of WMENU
; The initial selection, if not specified, is set to 
;  be the first valid selection 
; Set keyword SCROLL to add scroll bar 
;
; R.S. Mallozzi, UAH/NASA, Mar, 1994

; NOTE: THIS PROCEDURE HAS BEEN CUSTOMIZED FOR WINGSPAN
; NOTE: THIS PROCEDURE HAS BEEN CUSTOMIZED FOR TO ALLOW OVERPLOTTING OF BURST
;        TIME HISTORY


FUNCTION TIME_HIST_WIDMENU, SELECTIONS, $
         TITLE = TITLE, SCROLL = SCROLL, INITIAL = INITIAL

COMMON TWIDS, WIDS
COMMON WGSP_GET_RANGE, WLO_X, WHI_X, $
            NUMERICAL_XRANGE, NUMERICAL_YRANGE, OPLOT_TIME_HIST


DEVICE_SAVE = !D.NAME

DEVICE, RETAIN = 2
IF WIDS.TOP_BASE NE 0 THEN $
   WIDGET_CONTROL, WIDS.TOP_BASE, SENSITIVE = 0
IF WIDS.WORK_BASE NE 0 THEN $
   WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 0
IF WIDS.MFIT_BASE NE 0 THEN $
   WIDGET_CONTROL, WIDS.MFIT_BASE, SENSITIVE = 0

NUM_BUTTONS = N_ELEMENTS(SELECTIONS)

IF N_ELEMENTS(TITLE) NE 0 THEN BEGIN
   IF TITLE GT NUM_BUTTONS THEN TITLE = NUM_BUTTONS - 1
   TTITLE = SELECTIONS(TITLE)
   LOCAL_SELECTIONS = SELECTIONS(WHERE(INDGEN(NUM_BUTTONS) NE TITLE))
   TCHECK = 1
ENDIF ELSE BEGIN
   TTITLE = ''
   LOCAL_SELECTIONS = SELECTIONS
   TCHECK = 0
ENDELSE

; Parent
MENU_BASE = WIDGET_BASE(TITLE=' ', /COLUMN, MAP = 0)

 ; Add the title if it's supplied
 IF N_ELEMENTS(TITLE) NE 0 THEN T_BASE = WIDGET_LABEL(MENU_BASE, VALUE=TTITLE)

 ; Selection buttons
 IF KEYWORD_SET(SCROLL) THEN BEGIN
    SUB_BASE = WIDGET_BASE(MENU_BASE, /COLUMN, /FRAME, /SCROLL)
 ENDIF ELSE BEGIN
    SUB_BASE = WIDGET_BASE(MENU_BASE, /COLUMN, /FRAME)
 ENDELSE

 M_BUT = LONARR(NUM_BUTTONS)
 FOR i=0, N_ELEMENTS(LOCAL_SELECTIONS)-1 DO BEGIN
     M_BUT(i) = WIDGET_BUTTON(SUB_BASE, VALUE = LOCAL_SELECTIONS(i), UVALUE = i)
 ENDFOR

 ; Add a radio button for oplotting burst time history
 JUNK = WIDGET_BASE(MENU_BASE, /ROW)
 R_LABEL = WIDGET_LABEL(JUNK, VALUE = 'Overplot Burst Time History')
 RB_LABELS = ['OFF', 'ON']
 R_BUT = CW_BGROUP(JUNK, RB_LABELS, /ROW, $
            /EXCLUSIVE, /NO_RELEASE, UVALUE = 'RBUTTONS')

; Realize the BASE widget and the children
WIDGET_CONTROL, MENU_BASE, /REALIZE

; Place the menu in center of screen
DEVICE, GET_SCREEN_SIZE = CURRENT_SCREEN
WIDGET_CONTROL, MENU_BASE, TLB_GET_SIZE = MENU_SIZE
MENU_CENTER = [(CURRENT_SCREEN(0) / 2.0) - (MENU_SIZE(0) / 2.0), $
               (CURRENT_SCREEN(1) / 2.0) - (MENU_SIZE(1) / 2.0)]
WIDGET_CONTROL, MENU_BASE, $
                TLB_SET_XOFFSET = MENU_CENTER(0), $
                TLB_SET_YOFFSET = MENU_CENTER(1)
WIDGET_CONTROL, MENU_BASE, MAP = 1

; Place the cursor 
LNUM = N_ELEMENTS(LOCAL_SELECTIONS)
IF N_ELEMENTS(INITIAL) NE 0 THEN BEGIN
   IF TCHECK EQ 1 THEN BEGIN
      CASE 1 OF
           (TITLE GE INITIAL) : SPOT = INITIAL
           (TITLE LT INITIAL) : SPOT = INITIAL - 1
           (INITIAL GT LNUM)  : SPOT = LNUM - 1 
           ELSE               : SPOT = 0
      ENDCASE
   ENDIF ELSE BEGIN
      CASE 1 OF
           (INITIAL GT LNUM) : SPOT = LNUM -1 
           ELSE              : SPOT = INITIAL
      ENDCASE
   ENDELSE
   IF SPOT LT 0 THEN SPOT = 0
ENDIF ELSE BEGIN
   SPOT = 0
ENDELSE
WIDGET_CONTROL, M_BUT(SPOT), /INPUT_FOCUS
WIDGET_CONTROL, R_BUT, SET_VALUE = OPLOT_TIME_HIST

; Loop until the widget is /DESTROYED
WHILE 1 DO BEGIN

EVENT = WIDGET_EVENT(MENU_BASE)

TYPE = TAG_NAMES(EVENT, /STRUCTURE)
CASE TYPE OF

 ; The button widget events
 'WIDGET_BUTTON': BEGIN
                   WIDGET_CONTROL,EVENT.ID,GET_VALUE = VALUE,GET_UVALUE = UVALUE
                   
                   FOR I=0, N_ELEMENTS(LOCAL_SELECTIONS)-1 DO BEGIN
                       IF UVALUE EQ I THEN BEGIN
                          WIDGET_CONTROL, EVENT.TOP, /DESTROY
                          IF TCHECK EQ 1 THEN BEGIN
                             IF I GE TITLE THEN ADD_I = TCHECK ELSE ADD_I = 0
                          ENDIF ELSE ADD_I = 0
                          IF WIDS.TOP_BASE NE 0 THEN $
                             WIDGET_CONTROL, WIDS.TOP_BASE, SENSITIVE = 1
                          IF WIDS.WORK_BASE NE 0 THEN $
                                  WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 1
                          IF WIDS.MFIT_BASE NE 0 THEN $
                                  WIDGET_CONTROL, WIDS.MFIT_BASE, SENSITIVE = 1
                          SET_PLOT, DEVICE_SAVE
                          RETURN, I + ADD_I
                       ENDIF
                   ENDFOR
                  END
 ; The radio button widget events
 '': BEGIN
     WIDGET_CONTROL, EVENT.ID, GET_UVALUE = UVALUE
     CASE UVALUE OF 
          'RBUTTONS': BEGIN
                      CASE EVENT.VALUE OF
                           0: OPLOT_TIME_HIST = 0
                           1: OPLOT_TIME_HIST = 1
                      ENDCASE
                      END
     ENDCASE
     END

ENDCASE ; for TYPE

ENDWHILE



END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; ROUND_AXIS: Uses the M_ROUND_ routines to return a rounded axis, based upon
;the keywords: /POWER rounds out to the nearest power of ten,
;              /MINOR rounds out to the nearest minor tickmark.
PRO ROUND_AXES, THE_RANGE, THE_TYPE, POWER = POWER, MINOR = MINOR


E_AXIS = STRING(FORMAT= '(A1)', 'E')

IF THE_TYPE THEN BEGIN
   IF KEYWORD_SET(POWER) THEN $
      E_AXIS = STRING(FORMAT= '(A1)', ' ')
ENDIF 


; RSM: 2 July, 1996, check if value is already a power of 10; if value 
;      is LE than TOL, then round it
TOL = 1.0E-3

IF (THE_RANGE(0) LE TOL) THEN BEGIN
   THE_RANGE(0) = M_ROUND_BEG(THE_RANGE(0), THE_TYPE, E_AXIS)
ENDIF ELSE BEGIN
   IF ((KEYWORD_SET(POWER)) AND ((THE_RANGE(0) MOD 10.0) GT TOL)) THEN $
      THE_RANGE(0) = M_ROUND_BEG(THE_RANGE(0), THE_TYPE, E_AXIS)
ENDELSE

IF (THE_RANGE(1) LE TOL) THEN BEGIN
   THE_RANGE(1) = M_ROUND_BEG(THE_RANGE(1), THE_TYPE, E_AXIS)
ENDIF ELSE BEGIN
   IF ((KEYWORD_SET(POWER)) AND ((THE_RANGE(1) MOD 10.0) GT TOL)) THEN $
      THE_RANGE(1) = M_ROUND_END(THE_RANGE(1), THE_TYPE, E_AXIS)
ENDELSE



END;  PRO ROUND_AXIS
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
