; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
function fxfits_notes, header, notes, n_notes
n_notes = 0
w = where(strpos(header, 'NOTE') eq 0)
if w(0) eq -1 then n_notes = 0 $
else begin
   n_notes = n_elements(w)
   notes = strarr(n_notes)
   endelse
for i = 0, n_notes - 1 do begin
   notes(i) = strmid(header(w(i)), 10, 80)
   quote = where(byte(notes(i)) eq 39)

   if quote(0) ne -1 then begin
      notes(i) = strmid(notes(i), quote(0) + 1, quote(1) - 1)
      endif
   endfor
return, 'O.K.'
end; function fxfits_notes
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
function fxopen_fits, filename, filetype, trigger, header
on_ioerror, error
IO_OK = EXECUTE('header = headfits(filename)')

; HEADFITS was modified to return 'IO_ERROR' when there is an error.
;  The FITS library routine will not trap this error -- use the
;  modified HEADFITS in the WIGSPAN/MFIT package.
if (N_ELEMENTS(header) eq 1) then begin
   if (STRUPCASE(header) eq 'IO_ERROR') then IO_OK = 0
endif

IF NOT IO_OK THEN GOTO, ERROR
trigger = sxpar(header, 'TRIG-DAY')*Double(86400) + sxpar(header, 'TRIG-TIM')
filetype = sxpar(header, 'FILETYPE')

error:
   message = 'O.K.'
   if !err lt 0 then message = !err_string
return, message
end; function fxopen_fits
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
function fxread_spectra, filename, times, thresholds, spectra, errors, $
                       cal_name, cal_header, data_header

; For MP paper correction only:
COMMON angles, zen_angle, det_mode

on_ioerror, error
fxbopen, unit, filename, 1, cal_header
length = sxpar(cal_header, 'NAXIS2')
dets = intarr(length)
starts = dblarr(length)
stops = dblarr(length)
cal_name = strarr(length)

tform4 = sxpar(cal_header, 'TFORM4')
n_channels = strmid(tform4, 0, strpos(tform4, 'E')) - 1
thresholds = fltarr(n_channels + 1, length)

fxbread, unit, dets, 1
fxbread, unit, starts, 2
fxbread, unit, stops, 3
fxbread, unit, thresholds, 4

; For MP paper correction only:
fxbread, unit, angles, 8

keywords = fxpar(cal_header, 'TTYPE*',count=num_types)
ndex = where(keywords(*) eq 'CAL_NAME')
if ndex(0) eq -1 then ndex(0) = 7 ; Handle old files!
fxbread, unit, cal_labels, ndex(0) + 1
fxbclose, unit                             ; Close the file!

fxbopen, unit, filename, 2, data_header

dsel = sxpar(data_header, 'DSELECT')
w = where(byte(dsel) eq 89)           ;Parse the string for occurances of 'Y'
detectors = 7 - w                     ;Detector number is the complement.
num_dets = n_elements(detectors)

; For MP paper correction only:
zen_angle = total(angles) / num_dets  ; Average detector zenith angle for MP
                                      ;paper correction.
det_mode = STRTRIM(sxpar(data_header, 'DET_MODE'), 2) ; LAD or SD

cal_name = ''                         ;Construct the CTOE_LABEL to return
t = fltarr(n_channels + 1)
for i = 0, num_dets - 1 do $          ;Average thresholds over detectors
   for j = 0 , length - 1 do $
      if detectors(i) eq dets(j) then begin   ;Detectors are 'indexed' by dets
         t = t + thresholds(*, j)
         cal_name = cal_name + strtrim(cal_labels(j), 2) + ' '
         endif
thresholds = fltarr(2, n_channels)
thresholds(0, *) = t(0:n_channels - 1)/num_dets
thresholds(1, *) = t(1:n_channels)/num_dets

length = sxpar(data_header, 'NAXIS2')
times = fltarr(2, length)
spectra = fltarr(n_channels, length)
errors = fltarr(n_channels, length)

fxbread, unit, times, 1
fxbread, unit, spectra, 2
fxbread, unit, errors, 3

error:
   message = 'O.K.'
   if !err lt 0 then message = !err_string
fxbclose, unit                             ; Close the file!
return, message
end; function fxread_spectra
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO READ_COMPRESSED, FILENAME, NUMEBINS, E_EDGES, NUMCHAN, CH_EDGE, $
                     MAT_TYPE, RC_DRM, SCAT, IO_ERR = IO_ERR

COMMON DRM_C, CH_OUT, E_IN, DRM, DRM_LABEL, DRM_DIM1, $
              DRM_DIM2, DRM_DIM3

COMMON TWIDS, WIDS

; For MP paper correction only:
COMMON angles, zen_angle, det_mode


PRIM_HEAD = HEADFITS(FILENAME)
IF KEYWORD_SET(IO_ERR) THEN BEGIN
   FILETYPE = SXPAR(PRIM_HEAD, 'FILETYPE')
   IF FILETYPE NE 'BATSE_DRM' THEN BEGIN
      IO_ERR = -1
      RETURN
      ENDIF
   ENDIF
FXBOPEN, UNIT, FILENAME, 1, DRM_HEAD
NUM_DRM = SXPAR(DRM_HEAD, 'NAXIS2')
FXBREAD, UNIT, MAT_TYPE, 2, 1
NUMEBINS = LONG(SXPAR(PRIM_HEAD, 'N_E_BINS'))
NUMCHAN  = LONG(SXPAR(PRIM_HEAD, 'N_E_CHAN'))

THE_MESS = 'Reading FITS DRM file ' 
DO_FIX = 2
IF DET_MODE EQ 'SD' THEN BEGIN
;   DO_FIX = WIDMENU(['Add millipore paper correction?', 'Yes', 'No'], TITLE = 0)
   DO_FIX = 1
   IF DO_FIX THEN $
      THE_MESS = 'Reading & Fixing FITS DRM file ' 
ENDIF

IF N_ELEMENTS(WIDS.MFIT_BASE) NE 0 THEN BEGIN
   IF WIDS.MFIT_BASE NE LONG(0) THEN $
      MFIT_INFORM, THE_MESS + STRTRIM(FILENAME, 2) $
   ELSE $
      INFORM, THE_MESS + STRTRIM(FILENAME, 2)
ENDIF ELSE BEGIN
   INFORM, THE_MESS + STRTRIM(FILENAME, 2)
ENDELSE

RC_DRM = FLTARR(NUMEBINS, NUMCHAN)
IF MAT_TYPE EQ 2 THEN $
   SCAT = FLTARR(NUMEBINS, NUMCHAN)
E_EDGES = FLTARR(NUMEBINS + 1)
CH_EDGE = FLTARR(NUMCHAN + 1)
N_ZEROS = INTARR(NUMCHAN)

FOR J = 1, NUM_DRM DO BEGIN
   FXBREAD, UNIT, DUMMY, 10, J
   E_EDGES = E_EDGES + DUMMY
   FXBREAD, UNIT, DUMMY, 11, J
   CH_EDGE = CH_EDGE + DUMMY
   FXBREAD, UNIT, N_ZEROS, 12, J

   IF MAT_TYPE EQ 0 THEN $
      FXBREAD, UNIT, COMPRESSED, 13, J $
   ELSE IF MAT_TYPE EQ 1 THEN $
      FXBREAD, UNIT, COMPRESSED, 14, J $
   ELSE IF MAT_TYPE EQ 3 THEN $
      FXBREAD, UNIT, COMPRESSED, 15, J $
   ELSE IF MAT_TYPE EQ 2 THEN BEGIN
      FXBREAD, UNIT, COMPRESSED, 13, J
      FXBREAD, UNIT, SCATTERED, 14, J
      ENDIF

   UPPER = NUMEBINS - 1

   INDEX = LONG(0)
   FOR I = 0, NUMCHAN - 1 DO BEGIN
      LOWER = N_ZEROS(I) - 1
      RC_DRM(LOWER: UPPER, I) = RC_DRM(LOWER: UPPER, I) + $
                             COMPRESSED(INDEX: INDEX + LONG(UPPER - LOWER))
      INDEX = INDEX + LONG(NUMEBINS - LOWER)
      ENDFOR

   IF MAT_TYPE EQ 2 THEN BEGIN
      INDEX = LONG(0)
      FOR I = 0, NUMCHAN - 1 DO BEGIN
         LOWER = N_ZEROS(I) - 1
         SCAT(LOWER: UPPER, I) = SCAT(LOWER: UPPER, I) + $
                                 SCATTERED(INDEX: INDEX + LONG(UPPER - LOWER))
         INDEX = INDEX + LONG(NUMEBINS - LOWER)
         ENDFOR
      ENDIF
   ENDFOR

E_EDGES = E_EDGES / NUM_DRM
CH_EDGE = CH_EDGE / NUM_DRM
FXBCLOSE, UNIT


IF N_ELEMENTS(WIDS.MFIT_BASE) NE 0 THEN BEGIN
   IF WIDS.MFIT_BASE NE LONG(0) THEN $
      MFIT_INFORM, '' $
   ELSE $
      INFORM, '' 
ENDIF ELSE BEGIN
   INFORM, ''
ENDELSE

BREAK_FILE, FILENAME, DRM_DEV, DRM_DIR, DRM_NAME, DRM_EXT, DRM_VERSION
DRM_LABEL = STRING(REPLICATE(32B, 70))

; For MP paper correction only:
IF DO_FIX THEN BEGIN
   E_MID = (E_EDGES(0: NUMEBINS - 1) + E_EDGES(1: NUMEBINS)) / 2.
   TRANS = EXP(-244.5 / COS(ZEN_ANGLE * !DTOR) * E_MID ^ (-2.95))
   FOR II = 0, NUMCHAN - 1 DO RC_DRM(*, II) = RC_DRM(*, II) * TRANS(*)
   STRPUT, DRM_LABEL, DRM_NAME + DRM_EXT + ' mp fixed'
ENDIF ELSE BEGIN
   STRPUT, DRM_LABEL, DRM_NAME + DRM_EXT
ENDELSE


END; PRO READ_COMPRESSED
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
