; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + 
PRO M_SELECT_CNTS, NUM_CHAN, NUM_DET, $
        OBS_CRATE, BACK_CRATE, LIVE_TIME, DET_NAMES, $
        CHAN_OFFSET, FIRST_CHAN_PLOT, LAST_CHAN_PLOT, $
        CNT_RATE_SIG, YLOG, YBEG, YEND, HAVE_YAXIS_LIMITS, $
        ADJUST = ADJUST

; This subroutine determines the Y-axis limits for count rate plots, i.e.
;  the values of YBEG (*, 1) and YEND (*, 1).
;  Michael S. Briggs, UAH / MSFC ES-64, 17 November 1992.
;  The plot limits are based upon the data and sigma estimated from the data,
;  whereas the actual plots use sigma estimated from the model, if the
;  model is available.     M. Briggs, 11 July 1993.
;
; RSM, Nov, 1994 - converted from FORTRAN to IDL
; RSM, 3 Mar, 1995 - major revisions of control variables (many removed)

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     
; Declare the control variables:
COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

 
COMMON PARAM_C, MAX_CHAN, MAX_EBINS, MAX_PLOT, MAX_TERMS, MAX_CHI_1D, $
          MAX_CHI_2D, MAX_PLOT_TYPE, MAX_PARAM, MAX_DET

; Internal variables
LDET = 0 
ICHAN = 0
REPLY = ''
; min and max (over plot chan range) counting rates by detectors:
CNT_LOW = FLTARR(MAX_DET)
CNT_HIGH = FLTARR(MAX_DET)
ALL_CNT_LOW = 0.0 
ALL_CNT_HIGH = 0.0      ; min and max for all detectors
TEMP1 = 0.0
TEMP2 = 0.0
FLAG = 0
NET_RATE = 0.0

; If user has already made count rate axes choices, remind them of their
;  former choices:

FOR LDET=0, NUM_DET - 1 DO BEGIN
    IF (HAVE_YAXIS_LIMITS (LDET, 1)) THEN BEGIN
       PRINT, ' '
       PRINT, ' '
       PRINT, 'CURRENT count rate axis limits for detector ', $
               STRTRIM(DET_NAMES(LDET), 2), ':'
       PRINT, '     ', STRTRIM(YBEG(LDET, 1),2), ' to ', $
              STRTRIM(YEND(LDET, 1), 2)
    ENDIF
ENDFOR


; First determine count rate range of data selected for plot,
;  both by detector and for all detectors:

; Log axis:
;  Minimum is min of 1) the count rate if detection, or 2) 1 sigma, if
;  non-detection.   Detection = count rate > 1 sigma.
;  Maximum is the max of count rate + 1 sigma.
; Linear axis:
;  Minimum is min of count rate - 1 sigma.
;  Maximum is the max of count rate + 1 sigma.

; Have a default for the case of no useable data:

FOR LDET=0, NUM_DET-1 DO BEGIN

    ; any useable data?
    IF (LAST_CHAN_PLOT (LDET) GE FIRST_CHAN_PLOT (LDET)) THEN BEGIN
       CNT_LOW (LDET) = 1.E+37
       CNT_HIGH (LDET) = 1.E-37
         
       FOR ICHAN=FIRST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET), $
           LAST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET) DO BEGIN

           ; does this channel have data?
           IF (LIVE_TIME (ICHAN, LDET) NE 0.0) THEN BEGIN
              NET_RATE = OBS_CRATE (ICHAN, LDET) - BACK_CRATE (ICHAN, LDET)

              IF (YLOG) THEN BEGIN
                 IF (NET_RATE GT CNT_RATE_SIG (ICHAN, LDET)) THEN BEGIN
                    CNT_LOW (LDET) = MIN ([CNT_LOW (LDET), NET_RATE])
                 ENDIF ELSE BEGIN
                    CNT_LOW (LDET) = $
                       MIN ([CNT_LOW (LDET), CNT_RATE_SIG (ICHAN, LDET)])
                 ENDELSE
              ENDIF ELSE BEGIN
                 CNT_LOW (LDET) = $
                    MIN ([CNT_LOW (LDET), $
                          NET_RATE - CNT_RATE_SIG (ICHAN, LDET)])
              ENDELSE

              CNT_HIGH (LDET) = $
                 MAX ([CNT_HIGH (LDET), NET_RATE + CNT_RATE_SIG (ICHAN, LDET)])

           ENDIF ; this chan useable?
       ENDFOR ; ichan
    ENDIF ELSE BEGIN ; any useable data?
       CNT_LOW (LDET) = 1.E-9
       CNT_HIGH (LDET) = 1.E+9
    ENDELSE ; any useable data?

ENDFOR ; ldet 

ALL_CNT_LOW = 1.E+37
ALL_CNT_HIGH = 1.E-37                     

FOR LDET=0, NUM_DET-1 DO BEGIN

    ; only consider detectors with useable data:
    IF (LAST_CHAN_PLOT (LDET) GE FIRST_CHAN_PLOT (LDET)) THEN BEGIN
       ALL_CNT_LOW = MIN ([ALL_CNT_LOW, CNT_LOW (LDET)])
       ALL_CNT_HIGH = MAX ([ALL_CNT_HIGH, CNT_HIGH (LDET)])
    ENDIF
     
ENDFOR


; Determine the Y-axis limits for count-rate (PLOT_TYPE = 1):

; First we branch on control variable CV_AXES_SAME: are the axes required
;  to have identical limits for each detector?
;  If there is only 1 detector, then both cases (CV_AXES_SAME) are
;  identical--we will take axes independent branch so that messages
;  will make more sense.

; Determine the Y-axis limits, required by control variable
;  CV_AXES_SAME to be identical for all the detectors:

IF (CV_AXES_SAME AND NUM_DET GT 1) THEN BEGIN

    ; Y-axis limits, autoranging:

    TEMP1 = M_ROUND_BEG (ALL_CNT_LOW, YLOG, 'C')
    TEMP2 = M_ROUND_END (ALL_CNT_HIGH, YLOG, 'C')

    FOR LDET=0, NUM_DET - 1 DO BEGIN
        YBEG (LDET, 1) = TEMP1
        YEND (LDET, 1) = TEMP2
    ENDFOR

ENDIF ELSE BEGIN ; cv_axes_same ?

    ; Determine the count-rate Y-axis limits, allowed by control variable
    ;  CV_AXES_SAME to be independently determined for the various detectors:
    ; Y-axis limits, autoranging:

    FOR LDET=0, NUM_DET - 1 DO BEGIN
        YBEG (LDET, 1) = M_ROUND_BEG (CNT_LOW (LDET), YLOG, 'C')
        YEND (LDET, 1) = M_ROUND_END (CNT_HIGH (LDET), YLOG, 'C')
    ENDFOR

ENDELSE ; cv_axes_same ?

; Allow manual entries
IF KEYWORD_SET(ADJUST) THEN BEGIN
 
   PRINT, ' '
   PRINT, 'The count rate range spanned by the selected channels ', $ 
          'of all detectors:'
   PRINT, '     ', STRTRIM(ALL_CNT_LOW, 2), $
          ' to ', STRTRIM(ALL_CNT_HIGH, 2), ' counts/s-keV'

   TEMP1 = M_ROUND_BEG(ALL_CNT_LOW, YLOG, 'C')
   TEMP2 = M_ROUND_END(ALL_CNT_HIGH, YLOG, 'C')

S_AGAIN:

   REPLY = ''
   PRINT, ' '
   PRINT, 'Enter <CR> to keep plot count-rate range of'
   PRINT, '     ', STRTRIM(TEMP1, 2), ' to ', STRTRIM(TEMP2, 2)
   READ, 'or enter new values: ', REPLY
   IF (REPLY NE '') THEN BEGIN
      SPACE = STRPOS(REPLY, ' ') 
      SPACE = SPACE(0)
      IF (SPACE EQ -1) THEN BEGIN ; Get second value
         TEMP1 = FLOAT(REPLY)
         READ,'Enter upper value: ', TEMP2
      ENDIF ELSE BEGIN
         TEMP1 = FLOAT(STRMID(REPLY, 0, SPACE))
         TEMP2 = FLOAT(STRMID(REPLY, SPACE+1, 999))
      ENDELSE
      IF (TEMP1 GE TEMP2 OR (YLOG AND TEMP1 LE 0.0)) THEN BEGIN
         PRINT, 'Invalid entry.'
         GOTO, S_AGAIN
      ENDIF

   ENDIF
   FOR LDET=0, NUM_DET - 1 DO BEGIN
       YBEG (LDET, 1) = TEMP1
       YEND (LDET, 1) = TEMP2
   ENDFOR

ENDIF ; adjust


; If the user has managed to choose count-rate Y-axis limits that
;  will cause some of the data to not be seen, give a warning message:

FOR LDET=0, NUM_DET - 1 DO BEGIN
    FLAG = 0
    FOR ICHAN=FIRST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET), $
        LAST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET) DO BEGIN

        ; does channel have data?
        IF (LIVE_TIME (ICHAN, LDET) NE 0.0) THEN BEGIN
           NET_RATE = OBS_CRATE (ICHAN, LDET) - BACK_CRATE (ICHAN, LDET)
           IF (YLOG) THEN BEGIN ; tests for log axis:

              ; 2-sigma upper limit off bottom scale?
              IF (NET_RATE LE CNT_RATE_SIG (ICHAN, LDET) AND $
                  2.0 * CNT_RATE_SIG (ICHAN, LDET) LT YBEG (LDET, 1)) THEN BEGIN
                  FLAG = 1
              ENDIF
              ; detection off bottom scale?
              IF (NET_RATE GT CNT_RATE_SIG (ICHAN, LDET) AND $
                 NET_RATE LT YBEG (LDET, 1)) THEN FLAG = 1
              ; detection off top scale?
              IF (NET_RATE GT CNT_RATE_SIG (ICHAN, LDET) AND $
                 NET_RATE GT YEND (LDET, 1)) THEN FLAG = 1

           ENDIF ELSE BEGIN ; tests for linear axis:

              ; point off bottom axis?
              IF (NET_RATE - CNT_RATE_SIG (ICHAN, LDET) LT $
                 YBEG (LDET, 1)) THEN FLAG = 1
              ; point off top axis?
              IF (NET_RATE + CNT_RATE_SIG (ICHAN, LDET) GT $
                 YEND (LDET, 1)) THEN FLAG = 1
           ENDELSE
        ENDIF
    
    ENDFOR

    IF (FLAG EQ 1) THEN BEGIN 
       PRINT, ' '
       PRINT, '** M_SELECT_CNTS Warning'
       PRINT, 'Detector: ', STRTRIM(DET_NAMES(LDET), 2)
       PRINT, 'Some points will not appear on the graph due to choice' 
       PRINT, 'of limits for count-rate Y-axis.' 
    ENDIF
ENDFOR ; ldet


; Record which axis have been determined via checking which are different
;  from the defaults 1.E-9 to 1.E+9:
FOR LDET=0, NUM_DET - 1 DO BEGIN
    IF (YBEG (LDET, 1) NE 1.E-9 OR YEND (LDET, 1) NE 1.E+9) THEN BEGIN
       HAVE_YAXIS_LIMITS (LDET, 1) = LONG(1)
    ENDIF ELSE BEGIN
       HAVE_YAXIS_LIMITS (LDET, 1) = LONG(0)
    ENDELSE
ENDFOR



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + 
PRO M_SELECT_ENERGY, NUM_CHAN, NUM_DET, $
        CHAN_OFFSET, CHAN_ENERGY, CHAN_WIDTH, $
        OBS_CRATE, BACK_CRATE, LIVE_TIME, $
        DET_NAMES, CNT_RATE_SIG, USED_REBIN, REBIN_CHAN, $
        FIRST_CHAN_PLOT, LAST_CHAN_PLOT, XBEG, XEND, HAVE_XAXIS_LIMITS, $
        CDET, ADJUST = ADJUST, SILENT = SILENT

; This subroutine determines the channel range to plot, FIRST_CHAN_PLOT
;  to LAST_CHAN_PLOT, and the limits of the energy or X-axis, XBEG to
;  XEND keV.
;  Michael S. Briggs, UAH / MSFC ES-64, 17 November 1992.
;
; RSM, Nov, 1994 - converted from FORTRAN to IDL
; RSM, 3 Mar, 1995 - major revisions of control variables (many removed)

; Declare the control variables:
COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

 
COMMON PARAM_C, MAX_CHAN, MAX_EBINS, MAX_PLOT, MAX_TERMS, MAX_CHI_1D, $
          MAX_CHI_2D, MAX_PLOT_TYPE, MAX_PARAM, MAX_DET


; Internal variables:
LDET = 0 
ICHAN = 0
REPLY = ''
FIRST_GOOD_CHAN = LONARR(MAX_DET)    ; lowest chan with data
LAST_GOOD_CHAN = LONARR(MAX_DET)     ; highest chan with data
LOW_E = FLTARR(MAX_DET)         ; by det, low E of FIRST_GOOD_CHAN
HIGH_E = FLTARR(MAX_DET)        ; by det., high E of LAST_GOOD_CHAN
ALL_LOW_E = 0.0            ; lowest E of FIRST_GOOD_CHAN of any det
ALL_HIGH_E = 0.0           ; highest E of LAST_GOOD_CHAN of any det
TEMP1 = 0.0 
TEMP2 = 0.0
CIND = 0 
CIND2 = 0
WANT_ECHO = ''
LEFT_PLOT_E = 0.0 
RIGHT_PLOT_E = 0.0
WANT_E_LOW = 0.0 
WANT_E_HIGH = 0.0
E1 = 0.0 
E2 = 0.0
E_FLAG = 0
PREV_FIRST_CHAN = 0L 
PREV_LAST_CHAN  = 0L
DLEN = 0
UN_FIRST = 0
UN_LAST  = 0

; Determine available channel range, i.e. channel range with data. 
;  Data to be fit has positive LIVE_TIME.
;  Data to plot but to exclude from fitting has its LIVE_TIME multiplied
;  by -1.
;  Here we determine the plot channel range by excluding leading and
;  trailing channels with LIVE_TIME = 0.
;  Also eliminate leading channels for which the observed and background
;  count rates are non-positive: these are almost certainly below the LLD.

FOR LDET=0, NUM_DET - 1 DO BEGIN

    FIRST_GOOD_CHAN (LDET) = 0L
    WHILE (LIVE_TIME (FIRST_GOOD_CHAN (LDET), LDET) EQ 0.0 AND $
          FIRST_GOOD_CHAN (LDET) LT NUM_CHAN (LDET) - 1) DO BEGIN
          FIRST_GOOD_CHAN (LDET) = FIRST_GOOD_CHAN (LDET) + 1
    ENDWHILE

    WHILE (OBS_CRATE (FIRST_GOOD_CHAN (LDET), LDET) LE 0.0 AND $
          BACK_CRATE (FIRST_GOOD_CHAN (LDET), LDET) LE 0.0 AND $
          FIRST_GOOD_CHAN (LDET) LT NUM_CHAN (LDET) - 1) DO BEGIN
          FIRST_GOOD_CHAN (LDET) = FIRST_GOOD_CHAN (LDET) + 1
    ENDWHILE

    LAST_GOOD_CHAN (LDET) = NUM_CHAN (LDET) - 1L
    WHILE (LIVE_TIME (LAST_GOOD_CHAN (LDET), LDET) EQ 0.0 AND $
          LAST_GOOD_CHAN (LDET) GT 0) DO BEGIN
          LAST_GOOD_CHAN (LDET) = LAST_GOOD_CHAN (LDET) - 1L
    ENDWHILE

    IF (LAST_GOOD_CHAN (LDET) LT FIRST_GOOD_CHAN (LDET)) THEN BEGIN
       PRINT, ' '
       PRINT, '** M_SELECT_ENERGY Warning'
       PRINT, 'No data for detector ', STRTRIM(DET_NAMES(LDET), 2)
    ENDIF

    ; shift to user numbering of channels:
    FIRST_GOOD_CHAN (LDET) = FIRST_GOOD_CHAN (LDET) + CHAN_OFFSET (LDET)
    LAST_GOOD_CHAN (LDET) =  LAST_GOOD_CHAN (LDET) + CHAN_OFFSET (LDET)
        
ENDFOR ; ldet

; Determine the plot channel range for each detector:

WANT_ECHO = 'N'
;PRINT, ' '
;READ, 'Echo data?  (Y / N = <CR>): ', WANT_ECHO

FOR LDET=0, NUM_DET - 1 DO BEGIN

    ; do we have useable data?
    IF (LAST_GOOD_CHAN (LDET) GE FIRST_GOOD_CHAN (LDET)) THEN BEGIN

       ; Prompt user for choice
       ;  as a subset of the available channel range (if avail chan range
       ;  is not the empty set):
       IF (STRUPCASE(WANT_ECHO) EQ 'Y') THEN BEGIN
          PRINT, ' '
          PRINT, 'Bins available for detector ', $
                 STRTRIM(DET_NAMES(LDET), 2), ':', $
                 '    Bin number      Energy Range (keV)    Counts/s-keV   Sigma'

          FOR ICHAN=0, NUM_CHAN (LDET)- 1 DO BEGIN
              E1 = CHAN_ENERGY (ICHAN, LDET) - $
                   0.5 * CHAN_WIDTH (ICHAN, LDET)
              E2 = CHAN_ENERGY (ICHAN, LDET) +  $
                   0.5 * CHAN_WIDTH (ICHAN, LDET)
              PRINT, ' '
              PRINT, CHAN_OFFSET (LDET) + ICHAN, '          ', $
                     STRTRIM(E1, 2), '  ', STRTRIM(E2, 2), $
                     OBS_CRATE (ICHAN, LDET) - $
                     BACK_CRATE (ICHAN, LDET), $
                     CNT_RATE_SIG (ICHAN, LDET)
          ENDFOR
       ENDIF
    

       FOR PLOT_TYPE=0, MAX_PLOT_TYPE-1 DO BEGIN
       IF (HAVE_XAXIS_LIMITS(CDET, PLOT_TYPE)) THEN BEGIN
          ; if already have plot chan range then modify if needed to
          ;  to legal values:
          FIRST_CHAN_PLOT (LDET) = $
               MAX ([FIRST_CHAN_PLOT (LDET), FIRST_GOOD_CHAN (LDET)])
          LAST_CHAN_PLOT (LDET) = $
               MIN ([LAST_CHAN_PLOT (LDET), LAST_GOOD_CHAN (LDET)])
 
          ; if above combination of old values and legal range results in
          ;  null range, then use legal range:
          IF (LAST_CHAN_PLOT (LDET) LT FIRST_CHAN_PLOT (LDET)) $
             THEN BEGIN
             FIRST_CHAN_PLOT (LDET) = FIRST_GOOD_CHAN (LDET)
             LAST_CHAN_PLOT (LDET) = LAST_GOOD_CHAN (LDET)
          ENDIF
       ENDIF ELSE BEGIN
          ; no choice has yet been made, so start with legal range:
          FIRST_CHAN_PLOT (LDET) = FIRST_GOOD_CHAN (LDET)
          LAST_CHAN_PLOT (LDET) = LAST_GOOD_CHAN (LDET)
       ENDELSE
       ENDFOR
; branch here to reask question
RETRY:

    IF NOT KEYWORD_SET(SILENT) THEN BEGIN      ; We want silence on first pass
       PREV_FIRST_CHAN = FIRST_CHAN_PLOT (LDET)
       PREV_LAST_CHAN = LAST_CHAN_PLOT (LDET)

       CIND = FIRST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET)
       LEFT_PLOT_E = CHAN_ENERGY (CIND, LDET) - $
                     0.5 * CHAN_WIDTH (CIND, LDET)
       CIND = LAST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET)
       RIGHT_PLOT_E = CHAN_ENERGY (CIND, LDET) + $
                      0.5 * CHAN_WIDTH (CIND, LDET)

       DLEN = STRTRIM(DET_NAMES (LDET), 2)

       PRINT, ' '
       PRINT, ' '
       PRINT, 'CURRENT Detector: '
       PRINT, '     ', STRTRIM(DET_NAMES(LDET), 2)
       PRINT, 'VALID plot channel range: '
       PRINT, '     ', STRTRIM(FIRST_GOOD_CHAN(LDET), 2), ' to ', $
              STRTRIM(LAST_GOOD_CHAN(LDET), 2)
       PRINT, 'CURRENT plot channel range: '
       PRINT, '     ', STRTRIM(FIRST_CHAN_PLOT(LDET), 2), ' to ', $
              STRTRIM(LAST_CHAN_PLOT(LDET), 2), ' = ', $
              STRTRIM(LEFT_PLOT_E, 2), ' to ', $
              STRTRIM(RIGHT_PLOT_E, 2), ' keV.'

       REPLY = ''
       PRINT, ' '
       PRINT,'INPUT <CR> for no changes,'
       PRINT,'     channel range (no decimal point), or'
       READ,'     energy range (with decimal point): ', REPLY
       IF (REPLY NE '') THEN BEGIN
          SPACE = STRPOS(REPLY, ' ') 
          SPACE = SPACE(0)
          ENTRY1 = REPLY
          IF (SPACE EQ -1) THEN BEGIN ; Get second value
             ENTRY2 = ''
             READ,'Enter upper value: ', ENTRY2
          ENDIF ELSE BEGIN
             ENTRY1 = STRMID(REPLY, 0, SPACE)
             ENTRY2 = STRMID(REPLY, SPACE+1, 999)
          ENDELSE
          IF (FLOAT(ENTRY2) LE FLOAT(ENTRY1)) THEN BEGIN
             PRINT, 'Invalid entry.'
             GOTO, RETRY
          ENDIF
       ENDIF

       ; has user entered input in order to make a change?
       IF (REPLY NE '') THEN BEGIN

          ; search for a "." in the string:
          E_FLAG = 0
          DOT1 = STRPOS(ENTRY1, '.')
          DOT2 = STRPOS(ENTRY2, '.')
          IF (DOT1 NE -1) OR (DOT2 NE -1) THEN E_FLAG = 1

          IF (E_FLAG) THEN BEGIN

             ; if string includes a ".", input is an energy range;
             ;  convert energy range to channel range that is within
             ;  the energy range:
             WANT_E_LOW  = FLOAT(ENTRY1)
             WANT_E_HIGH = FLOAT(ENTRY2)

             FIRST_CHAN_PLOT (LDET) = FIRST_GOOD_CHAN (LDET)
             LAST_CHAN_PLOT (LDET) = LAST_GOOD_CHAN (LDET)

             CIND = FIRST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET)
             LEFT_PLOT_E = CHAN_ENERGY (CIND, LDET) - $
                           0.5 * CHAN_WIDTH (CIND, LDET)
             CIND = LAST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET)
             RIGHT_PLOT_E = CHAN_ENERGY (CIND, LDET) + $
                            0.5 * CHAN_WIDTH (CIND, LDET)

             WHILE (LEFT_PLOT_E LT WANT_E_LOW AND $
                   FIRST_CHAN_PLOT (LDET) LT LAST_GOOD_CHAN (LDET)) $
                   DO BEGIN
                   FIRST_CHAN_PLOT (LDET) = FIRST_CHAN_PLOT (LDET) + 1
                   CIND = FIRST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET)
                   LEFT_PLOT_E = CHAN_ENERGY (CIND, LDET) - $
                                 0.5 * CHAN_WIDTH (CIND, LDET)
             ENDWHILE

             WHILE (RIGHT_PLOT_E GT WANT_E_HIGH AND $
                   LAST_CHAN_PLOT (LDET) GT FIRST_CHAN_PLOT (LDET)) $
                   DO BEGIN
                   LAST_CHAN_PLOT (LDET) = LAST_CHAN_PLOT (LDET) - 1
                   CIND = LAST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET)
                   RIGHT_PLOT_E = CHAN_ENERGY (CIND, LDET) + $
                                  0.5 * CHAN_WIDTH (CIND, LDET)
             ENDWHILE
                        
          ENDIF ELSE BEGIN

             ; if string does not contain a ".", input is a channel range:
             FIRST_CHAN_PLOT (LDET) = FIX( ENTRY1 ) 
             LAST_CHAN_PLOT (LDET)  = FIX( ENTRY2 )
             ; force choice to be legal:
             FIRST_CHAN_PLOT (LDET) = $
                   MAX ([FIRST_CHAN_PLOT (LDET), FIRST_GOOD_CHAN (LDET)])
             LAST_CHAN_PLOT (LDET) = $
                   MIN ([LAST_CHAN_PLOT (LDET), LAST_GOOD_CHAN (LDET)])
          
          ENDELSE

          ; Now, if the user has rebinned the channels into larger
          ;  super channels, force his/her choices to be boundaries of
          ;  super channels:

          UN_FIRST = FIRST_CHAN_PLOT (LDET)
          UN_LAST = LAST_CHAN_PLOT (LDET)
          M_REBIN_ROUND, LDET, CHAN_OFFSET, USED_REBIN, REBIN_CHAN, $
                         FIRST_CHAN_PLOT, LAST_CHAN_PLOT
          IF (FIRST_CHAN_PLOT (LDET) NE UN_FIRST OR $
             LAST_CHAN_PLOT (LDET) NE UN_LAST) THEN $
             PRINT, 'Channels ', STRTRIM(UN_FIRST, 2), ' & ', $
                    STRTRIM(UN_LAST, 2), ' rounded to ', $
                    STRTRIM(FIRST_CHAN_PLOT(LDET), 2), ' & ', $
                    STRTRIM(LAST_CHAN_PLOT(LDET), 2), $
                    ' to match boundries of rebinned channels.'

          ; in any case, require first_chan_plot <= last_chan_plot:
          FIRST_CHAN_PLOT (LDET) = $
                MIN ([FIRST_CHAN_PLOT (LDET), LAST_CHAN_PLOT (LDET)])

          ; if plot channel range was altered, branch to echo choice
          ;  and reask:

          ; RSM, 26 Feb, 1995, Previous code did not realize 
          ;  the user made changes 
          ;  when the requested range was larger than the valid range.
          ;  Therefore, just check RETRY to see if the user made an entry:
;         IF (FIRST_CHAN_PLOT (LDET) NE PREV_FIRST_CHAN OR $
;            LAST_CHAN_PLOT (LDET) NE PREV_LAST_CHAN) THEN $
          IF (REPLY NE '') THEN GOTO, RETRY

       ENDIF ; user made input
       ENDIF ; Silent mode


    ENDIF ELSE BEGIN ; have useable data ?

       ; backwards order to flag missing data:
       FIRST_CHAN_PLOT (LDET) = NUM_CHAN (LDET)
       LAST_CHAN_PLOT (LDET) = 0 

    ENDELSE ; have useable data ?

ENDFOR ; ldet

; Now determine energy range of data selected for plot, both by
;  detector and for all detectors:
;  Have a default for the case of no useable data:

FOR LDET=0, NUM_DET - 1 DO BEGIN               
    IF (LAST_CHAN_PLOT (LDET) GE FIRST_CHAN_PLOT (LDET)) THEN BEGIN
       CIND = FIRST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET)
       LOW_E (LDET) = CHAN_ENERGY (CIND, LDET) - 0.5 * CHAN_WIDTH (CIND, LDET)
       CIND = LAST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET)
       HIGH_E (LDET) = CHAN_ENERGY (CIND, LDET) + 0.5 * CHAN_WIDTH (CIND, LDET)
    ENDIF ELSE BEGIN
       LOW_E (LDET ) = 1.
       HIGH_E (LDET) = 10000.
    ENDELSE
ENDFOR

ALL_LOW_E = 10000.
ALL_HIGH_E = 1.

FOR LDET=0, NUM_DET - 1 DO BEGIN
    ; only consider detectors with useable data:
    IF (LAST_CHAN_PLOT (LDET) GE FIRST_CHAN_PLOT (LDET)) THEN BEGIN
       ALL_LOW_E = MIN ([ALL_LOW_E, LOW_E (LDET)])
       ALL_HIGH_E = MAX ([ALL_HIGH_E, HIGH_E (LDET)])
    ENDIF
ENDFOR

; Determine the energy-axis limits:
     
; First we branch on control variable CV_AXES_SAME: are the axes required
;  to have identical limits for each detector?      
;  If there is only 1 detector, then both cases (CV_AXES_SAME) are
;  identical--we will take axes independent branch so that messages
;  will make more sense.

; Determine the energy-axis limits, required by control variable
;  CV_AXES_SAME to be identical for all the detectors:

IF (CV_AXES_SAME AND NUM_DET GT 1) THEN BEGIN

   ; energy-axis limits, autoranging:

   TEMP1 = M_LOG_RND_DOWN (MAX ([ALL_LOW_E, 1.]), 'E')
   TEMP2 = M_LOG_RND_UP (MIN ([ALL_HIGH_E, 1.E+5]), 'E')
   FOR LDET=0, NUM_DET - 1 DO BEGIN
       FOR PLOT_TYPE=0, MAX_PLOT_TYPE-1 DO BEGIN
           XBEG(LDET, PLOT_TYPE) = TEMP1
           XEND(LDET, PLOT_TYPE) = TEMP2
       ENDFOR
   ENDFOR

ENDIF ELSE BEGIN ; cv_axes_same ?

   ; Determine the enegry-axis limits, allowed by control variable
   ;  CV_AXES_SAME to be independently determined for the various
   ;  detectors:

   ; energy-axis limits, autoranging:

   FOR LDET=0, NUM_DET - 1 DO BEGIN
       FOR PLOT_TYPE=0, MAX_PLOT_TYPE-1 DO BEGIN
           XBEG(LDET, PLOT_TYPE) = $
                M_LOG_RND_DOWN (MAX ([LOW_E (LDET), 1.]), 'E')
           XEND(LDET, PLOT_TYPE) = $
                M_LOG_RND_UP (MIN ([HIGH_E (LDET), 1.E+5]), 'E')
       ENDFOR
   ENDFOR

ENDELSE ; cv_axes_same ?


; Now that we have energy-axis limits, warn if they will cause
;  some data not to appear on the graph:

FOR LDET=0, NUM_DET - 1 DO BEGIN

    CIND = FIRST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET)
    CIND2 = LAST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET)
 
    FOR PLOT_TYPE=0, MAX_PLOT_TYPE-1 DO BEGIN
    IF (XBEG(LDET, PLOT_TYPE) GT CHAN_ENERGY (CIND, LDET) OR $
       XEND(LDET, PLOT_TYPE) LT CHAN_ENERGY (CIND2, LDET) ) THEN BEGIN

       PRINT, ' '
       PRINT, '** M_SELECT_ENERGY Warning'
       PRINT, 'Detector: ', STRTRIM(DET_NAMES(LDET), 2)
       PRINT, 'Selected energy range does not contain all the data.'
       PRINT, 'SELECTED energy range:'
       PRINT, '     ', STRTRIM(XBEG(LDET, PLOT_TYPE), 2), ' to ', $
              STRTRIM(XEND(LDET, PLOT_TYPE), 2)
       PRINT, 'DATA energy range:'
       PRINT, '     ', STRTRIM(CHAN_ENERGY(CIND, LDET), 2), ' to ', $
              STRTRIM(CHAN_ENERGY(CIND2, LDET), 2)
    ENDIF
    ENDFOR

ENDFOR

; Mark limits as having been selected:
FOR LDET=0, MAX_DET-1 DO BEGIN
    FOR PLOT_TYPE=0, MAX_PLOT_TYPE-1 DO BEGIN
        HAVE_XAXIS_LIMITS(LDET, PLOT_TYPE) = 1L
    ENDFOR
ENDFOR


END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + 
PRO M_SELECT_NUFNU, NUM_CHAN, NUM_DET, LIVE_TIME, DET_NAMES, $
        CHAN_OFFSET, NU_F_NU_DATA, NU_F_NU_SIG, $
        FIRST_CHAN_PLOT, LAST_CHAN_PLOT, YLOG, YBEG, YEND, HAVE_YAXIS_LIMITS, $
        ADJUST = ADJUST

; This subroutine determines the Y-axis limits for nu-F-nu plots, i.e.
;  YBEG (*, 5) and YEND (*, 5).
;  Adapted from M_SELECT_PHOTS.
;  Michael S. Briggs, UAH / MSFC ES-66, 30 June 1993.
;
; RSM, Nov, 1994 - converted from FORTRAN to IDL
; RSM, 3 Mar, 1995 - major revisions of control variables (many removed)

; Declare the control variables:
COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

; Defines parameters MAX_ & AMAX_
COMMON PARAM_C, MAX_CHAN, MAX_EBINS, MAX_PLOT, MAX_TERMS, MAX_CHI_1D, $
          MAX_CHI_2D, MAX_PLOT_TYPE, MAX_PARAM, MAX_DET

; Internal variables:
LDET = 0 
ICHAN = 0
REPLY = ''
; min and max (over plot chan range) nuFnu rates by detectors:
NUFNU_LOW = FLTARR(MAX_DET)
NUFNU_HIGH = FLTARR(MAX_DET)
; min and max for all detectors
ALL_NUFNU_LOW = 0.0 
ALL_NUFNU_HIGH = 0.0      
TEMP1 = 0.0
TEMP2 = 0.0
FLAG = 0
                              
; If user has already made axes choices, remind them of their
;  former choices:
FOR LDET=0, NUM_DET - 1 DO BEGIN          
    IF (HAVE_YAXIS_LIMITS (LDET, 5)) THEN BEGIN
       PRINT, ' '
       PRINT, 'CURRENT nu-Fnu axis limits for detector ', $
              STRTRIM(DET_NAMES(LDET), 2), ':'
       PRINT, '     ', STRTRIM(YBEG (LDET, 5), 2), ' to ', $
              STRTRIM(YEND (LDET, 5), 2)
    ENDIF
ENDFOR

; First determine nuFnu-rate range of postive data selected for plot,
;  both by detector and for all detectors:

; For log axis:
;  Minimum is min of 1) the nuFnu rate if detection, or 2) 1 sigma, if
;  non-detection.   Detection = nuFnu rate > 1 sigma.
;  Maximum is max of nuFnu rate + 1 sigma.
; For linear axis:
;  Mininum is min of nuFnu rate - 1 sigma.
;  Maximum is max of nuFnu rate + 1 sigma.

; Have a default for the case of no useable data:
FOR LDET=0, NUM_DET - 1 DO BEGIN

    NUFNU_LOW (LDET) = 1.E+37
    NUFNU_HIGH (LDET) = 1.E-37

    FOR ICHAN=FIRST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET),  $
        LAST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET) DO BEGIN

        ; does this channel have data?
        IF (LIVE_TIME (ICHAN, LDET) NE 0.0 AND $
           NU_F_NU_SIG (ICHAN, LDET) GT 0.0) THEN BEGIN

           IF (YLOG) THEN BEGIN
              IF (NU_F_NU_DATA (ICHAN, LDET) GT $
                 NU_F_NU_SIG (ICHAN, LDET)) THEN BEGIN
                 NUFNU_LOW (LDET) = $
                      MIN ([NUFNU_LOW (LDET), NU_F_NU_DATA (ICHAN, LDET)])
              ENDIF ELSE BEGIN
                 NUFNU_LOW (LDET) = $
                      MIN ([NUFNU_LOW (LDET), NU_F_NU_SIG (ICHAN, LDET)])
              ENDELSE
           ENDIF ELSE BEGIN
              NUFNU_LOW (LDET) = MIN ([NUFNU_LOW (LDET), $
                        NU_F_NU_DATA (ICHAN, LDET) - NU_F_NU_SIG (ICHAN,LDET)])
           ENDELSE

           NUFNU_HIGH (LDET) = MAX ([NUFNU_HIGH (LDET), $
                 NU_F_NU_DATA (ICHAN, LDET) + NU_F_NU_SIG (ICHAN, LDET)])

        ENDIF ; this chan useable?
    ENDFOR ; ichan
    
ENDFOR ; ldet 

ALL_NUFNU_LOW = 1.E+37
ALL_NUFNU_HIGH = 1.E-37

FOR LDET=0, NUM_DET - 1 DO BEGIN
    ALL_NUFNU_LOW = MIN ([ALL_NUFNU_LOW, NUFNU_LOW (LDET)])
    ALL_NUFNU_HIGH = MAX ([ALL_NUFNU_HIGH, NUFNU_HIGH (LDET)])
ENDFOR

; Determine the Y-axis limits for nuFnu rate (PLOT_TYPE = 5):

; First we branch on control variable CV_AXES_SAME: are the axes required
;  to have identical limits for each detector?
;  If there is only 1 detector, then both cases (CV_AXES_SAME) are
;  identical--we will take axes independent branch so that messages
;  will make more sense.

; Determine the Y-axis limits, required by control variable
;  CV_AXES_SAME to be identical for all the detectors:

IF (CV_AXES_SAME AND NUM_DET GT 1) THEN BEGIN

   ; Y-axis limits, autoranging:

   TEMP1 = M_ROUND_BEG (ALL_NUFNU_LOW, YLOG, 'N')
   TEMP2 = M_ROUND_END (ALL_NUFNU_HIGH, YLOG, 'N')

   FOR LDET=0, NUM_DET - 1 DO BEGIN
       YBEG (LDET, 5) = TEMP1
       YEND (LDET, 5) = TEMP2
   ENDFOR
      
ENDIF ELSE BEGIN ; cv_axes_same ?
     
   ; Determine the nuFnu rate Y-axis limits
   ; Y-axis limits, autoranging:

   FOR LDET=0, NUM_DET - 1 DO BEGIN
       YBEG (LDET, 5) = M_ROUND_BEG (NUFNU_LOW (LDET), YLOG, 'N')
       YEND (LDET, 5) = M_ROUND_END (NUFNU_HIGH (LDET), YLOG, 'N')
   ENDFOR

ENDELSE ; cv_axes_same ?


; Allow manual entries            
IF KEYWORD_SET(ADJUST) THEN BEGIN
   PRINT, ' '
   PRINT, 'The nuF_nu rate range spanned by the selected channels ', $ 
          'of all detectors:'
   PRINT, '     ', STRTRIM(ALL_NUFNU_LOW, 2), ' to ', $
          STRTRIM(ALL_NUFNU_HIGH, 2), $ 
          ' keV^2/s-cm^2-keV'

S_AGAIN:
            
  TEMP1 = M_ROUND_BEG (ALL_NUFNU_LOW, YLOG, 'N')
  TEMP2 = M_ROUND_END (ALL_NUFNU_HIGH, YLOG, 'N')

  PRINT, ' '
  PRINT, 'Enter <CR> to keep plot nuFnu rate range of'
  PRINT, '     ', STRTRIM(TEMP1, 2), ' to ', STRTRIM(TEMP2, 2)
  READ, 'or enter new values: ', REPLY
  IF (REPLY NE '') THEN BEGIN
     SPACE = STRPOS(REPLY, ' ') 
     SPACE = SPACE(0)
     IF (SPACE EQ -1) THEN BEGIN ; Get second value
        TEMP1 = FLOAT(REPLY)
        READ,'Enter upper value: ', TEMP2
     ENDIF ELSE BEGIN
        TEMP1 = FLOAT(STRMID(REPLY, 0, SPACE))
        TEMP2 = FLOAT(STRMID(REPLY, SPACE+1, 999))
     ENDELSE
     IF (TEMP1 GE TEMP2 OR (YLOG AND TEMP1 LE 0.0)) THEN BEGIN
        PRINT, 'Invalid entry.'
        GOTO, S_AGAIN
     ENDIF
  ENDIF
  FOR LDET=0, NUM_DET - 1 DO BEGIN
      YBEG (LDET, 5) = TEMP1
      YEND (LDET, 5) = TEMP2
  ENDFOR
ENDIF ; adjust

; If the user has managed to choose nuFnu rate Y-axis limits that
;  will cause some of the data to not be seen, give a warning message:
FOR LDET=0, NUM_DET - 1 DO BEGIN
    FLAG = 0
    FOR ICHAN=FIRST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET), $
        LAST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET) DO BEGIN

        ; does this channel have data?
        IF (LIVE_TIME (ICHAN, LDET) NE 0.0 AND $
           NU_F_NU_SIG (ICHAN, LDET) GT 0.0) THEN BEGIN

           IF (YLOG) THEN BEGIN ; tests for log axis:
              ; 2-sigma upper limit off bottom scale:?
              IF (NU_F_NU_DATA (ICHAN, LDET) LE $
                 NU_F_NU_SIG (ICHAN, LDET) AND $
                 2.0 * NU_F_NU_SIG (ICHAN, LDET) LT YBEG (LDET, 5) AND $
                 NU_F_NU_DATA (ICHAN, LDET) GT -1.E+36) THEN BEGIN
                 FLAG = 1
              ENDIF
              ; detection off bottom scale?
              IF (NU_F_NU_DATA (ICHAN, LDET) GT $
                 NU_F_NU_SIG (ICHAN, LDET) AND $
                 NU_F_NU_DATA (ICHAN, LDET) LT YBEG (LDET, 5)) THEN BEGIN
                 FLAG = 1
              ENDIF
              ; detection off top scale?
              IF (NU_F_NU_DATA (ICHAN, LDET) GT $
                 NU_F_NU_SIG (ICHAN, LDET) AND $
                 NU_F_NU_DATA (ICHAN, LDET) GT YEND (LDET, 5)) THEN BEGIN
                 FLAG = 1
              ENDIF
           ENDIF ELSE BEGIN ; tests for linear axis:
              ; point off bottom scale?
              IF (NU_F_NU_DATA (ICHAN, LDET) - NU_F_NU_SIG (ICHAN, LDET) $
                 LT YBEG (LDET, 5)) THEN FLAG = 1
              ; point off top scale?
              IF (NU_F_NU_DATA (ICHAN, LDET) + NU_F_NU_SIG (ICHAN, LDET) $
                 GT YEND (LDET, 5)) THEN FLAG = 1
           ENDELSE
        ENDIF ; have data ?
    ENDFOR ; ichan

    IF (FLAG EQ 1) THEN BEGIN
       PRINT, ' '
       PRINT, '** M_SELECT_NUFNU Warning'
       PRINT, 'Detector: ', STRTRIM(DET_NAMES(LDET), 2)
       PRINT, 'Some points will not appear on the graph due to choice' 
       PRINT, 'of limits for nuFnu rate Y-axis.' 
    ENDIF
ENDFOR ; ldet

; Record which axis have been determined via checking which are different
;  from the defaults 1.E-9 to 1.E+9:
FOR LDET=0, NUM_DET - 1 DO BEGIN
    IF (YBEG (LDET, 5) NE 1.E-9 OR YEND (LDET, 5) NE 1.E+9) THEN BEGIN
       HAVE_YAXIS_LIMITS (LDET, 5) = LONG(1)
    ENDIF ELSE BEGIN
       HAVE_YAXIS_LIMITS (LDET, 5) = LONG(0)
    ENDELSE
ENDFOR



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + 
PRO M_SELECT_PHOTS, NUM_CHAN, NUM_DET, LIVE_TIME, DET_NAMES, $
        CHAN_OFFSET, PHOT_OBS_RATE, PHOT_OBS_SIG, $
        FIRST_CHAN_PLOT, LAST_CHAN_PLOT, YLOG, YBEG, YEND, HAVE_YAXIS_LIMITS, $
        ADJUST = ADJUST

;     This subroutine determines the Y-axis limits for photon plots, i.e.
;     YBEG (*, 2) and YEND (*, 2).
;     Michael S. Briggs, UAH / MSFC ES-64, 17 November 1992.
;
; RSM, Nov, 1994 - converted from FORTRAN to IDL
; RSM, 3 Mar, 1995 - major revisions of control variables (many removed)

; Declare the control variables:
COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

; Defines parameters MAX_ & AMAX_
COMMON PARAM_C, MAX_CHAN, MAX_EBINS, MAX_PLOT, MAX_TERMS, MAX_CHI_1D, $
          MAX_CHI_2D, MAX_PLOT_TYPE, MAX_PARAM, MAX_DET

; Internal variables:
LDET = 0 
ICHAN = 0
REPLY = ''
; min and max (over plot chan range) photon rates by detectors:
PHOT_LOW = FLTARR(MAX_DET)
PHOT_HIGH = FLTARR(MAX_DET)
; min and max for all detectors
ALL_PHOT_LOW = 0.0 
ALL_PHOT_HIGH = 0.0      
TEMP1 = 0.0 
TEMP2 = 0.0
FLAG = 0

; If user has already made photon rate axes choices, remind them of their
;  former choices:
FOR LDET=0, NUM_DET - 1 DO BEGIN
    IF (HAVE_YAXIS_LIMITS (LDET, 2)) THEN BEGIN
        PRINT, ' '
        PRINT, 'CURRENT photon flux axis limits for detector ', $
               STRTRIM(DET_NAMES(LDET), 2), ':'
        PRINT, '     ', STRTRIM(YBEG(LDET, 2), 2), ' to ', $
               STRTRIM(YEND(LDET, 2), 2)
    ENDIF
ENDFOR

; First determine photon-rate range of postive data selected for plot,
;  both by detector and for all detectors:

; Log axis:
;  Minimum is min of 1) the photon rate if detection, or 2) 1 sigma, if
;  non-detection.   Detection = photon rate > 1 sigma.
;  Maximum is max of photon rate + 1 sigma.
; Linear axis:
;  Minimum is min of photon rate - 1 sigma.
;  Maximum is max of photon rate + 1 sigma.

; Have a default for the case of no useable data:
FOR LDET=0, NUM_DET - 1 DO BEGIN

    PHOT_LOW (LDET) = 1.E+37
    PHOT_HIGH (LDET) = 1.E-37

    FOR ICHAN=FIRST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET), $
        LAST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET) DO BEGIN
        ; does this channel have data?
        IF (LIVE_TIME (ICHAN, LDET) NE 0.0 AND $
           PHOT_OBS_SIG (ICHAN, LDET) GT 0.0) THEN BEGIN

           IF (YLOG) THEN BEGIN
              IF (PHOT_OBS_RATE (ICHAN, LDET) GT $
                 PHOT_OBS_SIG (ICHAN, LDET)) THEN BEGIN
                 PHOT_LOW (LDET) = $
                      MIN ([PHOT_LOW (LDET), PHOT_OBS_RATE (ICHAN, LDET)])
              ENDIF ELSE BEGIN 
                 PHOT_LOW (LDET) = $
                      MIN ([PHOT_LOW (LDET), PHOT_OBS_SIG (ICHAN, LDET)])
              ENDELSE
           ENDIF ELSE BEGIN
              PHOT_LOW (LDET) = MIN ([PHOT_LOW (LDET), $
                   PHOT_OBS_RATE (ICHAN, LDET) - PHOT_OBS_SIG (ICHAN, LDET)])
           ENDELSE

           PHOT_HIGH (LDET) = MAX ([PHOT_HIGH (LDET), $
                PHOT_OBS_RATE (ICHAN, LDET) + PHOT_OBS_SIG (ICHAN, LDET)])

        ENDIF ; this chan useable?
    ENDFOR ; ichan

ENDFOR ; ldet 

ALL_PHOT_LOW = 1.E+37
ALL_PHOT_HIGH = 1.E-37

FOR LDET=0, NUM_DET - 1 DO BEGIN
    ALL_PHOT_LOW = MIN ([ALL_PHOT_LOW, PHOT_LOW (LDET)])
    ALL_PHOT_HIGH = MAX ([ALL_PHOT_HIGH, PHOT_HIGH (LDET)])
ENDFOR

; Determine the Y-axis limits for photon-rate (PLOT_TYPE = 2):

; First we branch on control variable CV_AXES_SAME: are the axes required
;  to have identical limits for each detector?
;  If there is only 1 detector, then both cases (CV_AXES_SAME) are
;  identical--we will take axes independent branch so that messages
;  will make more sense.

; Determine the Y-axis limits, required by control variable
;  CV_AXES_SAME to be identical for all the detectors:

IF (CV_AXES_SAME AND NUM_DET GT 1) THEN BEGIN

   ; Y-axis limits, autoranging:

   TEMP1 = M_ROUND_BEG (ALL_PHOT_LOW, YLOG, 'P')
   TEMP2 = M_ROUND_END (ALL_PHOT_HIGH, YLOG, 'P')
   FOR LDET=0, NUM_DET - 1 DO BEGIN
       YBEG (LDET, 2) = TEMP1
       YEND (LDET, 2) = TEMP2
   ENDFOR
                    
ENDIF ELSE BEGIN ; cv_axes_same ?
     
   ; Y-axis limits, autoranging:
   FOR LDET=0, NUM_DET - 1 DO BEGIN
       YBEG (LDET, 2) = M_ROUND_BEG (PHOT_LOW (LDET), YLOG, 'P')
       YEND (LDET, 2) = M_ROUND_END (PHOT_HIGH (LDET), YLOG, 'P')
   ENDFOR

ENDELSE ; cv_axes_same ?

; Allow manual entries            
IF KEYWORD_SET(ADJUST) THEN BEGIN
                           
   PRINT, ' '
   PRINT, 'The photon rate range spanned by the selected channels ', $
          'of all detectors:'
   PRINT, '     ', STRTRIM(ALL_PHOT_LOW, 2), ' to ', $
          STRTRIM(ALL_PHOT_HIGH, 2), $
          ' photons/s-keV-cm^2'

S_AGAIN:
            
   TEMP1 = M_ROUND_BEG (ALL_PHOT_LOW, YLOG, 'P')
   TEMP2 = M_ROUND_END (ALL_PHOT_HIGH, YLOG, 'P')

   PRINT, ' '
   PRINT, 'Enter <CR> to keep plot photon rate range of'
   PRINT, '     ', STRTRIM(TEMP1, 2), ' to ', STRTRIM(TEMP2, 2)
   READ,  'or enter new values: ', REPLY
   IF (REPLY NE '') THEN BEGIN
      SPACE = STRPOS(REPLY, ' ') 
      SPACE = SPACE(0)
      IF (SPACE EQ -1) THEN BEGIN ; Get second value
         TEMP1 = FLOAT(REPLY)
         READ,'Enter upper value: ', TEMP2
      ENDIF ELSE BEGIN
         TEMP1 = FLOAT(STRMID(REPLY, 0, SPACE))
         TEMP2 = FLOAT(STRMID(REPLY, SPACE+1, 999))
      ENDELSE
      IF (TEMP1 GE TEMP2 OR (YLOG AND TEMP1 LE 0.0)) THEN BEGIN
         PRINT, 'Invalid entry.'
         GOTO, S_AGAIN
      ENDIF
   ENDIF
   FOR LDET=0, NUM_DET - 1 DO BEGIN
       YBEG (LDET, 2) = TEMP1
       YEND (LDET, 2) = TEMP2
   ENDFOR

ENDIF ; adjust

; If the user has managed to choose photon-rate Y-axis limits that
;  will cause some of the data to not be seen, give a warning message:
FOR LDET=0, NUM_DET - 1 DO BEGIN
    FLAG = 0
    FOR ICHAN=FIRST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET), $
        LAST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET) DO BEGIN

        ; does this channel have data?
        IF (LIVE_TIME (ICHAN, LDET) NE 0.0 AND $
           PHOT_OBS_SIG (ICHAN, LDET) GT 0.0) THEN BEGIN
           IF (YLOG) THEN BEGIN ; tests for log axis:
              ; 2-sigma upper limit off bottom scale?
              IF (PHOT_OBS_RATE (ICHAN, LDET) LE $
                 PHOT_OBS_SIG (ICHAN, LDET) AND $
                 2.0 * PHOT_OBS_SIG (ICHAN, LDET) LT YBEG (LDET, 2)) THEN $
                 FLAG = 1
              ; detection off bottom scale?
              IF (PHOT_OBS_RATE (ICHAN, LDET) GT $
                 PHOT_OBS_SIG (ICHAN, LDET) AND $
                 PHOT_OBS_RATE (ICHAN, LDET) LT YBEG (LDET, 2)) THEN $
                 FLAG = 1
              ; detection off top scale?
              IF (PHOT_OBS_RATE (ICHAN, LDET) GT $
                 PHOT_OBS_SIG (ICHAN, LDET) AND $
                 PHOT_OBS_RATE (ICHAN, LDET) GT YEND (LDET, 2)) THEN $
                 FLAG = 1
           ENDIF ELSE BEGIN ; tests for linear axis:
              ; point off bottom scale?
              IF (PHOT_OBS_RATE (ICHAN, LDET) - PHOT_OBS_SIG (ICHAN, LDET) $
                 LT YBEG (LDET, 2)) THEN FLAG = 1
              ; point off top scale?
              IF (PHOT_OBS_RATE (ICHAN, LDET) + PHOT_OBS_SIG (ICHAN, LDET) $
                 GT YEND (LDET, 2)) THEN FLAG = 1
           ENDELSE
        ENDIF ; have data ?
    ENDFOR ; ichan

    IF (FLAG EQ 1) THEN BEGIN
       PRINT, ' '
       PRINT, '** M_SELECT_PHOTS Warning'
       PRINT, 'Detector: ', STRTRIM(DET_NAMES(LDET), 2)
       PRINT, 'Some points will not appear on the graph due to choice' 
       PRINT, 'of limits for photon rate Y-axis.' 
    ENDIF

ENDFOR ; ldet

; Record which axis have been determined via checking which are different
;  from the defaults 1.E-9 to 1.E+9:
FOR LDET=0, NUM_DET - 1 DO BEGIN
    IF (YBEG (LDET, 2) NE 1.E-9 OR YEND (LDET, 2) NE 1.E+9) THEN BEGIN
       HAVE_YAXIS_LIMITS (LDET, 2) = LONG(1)
    ENDIF ELSE BEGIN
       HAVE_YAXIS_LIMITS (LDET, 2) = LONG(0)
    ENDELSE
ENDFOR



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + 
PRO M_SELECT_RESID, NUM_DET, DET_NAMES, $
          RESIDUALS, SIGMAS, LIVE_TIME, CHAN_OFFSET, $
          FIRST_CHAN_PLOT, LAST_CHAN_PLOT, $
          PLOT_TYPE, YBEG, YEND, HAVE_YAXIS_LIMITS, ADJUST = ADJUST

;     This subroutine enables the user to choose the y-axis limits for
;     residuals plots.
;     Michael S. Briggs, UAH / ES-64, 18 December 1992.
;
; RSM, Nov, 1994 - converted from FORTRAN to IDL
; RSM, 3 Mar, 1995 - major revisions of control variables (many removed)

; Declare the control variables:
COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

; Defines parameters MAX_ & AMAX_
COMMON PARAM_C, MAX_CHAN, MAX_EBINS, MAX_PLOT, MAX_TERMS, MAX_CHI_1D, $
          MAX_CHI_2D, MAX_PLOT_TYPE, MAX_PARAM, MAX_DET

; Internal Variables:
DATA_MIN = FLTARR(MAX_DET)
DATA_MAX = FLTARR(MAX_DET)
SIG_MIN = FLTARR(MAX_DET)
SIG_MAX = FLTARR(MAX_DET)
ALL_DATA_MIN = 0.0 
ALL_DATA_MAX = 0.0
ALL_SIG_MIN = 0.0
ALL_SIG_MAX = 0.0
LDET = 0 
ICHAN = 0
TEMP1 = 0.0 
TEMP2 = 0.0
SSTRING = ''
REPLY = ''
TEMP_MAX = 0.0

; Figure out the range of the residuals, both for each detector and
;  for all detectors:
FOR LDET=0, NUM_DET - 1 DO BEGIN
    DATA_MIN (LDET) =  9.9E+36
    DATA_MAX (LDET) = -9.9E+36
    SIG_MIN (LDET) =  9.9E+36
    SIG_MAX (LDET) = -9.9E+36
ENDFOR                                    

FOR LDET=0, NUM_DET - 1 DO BEGIN
    FOR ICHAN=FIRST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET), $
        LAST_CHAN_PLOT (LDET) - CHAN_OFFSET (LDET) DO BEGIN

        ; MSB 23 March 93: added next IF statement:
        IF (LIVE_TIME (ICHAN, LDET) NE 0.0) THEN BEGIN
           DATA_MIN (LDET) = MIN([DATA_MIN (LDET), RESIDUALS (ICHAN, LDET)])
           DATA_MAX (LDET) = MAX([DATA_MAX (LDET), RESIDUALS (ICHAN, LDET)])
           SIG_MIN (LDET) = MIN([SIG_MIN (LDET), $
                            RESIDUALS (ICHAN, LDET) - SIGMAS (ICHAN, LDET)])
           SIG_MAX (LDET) = MAX([SIG_MAX (LDET), $
                            RESIDUALS (ICHAN, LDET) + SIGMAS (ICHAN, LDET)])
        ENDIF
    ENDFOR
ENDFOR

ALL_DATA_MIN = DATA_MIN (0) 
ALL_DATA_MAX = DATA_MAX (0)
ALL_SIG_MIN =  SIG_MIN (0)
ALL_SIG_MAX = SIG_MAX (0)
      
FOR LDET=0, NUM_DET - 1 DO BEGIN
    ALL_DATA_MIN = MIN ([ALL_DATA_MIN, DATA_MIN (LDET)])
    ALL_DATA_MAX = MAX ([ALL_DATA_MAX, DATA_MAX (LDET)])
    ALL_SIG_MIN = MIN ([ALL_SIG_MIN, SIG_MIN (LDET)])
    ALL_SIG_MAX = MAX ([ALL_SIG_MAX, SIG_MAX (LDET)])
ENDFOR

; Get user input for residual axes limits, asking for each detector or
;  for values to apply to all detectors, depending on CV_AXES_SAME.
;  If there is data for only 1 detector, then we ignore CV_AXES_SAME:


; same axes limits for all detectors:
IF (CV_AXES_SAME AND NUM_DET GT 1) THEN BEGIN

   TEMP_MAX = MAX ([ABS (ALL_SIG_MIN), ABS (ALL_SIG_MAX)])
   TEMP1 = -M_LOG_RND_UP (TEMP_MAX, 'R')
   TEMP2 = M_LOG_RND_UP (TEMP_MAX, 'R')

   FOR LDET=0, NUM_DET - 1 DO BEGIN
       YBEG (LDET, PLOT_TYPE) = TEMP1
       YEND (LDET, PLOT_TYPE) = TEMP2
   ENDFOR

ENDIF ELSE BEGIN ; cv_axes_same ?

   ; each detector has its own axis limits:
     
   FOR LDET=0, NUM_DET - 1 DO BEGIN
       TEMP_MAX = MAX ([ABS (SIG_MIN (LDET)), ABS (SIG_MAX (LDET))])
       TEMP1 = -M_LOG_RND_UP (TEMP_MAX, 'R')
       TEMP2 = M_LOG_RND_UP (TEMP_MAX, 'R')

       YBEG (LDET, PLOT_TYPE) = TEMP1
       YEND (LDET, PLOT_TYPE) = TEMP2
   ENDFOR

ENDELSE ; cv_axes_same ?

; Allow manual entries            
IF KEYWORD_SET(ADJUST) THEN BEGIN

   FOR LDET=0, NUM_DET - 1 DO BEGIN

       PRINT, ' '
       PRINT, 'Range of the data for detector ', $
              STRTRIM(DET_NAMES(LDET), 2), ':'
       PRINT, '     ', STRTRIM(DATA_MIN(LDET), 2), ' to ', $
              STRTRIM(DATA_MAX(LDET), 2)
       PRINT, 'Range of the data +/- sigmas:'
       PRINT, '     ', STRTRIM(SIG_MIN(LDET), 2), ' to ', $
              STRTRIM(SIG_MAX(LDET), 2)

       TEMP_MAX = MAX ([ABS (SIG_MIN (LDET)), ABS (SIG_MAX (LDET))])
       TEMP1 = -M_LOG_RND_UP (TEMP_MAX, 'R')
       TEMP2 = M_LOG_RND_UP (TEMP_MAX, 'R')

S_AGAIN:
       REPLY = ''
       PRINT, ' '
       PRINT, 'Input Y-axis limits for residuals plots for detector ', $
              STRTRIM(DET_NAMES(LDET), 2), ' or <CR> to keep'
       READ, '     ', STRTRIM(TEMP1, 2), $
             ' to ', STRTRIM(TEMP2, 2), ': ', REPLY
       IF (REPLY NE '') THEN BEGIN
          SPACE = STRPOS(REPLY, ' ') 
          SPACE = SPACE(0)
          IF (SPACE EQ -1) THEN BEGIN ; Get second value
             TEMP1 = FLOAT(REPLY)
             READ,'Enter upper value: ', TEMP2
          ENDIF ELSE BEGIN
             TEMP1 = FLOAT(STRMID(REPLY, 0, SPACE))
             TEMP2 = FLOAT(STRMID(REPLY, SPACE+1, 999))
         ENDELSE
         IF (TEMP2 LE TEMP1) THEN BEGIN
            PRINT, 'Invalid entry.'
            GOTO, S_AGAIN
         ENDIF
       ENDIF
       FOR LDET=0, NUM_DET - 1 DO BEGIN
           YBEG (LDET, PLOT_TYPE) = TEMP1
           YEND (LDET, PLOT_TYPE) = TEMP2
       ENDFOR
   ENDFOR ; LDET
ENDIF

; Mark axes limits as selected:
FOR LDET=0, NUM_DET - 1 DO BEGIN
    HAVE_YAXIS_LIMITS (LDET, PLOT_TYPE) = LONG(1)
ENDFOR



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
