; WINGSPAN WORK WIDGET AND EVENT PROCESSING PROCEDURES
; Used under the main widget to manipulate BURST HISTORY/SPECTRUM data
; R.S. Mallozzi, UAH/NASA, Dec, 1994
 
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
; This is the event handler 
PRO WORK_BASE_EVENT, EVENT

@wgsp_cb.pro
; Following COMMON block internal to this widget routine
;  and its event handler
COMMON TRACK_TYPE, DTYPE


TYPE = TAG_NAMES(Event, /STRUCTURE)
CASE TYPE OF

 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ; The button widget events
 'WIDGET_BUTTON': BEGIN
    WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE, GET_UVALUE = UVALUE
                   
    VALUE = STRUPCASE(VALUE)
    CASE 1 OF 
         VALUE EQ 'EXIT': BEGIN
               WIDGET_CONTROL, WIDS.WORK_BASE, /DESTROY
               WIDS.WORK_BASE = 0L
               END
         VALUE EQ 'TOGGLE WORK WINDOW': BEGIN
               CASE WGSP_WORK OF
                    'HISTORY': BEGIN
                       WGSP_WORK = 'SPECTRUM'
                       ; Change the data type
                       DTYPE = 'SPECTRUM' 
                       WIDGET_CONTROL, WIDS.SELECT_BKGD, SENSITIVE = 0
                       WW_UPDATE, 'SPECTRUM'
                       END
                    'SPECTRUM': BEGIN
                       WGSP_WORK = 'HISTORY'          
                       ; Change the data type
                       DTYPE = 'HISTORY'
                       WIDGET_CONTROL, WIDS.SELECT_BKGD, SENSITIVE = 1
                       WW_UPDATE, 'HISTORY'
                       END
               ENDCASE
               RETURN
               END
         VALUE EQ 'DEFAULT LOOKUPS': BEGIN           
               WORK_INFORM, 'Restoring default lookups.'
               END
         VALUE EQ 'FULL RESOLUTION': BEGIN          
               WORK_INFORM, 'Restoring full resolution.'
               END
         VALUE EQ 'SELECT SOURCE INTERVALS': BEGIN                 
               WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 0
               WORK_INFORM, 'Select source intervals.  ' + $
                  'Click in left margin to EXIT; right margin to RESET.'
               END
         VALUE EQ 'SELECT BACKGROUND INTERVALS': BEGIN                 
               WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 0
               WORK_INFORM, 'Select background intervals.  ' + $
                  'Click in left margin to EXIT; right margin to RESET.'
               END
         VALUE EQ 'COMBINE INTO 1 BIN': BEGIN                 
               WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 0
               WORK_INFORM, 'Select intervals to combine.  ' + $
                   'Click in left margin to EXIT; right margin to RESET.'
               END
         VALUE EQ 'COMBINE SELECTIONS': BEGIN                 
               END
         VALUE EQ 'GROUP BY TWOS': BEGIN                 
               WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 0
               WORK_INFORM, 'Select intervals to combine.  ' + $
                   'Click in left margin to EXIT; right margin to RESET.'
               END

         ; Pull-down menu for GROUP BY...
         ; 20 options are grouped under ELSE option for CASE statement

         VALUE EQ 'HALF ENERGY RESOLUTION': BEGIN                 
               WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 0
               WORK_INFORM, 'Select intervals for HALF ENERGY RESOLUTION.  ' + $
                   'Click in left margin to EXIT; right margin to RESET.'
               END
         VALUE EQ 'FULL ENERGY RESOLUTION': BEGIN                 
               WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 0
               WORK_INFORM, 'Select intervals for FULL ENERGY RESOLUTION.  ' + $
                   'Click in left margin to EXIT; right margin to RESET.'
               END
         VALUE EQ 'BIN FOR SIGNIFICANCE': BEGIN      
               WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 0
               END
         VALUE EQ 'REFINE': BEGIN                   
               WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 0
               WORK_INFORM, 'Select bin to REFINE.  ' + $
                   'Click in left margin to EXIT; right margin to RESET.'
               END
         VALUE EQ 'REFINE BY HALF': BEGIN          
               WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 0
               WORK_INFORM, 'Select bin to REFINE.  ' + $
                   'Click in left margin to EXIT; right margin to RESET.'
               END
         VALUE EQ 'CREATE HARDCOPY': BEGIN                     
               WORK_INFORM, 'Creating hardcopy: Burst ' + $
                            STRLOWCASE(DTYPE) + '.'
               END
         VALUE EQ 'ZOOM IN: SELECTION': BEGIN        
               END
         VALUE EQ 'ZOOM': BEGIN                 
               WORK_INFORM, 'Click in left margin to EXIT, ' + $
                            'right margin to RESET, lower margin for NUMERICAL.'
               END
         VALUE EQ 'ZOOM OUT: FULL RANGE': BEGIN
               WORK_INFORM, 'Restoring full range.'
               END
         VALUE EQ 'ZOOM OUT: BY TWOS': BEGIN
               END
         VALUE EQ 'FULL SCREEN': BEGIN
               WIDGET_CONTROL, WIDS.WORK_BASE, MAP = 0
               IF (DTYPE EQ 'HISTORY')  THEN MFIT_PLOT_OPTION = 14
               IF (DTYPE EQ 'SPECTRUM') THEN MFIT_PLOT_OPTION = 15
               MFIT_OPTION = 'PLOT' 
               EXPAND_PLOT, MFIT_OPTION, MFIT_PLOT_OPTION
               WIDGET_CONTROL, WIDS.WORK_BASE, MAP = 1
               END

         ; If no matches are found for CASE, then VALUE must be one of the
         ;  pull-down menu options for COMBINE, GROUP BY...
         ELSE: BEGIN
               WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE=0
               FOR i=1, 20 DO BEGIN
                   IF STRTRIM(VALUE, 2) EQ STRTRIM(i, 2) THEN BEGIN
                      WORK_INFORM, 'Select intervals to combine.  ' + $
                      'Click in left margin to EXIT; right margin to RESET.'
                      COMBINE_OPTION = i
                   ENDIF
               ENDFOR
               VALUE = 'GROUP BY...'
         END

    ENDCASE ; for VALUE

    BUTTON_COMMAND = STRUPCASE(VALUE(0))
    IF (BUTTON_COMMAND NE 'FULL SCREEN') THEN $
       WW_CONSTRUCT_LOOKUP, DTYPE, BUTTON_COMMAND, COMBINE_OPTION 

    IF (BUTTON_COMMAND NE 'EXIT') THEN BEGIN
       WORK_INFORM, 'Ready.' 
       WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE=1
    ENDIF ELSE BEGIN
       WM_UPDATE
    ENDELSE
    END


ENDCASE ; for TYPE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
PRO W_WORK, TYPE
; TYPE = 'HISTORY' or 'SPECTRUM'

@wgsp_cb.pro
; Following COMMON block internal to this widget routine
;  and its event handler
COMMON TRACK_TYPE, DTYPE


DTYPE = STRUPCASE(TYPE)

DEVICE, RETAIN = CV_IDL_RETAIN

IF XREGISTERED('W_WORK') THEN BEGIN
 ; Because of COMMON BLOCKS, only one instance of the WINGSPAN widget
 ; is allowed
 INFORM, 'W_WORK already running.'
 RETURN
ENDIF

; Parent
WTITLE = 'BURST HISTORY/SPECTRUM'
WIDS.WORK_BASE = WIDGET_BASE(TITLE = WTITLE, /COLUMN)
            
 ; Status  
 WIDS.MW_STATUS = WIDGET_TEXT(WIDS.WORK_BASE, VALUE = ' ', YSIZE=1)
 WIDS.MW_DATA = WIDGET_TEXT(WIDS.WORK_BASE, VALUE = ' ', YSIZE=1)

 BASE1 = WIDGET_BASE(WIDS.WORK_BASE, /ROW)

  ; Selection buttons
  SUB_BASE = WIDGET_BASE(BASE1, /COLUMN, /FRAME)

  BUTTON = WIDGET_BUTTON(SUB_BASE, VALUE = 'EXIT')
  BUTTON = WIDGET_BUTTON(SUB_BASE, VALUE = 'Toggle Work Window')
  BUTTON = WIDGET_BUTTON(SUB_BASE, VALUE = 'Default Lookups')
  BUTTON = WIDGET_BUTTON(SUB_BASE, VALUE = 'Full Resolution')
  BUTTON = WIDGET_BUTTON(SUB_BASE, VALUE = 'Select Source Intervals')
  BUTTON = WIDGET_BUTTON(SUB_BASE, VALUE = 'Select Background Intervals')
  ; Need this button ID so that background selection can be turned off 
  ;  in the burst spectrum window
  WIDS.SELECT_BKGD = BUTTON
  
  ; Pull-down menu for COMBINE
  BUTTON = WIDGET_BUTTON(SUB_BASE, VALUE = 'Combine', MENU = 1)
  BUTTON_LABELS = ['Combine Into 1 Bin', 'Combine Selections', $
                    'Group By Twos', 'Group By...']
  FOR I=0, N_ELEMENTS(BUTTON_LABELS) - 2 DO BEGIN
      PULLDOWN = WIDGET_BUTTON(BUTTON, VALUE = BUTTON_LABELS(I))
  ENDFOR
  ; Pull-down menu for GROUP BY...
  BUTTON = WIDGET_BUTTON(BUTTON, VALUE = BUTTON_LABELS(I), MENU = 1)
  BIN_ARRAY = INDGEN(20) + 1
  GBUTTON_LABELS = [STRING(BIN_ARRAY)]
  FOR J=0, N_ELEMENTS(BIN_ARRAY) - 1 DO BEGIN
      GBUT = WIDGET_BUTTON(BUTTON, VALUE = GBUTTON_LABELS(J))
  ENDFOR

  IF (TYPE EQ 'SPECTRUM') THEN BEGIN
     BUTTON_LABELS = [BUTTON_LABELS, $
                      'Half Energy Resolution', 'Full Energy Resolution'] 
     PULLDOWN = WIDGET_BUTTON(BUTTON, VALUE = BUTTON_LABELS(I+1))
     PULLDOWN = WIDGET_BUTTON(BUTTON, VALUE = BUTTON_LABELS(I+2))
  ENDIF

  BUTTON = WIDGET_BUTTON(SUB_BASE, VALUE = 'Bin For Significance')
  BUTTON = WIDGET_BUTTON(SUB_BASE, VALUE = 'Refine', MENU = 1)

  BUTTON_LABELS = ['Refine', 'Refine By Half']
  FOR I=0, N_ELEMENTS(BUTTON_LABELS) - 1 DO BEGIN
      PULLDOWN = WIDGET_BUTTON(BUTTON, VALUE = BUTTON_LABELS(I))
  ENDFOR
  

  BUTTON = WIDGET_BUTTON(SUB_BASE, VALUE = 'Create Hardcopy')

  BUTTON = WIDGET_BUTTON(SUB_BASE, VALUE = 'Zoom', MENU = 1)
  BUTTON_LABELS = ['Zoom', $
                   'Zoom In: Selection', $
                   'Zoom Out: Full Range', $
                   'Zoom Out: By Twos', $
                   'Full Screen']
  FOR I=0, N_ELEMENTS(BUTTON_LABELS) - 1 DO BEGIN
      PULLDOWN = WIDGET_BUTTON(BUTTON, VALUE = BUTTON_LABELS(I))
  ENDFOR
 
  ; The draw widget
  WIDS.WORK_DRAW = WIDGET_DRAW(BASE1, XSIZE=CURRENT_SCREEN(0)*0.65*WGSP_SCALE, $
                               YSIZE=CURRENT_SCREEN(1)*0.50, /FRAME)
 
; Realize the BASE widget and the children
WIDGET_CONTROL, WIDS.WORK_BASE, /REALIZE, SHOW = 1 

; Plot the data
WIDGET_CONTROL, GET_VALUE = WORK_WINDOW, WIDS.WORK_DRAW
WSET, WORK_WINDOW
; Don't allow background interval selection for burst spectrum window
IF (STRUPCASE(TYPE) EQ 'SPECTRUM') THEN BEGIN
   WIDGET_CONTROL, WIDS.SELECT_BKGD, SENSITIVE = 0
ENDIF
WW_UPDATE, TYPE

BREAK_FILE, CURRENT_VALUES(6), FDEV, FDIR, FNAME, FEXT, FVER
WIDGET_CONTROL, WIDS.MW_DATA, SET_VALUE = 'CURRENT DATA FILE:  ' + FNAME + FEXT
WORK_INFORM, 'Ready.'

; Register with XMANAGER
XMANAGER, 'WORK_BASE', WIDS.WORK_BASE, /MODAL



END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 


; BEGIN W_WORK_PROCS 

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO WW_UPDATE, TYPE, XRANGE = XRANGE, YRANGE = YRANGE

; Updates the Burst History or Spectrum in work window
; XRANGE, YRANGE are optional keywords that are passed on to 
;  PRO STD_DISPLAY to set the axis ranges explicitly, else they
;  are set by the data.
; TYPE = 'HISTORY' or 'SPECTRUM'

@wgsp_cb.pro


XTSAVE = !X.TITLE
YTSAVE = !Y.TITLE

ENERGY_SCALE = READ_MENU(1)
RATE_SCALE   = READ_MENU(2)

IF STRUPCASE(TYPE) EQ 'SPECTRUM' THEN BEGIN
   !X.TITLE     = READ_MENU(4)
   !Y.TITLE     = READ_MENU(5)
   TITLE        = CREATE_LABEL(COMBINED_TIMES, TIME_SPAN, ' s')

   STD_DISPLAY, SPECTRUM, SPECTRUM_ERR, COMBINED_ENERGIES, $
                ENERGY_RANGE, RATE_RANGE, ENERGY_SPAN, TITLE, $
                ENERGY_SCALE, RATE_SCALE, 2, 'SPECTRUM'
ENDIF

IF STRUPCASE(TYPE) EQ 'HISTORY' THEN BEGIN
   !X.TITLE = READ_MENU(3)
   !Y.TITLE = READ_MENU(5)
   IF (CV_RATE_MODE EQ 'R') THEN BEGIN       
      !Y.TITLE = STRMID(!Y.TITLE, 0, STRPOS(!Y.TITLE, ' keV'))
      ENDIF
   TITLE    = CREATE_LABEL(COMBINED_ENERGIES, ENERGY_SPAN, ' keV')

   STD_DISPLAY, HISTORY, HISTORY_ERR, COMBINED_TIMES, $
                TIME_RANGE, HIST_RANGE, TIME_SPAN, TITLE, $
;                'LINEAR', 'LINEAR', 1, 'HISTORY' 
                READ_MENU(9), READ_MENU(10), 1, 'HISTORY' 
ENDIF


!X.TITLE  = XTSAVE
!Y.TITLE  = YTSAVE 


              
END 
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; Shell to call CONSTRUCT_LOOKUP 
PRO WW_CONSTRUCT_LOOKUP, DTYPE, BUTTON_COMMAND, COMBINE_OPTION

@wgsp_cb.pro


WSAVE = !D.WINDOW
RATE_MODE_SAVE = CV_RATE_MODE  ; for now (TBD: pass in RATE_MODE_SAVE)

IF (DTYPE EQ 'HISTORY') THEN BEGIN
;   ENERGY_SCALE = READ_MENU(1)
;   RATE_SCALE   = 'LINEAR'
   ENERGY_SCALE = READ_MENU(9)
   RATE_SCALE   = READ_MENU(10)
   !X.TITLE = READ_MENU(3)
   !Y.TITLE = READ_MENU(5)

   ; Construct a time bin lookup table.
   !X.TITLE = READ_MENU(3)
   !Y.TITLE = READ_MENU(5)

   TITLE = CREATE_LABEL(COMBINED_TIMES, TIME_SPAN, ' s')

   ; User has changed rate mode!
   IF CV_RATE_MODE NE RATE_MODE_SAVE THEN BEGIN 
      INTEGRATE_HISTORY
      COMBINE, SPECTRA_E, SPECTRA_E_ERR, TIMES, TIME_LOOKUP, $
               HISTORY, HISTORY_ERR, COMBINED_TIMES
      INTEGRATE_BACKGROUND
      ; Bin up the background first to current lookup
      BIN_BACK               
      HIST_RANGE = [MIN(HISTORY), MAX(HISTORY)]   ; Range is invalid
      RATE_MODE_SAVE = CV_RATE_MODE          ; Make sure to reset it
   ENDIF

   !P.PSYM = 0
   CHANGED = 0
   PLOT_LABEL = CREATE_LABEL(COMBINED_ENERGIES, ENERGY_SPAN, ' keV')
   CONSTRUCT_LOOKUP, HISTORY, HISTORY_ERR, COMBINED_TIMES, $
                     SPECTRA_E, SPECTRA_E_ERR, TIMES, $
                     TIME_LOOKUP, DEFAULT_TL, TIME_RANGE, $
                     HIST_RANGE, TIME_SPAN, 'TIME BINS', $
;                     'LINEAR', RATE_SCALE, CHANGED, 1, $
                     READ_MENU(9), READ_MENU(10), CHANGED, 1, $
                     BUTTON_COMMAND, DTYPE, COMBINE_OPTION

   IF CHANGED THEN BEGIN  ;Integration region changed
      MESSAGE = 'Integrating Burst Spectrum over new time selection.'
      WORK_INFORM, MESSAGE
      INTEGRATE_SPECTRUM
      COMBINE, SPECTRA_T, SPECTRA_T_ERR, THRESHOLDS, $
               ENERGY_LOOKUP, SPECTRUM, $
               SPECTRUM_ERR, COMBINED_ENERGIES
      RATE_RANGE = [MIN(SPECTRUM - SPECTRUM_ERR), $
                    MAX(SPECTRUM + SPECTRUM_ERR)]
      RATE_SCALE = READ_MENU(2)
      !X.TITLE = READ_MENU(4)
      !Y.TITLE = READ_MENU(5)
      WSET, SPECTRUM_WINDOW
      STD_DISPLAY, SPECTRUM, SPECTRUM_ERR, COMBINED_ENERGIES, $
                   ENERGY_RANGE, RATE_RANGE, ENERGY_SPAN, TITLE, $
                   ENERGY_SCALE, RATE_SCALE, 2, 'SPECTRUM'
      IF ENERGY_SCALE EQ 'LINEAR' THEN DISPLAY_RAINBOW, THRESHOLDS
      IF HAVE_BACK THEN INTEGRATE_BACKGROUND
   ENDIF

ENDIF

IF (DTYPE EQ 'SPECTRUM') THEN BEGIN
   ENERGY_SCALE = READ_MENU(1)
   RATE_SCALE   = READ_MENU(2)
   !X.TITLE = READ_MENU(4)
   !Y.TITLE = READ_MENU(5)
   !P.PSYM = 0
   CHANGED = 0
   PLOT_LABEL = CREATE_LABEL(COMBINED_TIMES, TIME_SPAN, ' s')
   CONSTRUCT_LOOKUP, SPECTRUM, SPECTRUM_ERR, COMBINED_ENERGIES, $
                     SPECTRA_T, SPECTRA_T_ERR, THRESHOLDS, $
                     ENERGY_LOOKUP, DEFAULT_EL, ENERGY_RANGE, $
                     RATE_RANGE, ENERGY_SPAN, 'ENERGY BINS', $
                     ENERGY_SCALE, RATE_SCALE, CHANGED, 2, $
                     BUTTON_COMMAND, DTYPE, COMBINE_OPTION

   ENERGY_SCALE = READ_MENU(1)
   RATE_SCALE   = READ_MENU(2)
   IF ENERGY_SCALE EQ 'LINEAR' THEN DISPLAY_RAINBOW, THRESHOLDS

   TITLE = CREATE_LABEL(COMBINED_ENERGIES, ENERGY_SPAN, ' keV')

   ; User changed rate mode!
   IF CV_RATE_MODE NE RATE_MODE_SAVE THEN BEGIN 
      ; Make sure to reset it.
      RATE_MODE_SAVE = CV_RATE_MODE             
      CHANGED = 1
   ENDIF

   ;Integration region changed:
   IF CHANGED THEN BEGIN  
      MESSAGE = 'Integrating Burst History over new energy selection.'
      WORK_INFORM, MESSAGE
      INTEGRATE_HISTORY

      ; RDP: If user changed the energy selection, let MFIT know about it!
      IF N_ELEMENTS(HAVE_XAXIS_LIMITS) NE 0 THEN BEGIN
         HAVE_XAXIS_LIMITS(CDET, *) = LONARR(MAX_PLOT_TYPE)
      ENDIF

      COMBINE, SPECTRA_E, SPECTRA_E_ERR, TIMES, TIME_LOOKUP, $
               HISTORY, HISTORY_ERR, COMBINED_TIMES

      HIST_RANGE = [MIN(HISTORY), MAX(HISTORY)]
      !X.TITLE = READ_MENU(3)
      !Y.TITLE = READ_MENU(5)

      WSET, HISTORY_WINDOW

      IF HAVE_BACK THEN BEGIN
         BACK_CHISQ = FLTARR(MAX_CHAN)
         BACK_FIT, BACK_CHISQ
         INTEGRATE_BACKGROUND
         STD_DISPLAY, HISTORY, HISTORY_ERR, COMBINED_TIMES, $
                      TIME_RANGE, HIST_RANGE, TIME_SPAN, TITLE, $
;                      'LINEAR', 'LINEAR', 1, 'HISTORY' 
                      READ_MENU(9), READ_MENU(10), 1, 'HISTORY' 
      ENDIF $
      ELSE BEGIN
         STD_DISPLAY, HISTORY, HISTORY_ERR, COMBINED_TIMES, $
                      TIME_RANGE, HIST_RANGE, TIME_SPAN, TITLE, $
;                      'LINEAR', 'LINEAR', 0, 'HISTORY'
                      READ_MENU(9), READ_MENU(10), 0, 'HISTORY' 
      ENDELSE
   ENDIF

ENDIF

WSET, WSAVE



END ; PRO WW_CONSTRUCT_LOOKUP
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; END W_WORK_PROCS 
