; SPECTRA : FLTARR(number of file energy bins, number of file time bins)
;    Rates (counts/sec) at the time and energy resolution of the input FITS
;    file.  SPECTRA(I, J) is the rate in the Ith energy bin and the Jth time
;    bin.
; R_SPECTRA : FLTARR(number of user energy bins, number of user time bins)
;    Rates in SPECTRA rebinned according the user specified energy and time
;    lookup tables (ENERGY_LOOKUP and TIME_LOOKUP in COMMON AXES).
; SPECTRA_E : FLTARR(number of file time bins)
;    Rate history, at the file's time resolution, averaged over the range of
;    energy bins specified by ENERGY_SPAN in COMMON AXES.
; SPECTRA_T : FLTARR(number of file energy bins)
;    Rate spectrum, at the file's energy resolution, averaged over the range
;    of time bins specified by TIME_SPAN in COMMON AXES.
; SPECTRUM : FLTARR(number of user energy bins)
;    Rates in SPECTRA_T rebinned according to the lookup table, ENERGY_LOOKUP.
; HISTORY : FLTARR(number of user time bins)
;    Rates in SPECTRA_E rebinned according to the lookup table, TIME_LOOKUP.
; RATE_RANGE : FLTARR(2, 2)
;    Range of rates and energies plotted on the SPECTRUM plot.
; HIST_RANGE : FLTARR(2, 2)
;    Range of rates and times plotted on the HISTORY plot.
; HIST_RANGE_SAVE, RATE_RANGE_SAVE, INIT_HIST_RANGE: FLTARR(2)
;    Used by zoom to remember previous state.
; RATE_MODE_SAVE: CHAR: 'E' or 'R'
;    Used to detect changes made by the user in the rate axis units.
COMMON RATE_C, SPECTRA, R_SPECTRA, SPECTRA_E, SPECTRA_T, SPECTRUM, HISTORY, $
               RATE_RANGE, HIST_RANGE, HIST_RANGE_SAVE, RATE_RANGE_SAVE, $
               INIT_HIST_RANGE, RATE_MODE_SAVE 

; SPECTRA_ERRORS : FLTARR(number of file energy bins, number of file time bins)
;    Two dimensional, FLOAT array of RMS uncertainty in each of the rates in
;    SPECTRA.
COMMON ERROR_C, SPECTRA_ERRORS, R_SPECTRA_ERR, SPECTRA_E_ERR, SPECTRA_T_ERR, $
                SPECTRUM_ERR, HISTORY_ERR

; TIMES : FLTARR(2, number of time bins)
;    Integration interval for each time bin in the file.
; TIME_LOOKUP: Lookup table for the combined time binning.
; COMBINED_TIMES: FLTARR(2, number of time lookup bins).
; TIME_RANGE: Current horizontal range of the BURST HISTORY plot.
; TIME_SPAN: INTARR(number of selected regions + 1, 2) 
;    Selections from the BURST HISTORY plot. First column is total range.
COMMON TIME_C, TIMES, TIME_LOOKUP, COMBINED_TIMES, TIME_RANGE, TIME_SPAN, $
               DEFAULT_TL, TRIGGER, TIME_RANGE_SAVE, INIT_TIME_RANGE

; THRESHOLDS : FLTARR(2, number of energy bins)
;    Calculated edges for each energy bin in the file.
; ENERGY_LOOKUP: Lookup table for the combined energy binning.
; COMBINED_ENERGIES: FLTARR(2, number of energy lookup bins).
; ENERGY_RANGE: Current horizontal range of the BURST SPECTRUM plot.
; ENERGY_SPAN: INTARR(number of selected regions + 1, 2) 
;    Selections from the BURST SPECTRUM plot. First column is total range.
COMMON ENERGY_C, THRESHOLDS, ENERGY_LOOKUP, COMBINED_ENERGIES, ENERGY_RANGE, $
                 ENERGY_SPAN, DEFAULT_EL, ENERGIES, ENERGY_RANGE_SAVE

COMMON MENUS_C, PLOT_FIELDS, PLOT_MENU, PLOT_OPTIONS, PLOT_TYPES, ROTATION

COMMON WINDOWS_C, SPECTRUM_WINDOW, HISTORY_WINDOW, PLOT_WINDOW, $
                  PROCESSED_HISTORY, PROCESSED_SPECTRUM, LAYOUT_WINDOW, $
                  MFIT_WINDOW

; BACK_FIT: FLTARR(NCHAN), background fit model for each channel, summed over
;           the selected times in the data.
; BACK_ERRS: Summed errors for the above model.
; BACK_SPAN: INTARR(N,2), edges of the selected background region.
; B_TOT_TIME: Total livetime in the background selection.
; BACK_TIMES: FLTARR(MANY), array of times for display of continuous background
;             model (in BACK_HISTORY).
; BACK_HISTORY: FLTARR(MANY), continuous background history, summed over energy.
; DIRTY: Flag to indicate a new background was created.
; NDEGREE: Degree of background model.
; SUM_BACK: FLTARR(M), background fit model for the selected time bins, summed
;           over the selected energy bins.
; SUM_ERRS: Summed errors for the above model.
COMMON BACK_C, BACK_FIT, BACK_ERRS, BACK_SPAN, B_TOT_TIME, BACK_TIMES, $
               BACK_HISTORY, DIRTY, NDEGREE, HAVE_BACK, SUM_BACK, SUM_ERRS

;FITS_C: FITS file information:
; HEADER: FITS file primary header
; CH: FITS file calibration header
; DH: FITS file data header
COMMON FITS_C, HEADER, CH, DH

;FITS file notes (NOTES) and MFIT_SUPER display notes (ANNOTES):
COMMON ANNOTATION_C, NOTES, N_NOTES, N_ANNOTES, ANNOTES, ANN_X, ANN_Y

;DRM_C: Variables connected with the Detector Response Matrix (DRM):
; DRM_DIMi: Dimensions of the DRM (the third dimension is always 0 for now).
; E_IN: FLTARR(DRM_DIM1 + 1) Edges of the input (photon) bins.
; CH_OUT: FLTARR(DRM_DIM2 + 1) Edges of the output (channel data) bins.
; DRM: FLTARR(DRM_DIM1, DRM_DIM2, DIR_DIM3) Direct or summed DRM.
; DRM_LABEL: Name of the DRM FITS file.
COMMON DRM_C, CH_OUT, E_IN, DRM, DRM_LABEL, DRM_DIM1, $
              DRM_DIM2, DRM_DIM3

;PHOT_C: Photon observed counts and models calculated by MFIT:
; PHOT_ENERGY: FLTARR(MAX_EBINS) Energy of centers of photon bins.
; PHOT_WIDTH: FLTARR(MAX_EBINS) Widths of photon bins (no longer used by MFIT).
; PHOT_OBS_RATE: FLTARR(MAX_EBINS) Count data converted to photons (MFIT output)
; PHOT_OBS_SIG: FLTARR(MAX_EBINS) Errors in above rates (MFIT output).
; MODEL_PHOT_RATE: FLTARR(MAX_PLOT, MAX_TERM) High resolution photon model,
;                 term-by-term (MFIT output); first term (0) is total model.
COMMON PHOT_C, PHOT_ENERGY, PHOT_WIDTH, PHOT_OBS_RATE, PHOT_OBS_SIG, $
               MODEL_PHOT_RATE, NUM_EBINS, MODEL_CR_VARI, PHOT_MODEL_BYCHAN

;CNT_C: Count and Background rates and errors input to MFIT:
; CHAN_ENERGY: FLTARR(MAX_CHAN) Energy of centers of data channel bins.
; CHAN_WIDTH: FLTARR(MAX_CHAN) Widths of data channel bins.
; OBS_CRATE: FLTARR(MAX_CHAN) Observed (total) count rate spectrum.
; NET_CNT_RATE: FLTARR(MAX_CHAN) Background-subtracted count rate spectrum.
; CNT_RATE_SIG: FLTARR(MAX_CHAN) Data variances for above.
; BACK_CRATE: FLTARR(MAX_CHAN) Background count rate spectrum, averaged.
; BACK_CSIG: FLTARR(MAX_CHAN) Variances for above.
; LIVE_TIME: FLTARR(MAX_CHAN) Vector of livetimes corresponding to to the 
;           observed data; some channels may be masked by setting = 0.
COMMON CNT_C, NUM_CHAN, CHAN_ENERGY, CHAN_WIDTH, CNT_RATE_SIG, NET_CRATE, $
              OBS_CRATE, BACK_CRATE, BACK_CSIG, LIVE_TIME, $
              LOOKUP_FILE, FIT_WIN

;Fixed parameters for declaring MFIT arrays.
COMMON PARAM_C, MAX_CHAN, MAX_EBINS, MAX_PLOT, MAX_TERMS, MAX_CHI_1D, $
                MAX_CHI_2D, MAX_PLOT_TYPE, MAX_PARAM, MAX_DET

;MFIT_SUPER input arguments.
COMMON IN_ARGS_C, TOPLABEL, NUM_DET, DET_NAMES, $
                  NEW_DRM, NEW_DATA, DATA_FILE_NAME, COMMENT_DATA, $
                  CHAN_OFFSET, RES_FRAC_AT511, RES_EXP, $
                  FIRST_CHAN_PLOT, LAST_CHAN_PLOT, CALLING_PROG, CTOE_LABEL, $
                  USED_REBIN, REBIN_CHAN, CDET

;Some of the miscellaneous MFIT output arguments.
COMMON OUT_ARGS_C, COMMENT_FIT, MFIT_VERSION_TAG, TIME_TAG, $
                   FIRST_CHAN_FIT, LAST_CHAN_FIT, USE_DET, $
                   MFIT_SUPER_VER, FIT_ERR, PHOT_MODEL_VER, $
                   FIRST_NONZERO, REL_CHANGE_LIMIT, ABS_CHANGE_LIMIT, $
                   PHOT_EVAL, NUME_TO_USE, DATA_CR_VARI, MODEL_CNT_RATE_WS, $
                   MODEL_PHOT_RATE_WS, MODEL_CR_VARI_WS, $
                   CHAN_EVAL, CHAN_NUME_TO_USE, DIFF_EVAL, DIFF_NUME_TO_USE, $
                   NUM_DIFF, FIRST_PHOT_BIN, LAST_PHOT_BIN

;MODEL_C: Model (in count space) returned from MFIT:
; MODEL_ENERGY: FLTARR(MAX_PLOT) Energies for plotting continuous model (MFIT)
; MODEL_CNT_RATE: FLTARR(MAX_PLOT, MAX_TERMS) Spectral model in count space,
;                term-by-term (MFIT output).
; TERM_USED: INTARR(MAX_TERMS+1) Index to model terms used in the fit(MFIT out).
; NUM_LIST: Number of parameters fitted by MFIT and returned in variables below.
; PARAM_LIST: FLTARR(MAX_PARAM) Parameters of model fit (MFIT output).
; PARAM_UNCER_LIST: FLTARR(MAX_PARAM) 1-sigma uncertanties in above.
; PARAM_LABEL_LIST: STRARR(MAX_PARAM) Names of model fit parameters.
COMMON MODEL_C, MODEL_ENERGY, MODEL_CNT_RATE, $
                FIRST_EXTRAP_PLOT, LAST_EXTRAP_PLOT, TERM_USED, $
                NUM_TERMS_AVAIL, MODEL_CHISQ, NUM_LIST, PARAM_LIST, $
                PARAM_UNCER_LIST, DOF, PARAM_LABEL_LIST, $
                NUM_PARAM_OF_TERM, PARAM, PARAM_UNCER, PARAM_VARY, $
                CHANGE_MODEL, MAX_TRIES, NUM_ADD_TERMS, WRITE_CHI_STEPS, $
                REL_CONVERG, ABS_CONVERG, WM_FLAGS, COVAR, $
                FIRST_BIN_CALC, LAST_BIN_CALC
COMMON MODEL_NAME_C, TERM_NAMES, PARAM_NAMES, PARAM_UNITS

;RESID_C: Various residuals returned by MFIT after a successful fit.
COMMON RESID_C, PHOT_RESID, CNT_RESID, NU_F_NU_DATA, NU_F_NU_MODEL, NU_F_NU_SIG

;CHISQ_C: Returned by MFIT after a chi-sq map is requested:
COMMON CHISQ_C, NUM_CHI, X_CHI_LABEL, Y_CHI_LABEL, X_PARAM_VALS, Y_PARAM_VALS, $
                CHI_1D_VALS, CHI_2D_VALS, X_BEST_VALUE, Y_BEST_VALUE, $
                NUM_VARY, SIG_LEVELS, NINE_LEVELS, CHI_1D_PLOT_CH, $
                CHI_1D_PAR_INT, CHI_2D_PLOT_CH, CHI_2D_PAR_INT

;MFIT_SUPER control variables:
COMMON CONTROL_C, RUN_MODE, PASS_COMMAND, HAVE_MODEL, HAVE_FIT, HAVE_1D_CHI, $
                  HAVE_2D_CHI, HAVE_GAUSS_CHI, HAVE_DRM, HAVE_INITTED

;MODEL_PLOTTER internal variables:
COMMON PLOT_C, XBEG, XEND, PLOT_MODEL, PLOT_TYPE, XAXIS_LABEL, YAXIS_LABEL, $
               YBEG, YEND, XLOG, YLOG, HAVE_XAXIS_LIMITS, HAVE_YAXIS_LIMITS, $
               FIRST_MODEL_PLOT, LAST_MODEL_PLOT, GAUSS_VWIDTH, $
               HAVE_ZOOMED_HIST, HAVE_ZOOMED_PARAM, HAVE_ZOOMED_DATA, $
               XBEG_SAVE, YBEG_SAVE, XEND_SAVE, YEND_SAVE, $
               HAVE_XAXIS_LIMITS_SAVE, HAVE_YAXIS_LIMITS_SAVE

COMMON MODEL_PLOT_C, X_F_DATA, Y_F_DATA, DY_F_DATA, MX_DATA, MY_DATA, $
               MIND, X_F_TITLE, Y_F_TITLE, F_TITLE

;HARD_COPY internal variables:
COMMON PROCESSED_C, X_PH_DATA, Y_PH_DATA, DY_PH_DATA, X_PH_TITLE, Y_PH_TITLE, $
                    PH_TITLE, X_PS_DATA, Y_PS_DATA, X_PS_TITLE, Y_PS_TITLE, $
                    PS_TITLE, DY_PS_DATA, FLAT_INDEX, WGSP_LAST_PLOT

COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

COMMON LAYOUT_C, BOXES, PLOT_TITLES, HARD_MENU, PLOT_LABEL

; Current screen size:
COMMON THE_SCREEN, CURRENT_SCREEN

; Current values of the main menu:
COMMON CURRENT_MENU, CURRENT_VALUES, ANNOTATE_BH, ANNOTATE_BS, $
                     AUTO_READ, JOINT_FIT, ANNOTATE_SIZE, $
                     GLOBAL_CHARSIZE, GLOBAL_TICKLEN, $
                     GLOBAL_LINETHICK, GLOBAL_CHARTHICK, $
                     WGSP_SUPPRESS_NOTES, $
                     WGSP_ENERGY_SCALE, WGSP_RATE_SCALE, $
                     WGSP_HIST_RATE_SCALE

; WIDGET Ids: 
COMMON TWIDS, WIDS

; WINGSPAN IDL source code location and other machine-dependent strings:
COMMON SOURCE_LOC, WGSP_SOURCE, WGSP_PRINTER, WGSP_SCRATCH, $
       WGSP_FITS, WGSP_LOOKUP, WGSP_DRM, WGSP_NO_MFIT, $
       WGSP_SCALE, WGSP_PRINT_COMMAND, WGSP_CONFIRM, WGSP_HELP, $
       WGSP_WORK, MFIT_CONTROL_FILENAME, MFIT_FUNCINFO_FILENAME, $
       WGSP_SEPARATE_BASE, WGSP_BROWSER, $
       WGSP_COLOR_LO, WGSP_COLOR_HI, WGSP_COLOR

COMMON WGSP_GET_RANGE, WLO_X, WHI_X, $
            NUMERICAL_XRANGE, NUMERICAL_YRANGE, OPLOT_TIME_HIST
